/*
 * Copyright 09/13/02 Sun Microsystems, Inc.  All Rights Reserved.
 */
/*
 * log(x)
 * RETURN THE LOGARITHM OF x
 *
 * Method:
 * (1)  For x close to 1, let v = x-1. If  -2**-5 <= v < 2**-4, then
 *      use a 10 degree polynomial to compute log(x)=log(1+v):
 *              log(1+v) = v - 0.5*v^2 + P0*v^3 + P1*v^4 + ... + P7*v^10
 *
 * (2)  Otherwise, for 2**-7 <= x < 2**8, use an look up table to find an x' that
 *      matches x to 6 significant bits and t = log(x') is very close to a double
 *      precision floating point number. Thus
 *              log(x) = log(x') + log(x/x')
 *                     = t + (2z + B1*z^3 + B2*z^5), where z = (x-x')/(x+x').
 * (3)  Otherwise, check if x is inf/nan/subnormal/0/negative.
 * (4)  All remaining cases: write x = 2**n * y, 1<=y<2. Then
 *      log(x) = n*ln2 + log(y). Apply (2) to log(y) and we have
 *      log(x) = n*ln2 + log(y') + t + (2z + B1*z^3 + B2*z^5), where z = (y-y')/(y+y').
 *
 * Note 1: n*ln2 can be table looked up by n. To reduce the size of table, we
 *      break n = n1*2^5+n2 and look up n1*2^5*ln2, n2*ln2 respectively. Then
 *      n*ln2 = n1*2^5*ln2 + n2*ln2.
 *
 * Note 2: y is best obtained by bit manipulation. However, if the hardware is not
 *      designed for bit manipulation for floating point number, then one may
 *      compute y by x * 2**(-n1) * 2**(-n2). Thus we need two tables look up for
 *      2**(-n1) and 2**(-n2).
 *
 * Technical note:
 *
 *   A picture below shows the lengths and sizes of the crucial elements
 *
 *              n1LN2_H: [---.----------]       ... 2^9 --- 2^(-43)
 *              n2LN2_H: [---.----------]       ... 2^9 --- 2^(-43)
 *              log(y')    [-.----------]       ... 2^2 --- 2^(-43)
 *              w=log(y/y')     [------...]     ... less than 2**-8
 *              n1LN2_L:                [--...] ... less than 2**(-43)
 *              n1LN2_L:                [--...] ... less than 2**(-43)
 *
 * Constants :
 * The hexadecimal values are the intended ones for the following
 * constants. The decimal values may be used, provided that the
 * compiler will convert from decimal to binary accurately enough
 * to produce the hexadecimal values shown.
 *
 * Maximum error observed: less than 0.82 ULP after a 16,000,000 random test.
 *
 */
/* Copyright (C) 2011-2014 Free Software Foundation, Inc.
   This file is part of the GNU C Library.
   Contributed by Ulrich Drepper <drepper@gmail.com>, 2011.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <http://www.gnu.org/licenses/>.  */

#include <math.h>
#include <math_private.h>
#include <math-svid-compat.h>

/* xlog[2j,2j+1]={x[j],log(x[j])} for j = 0,1,2,...,1023, where
*  (1) HI(x[j]) is close to 0x3f800000+(j<<14)+0x2000
*  (2) log(x[j]) is very close to a floating point number y such that
*      (i)  y*2^43 is an integer
*      (ii) |(y-log(x[j]))/ulp(y)|< 2**-8
*/

const double _TBL_xlog[] = {
 7.87353503297997907306e-03,-4.84424813913381058228e+00, 7.99560553334410263615e-03,
-4.82886319655938223150e+00, 8.11767585942603335514e-03,-4.81371138998713377077e+00,
 8.23974595318925813470e-03,-4.79878576645967314107e+00, 8.36181633409116380207e-03,
-4.78407961064067421830e+00, 8.48388677464952979668e-03,-4.76958658811963687185e+00,
 8.60595687057736508097e-03,-4.75530065613099850452e+00, 8.72802746017422269309e-03,
-4.74121588424020501407e+00, 8.85009762998075107110e-03,-4.72732678838690389966e+00,
 8.97216781943364037610e-03,-4.71362795770266984619e+00, 9.09423832305908744211e-03,
-4.70011421729645917367e+00, 9.21630872203840420531e-03,-4.68678067710459345108e+00,
 9.33837890625026888214e-03,-4.67362260644677007804e+00, 9.46044933385502273548e-03,
-4.66063539875301557913e+00, 9.58251940124139021948e-03,-4.64781473605819428485e+00,
 9.70458975449937054358e-03,-4.63515633481574695907e+00, 9.82666002507358825324e-03,
-4.62265617620380453445e+00, 9.94873038252497288170e-03,-4.61031033569940973393e+00,
 1.00708009304617783497e-02,-4.59811503912237640179e+00, 1.01928710183715621335e-02,
-4.58606672281769078836e+00, 1.03149412251085266512e-02,-4.57416183047723734489e+00,
 1.04370116746394445140e-02,-4.56239697556293322123e+00, 1.05590818861922668137e-02,
-4.55076894707917745109e+00, 1.06811525009752397719e-02,-4.53927453919584422692e+00,
 1.08032228351010207990e-02,-4.52791077871006564237e+00, 1.09252929317279600524e-02,
-4.51667472538713354879e+00, 1.10473633920256959079e-02,-4.50556348654606608761e+00,
 1.11694334144951216981e-02,-4.49457439104651257367e+00, 1.12915039768306923323e-02,
-4.48370469651013081602e+00, 1.14135740363942583819e-02,-4.47295192696174126468e+00,
 1.15356444057218999327e-02,-4.46231352366135070042e+00, 1.16577147091142512697e-02,
-4.45178711134167315322e+00, 1.17797849620007808596e-02,-4.44137035542439662095e+00,
 1.19018553603602791902e-02,-4.43106097838244750164e+00, 1.20239258565084574520e-02,
-4.42085679284218713292e+00, 1.21459962934535870005e-02,-4.41075568667281459057e+00,
 1.22680664999469876686e-02,-4.40075561214268873300e+00, 1.23901368474765814776e-02,
-4.39085453840800710168e+00, 1.25122069067848239871e-02,-4.38105055864832593215e+00,
 1.26342775160141543300e-02,-4.37134172093919914914e+00, 1.27563475411863816444e-02,
-4.36172628489453018119e+00, 1.28784181364660649943e-02,-4.35220238134013470699e+00,
 1.30004884744073095915e-02,-4.34276834727211280551e+00, 1.31225585059048330955e-02,
-4.33342250640578185994e+00, 1.32446290127844895634e-02,-4.32416316625517538341e+00,
 1.33666993895025371097e-02,-4.31498878524826068315e+00, 1.34887696310743935429e-02,
-4.30589781892092560156e+00, 1.36108399741910190972e-02,-4.29688874679550281144e+00,
 1.37329099894686126881e-02,-4.28796013777946427581e+00, 1.38549806086363126029e-02,
-4.27911050026136763336e+00, 1.39770508932642468364e-02,-4.27033851627084004576e+00,
 1.40991210231746488812e-02,-4.26164282232309687970e+00, 1.42211914062500398986e-02,
-4.25302207427353096136e+00, 1.43432619092453816217e-02,-4.24447500041389957914e+00,
 1.44653322434032498289e-02,-4.23600037203777901595e+00, 1.45874021618763553831e-02,
-4.22759698837558062223e+00, 1.47094726130564887323e-02,-4.21926359728502120561e+00,
 1.48315427866054835165e-02,-4.21099909677957384702e+00, 1.49536134678975949491e-02,
-4.20280230481273520127e+00, 1.50756838106471009742e-02,-4.19467217682404225343e+00,
 1.51977536999667543066e-02,-4.18660764494779868983e+00, 1.53198240228826533527e-02,
-4.17860760149267207453e+00, 1.54418943672747152607e-02,-4.17067104973057212192e+00,
 1.55639648437503070461e-02,-4.16279698268078846013e+00, 1.57470704144479535513e-02,
-4.15110093644352673437e+00, 1.59912106886508087467e-02,-4.13571603963976031082e+00,
 1.62353513267169471956e-02,-4.12056423358035317506e+00, 1.64794918782338734031e-02,
-4.10563858760758648714e+00, 1.67236326626088241398e-02,-4.09093243041400000948e+00,
 1.69677734375002775558e-02,-4.07643941414858090866e+00, 1.72119137911766022231e-02,
-4.06215347266481785482e+00, 1.74560545966061866907e-02,-4.04806872222638958192e+00,
 1.77001950617631000073e-02,-4.03417961902448496403e+00, 1.79443358412112244360e-02,
-4.02048076586652314290e+00, 1.81884767513036046949e-02,-4.00696703095343309542e+00,
 1.84326169059479752499e-02,-3.99363352573902830045e+00, 1.86767581668063481493e-02,
-3.98047540691641188459e+00, 1.89208986174409510284e-02,-3.96748822084987295966e+00,
 1.91650392336622198142e-02,-3.95466753300001983007e+00, 1.94091800192974783812e-02,
-3.94200912796418379003e+00, 1.96533206732878440448e-02,-3.92950896393722359790e+00,
 1.98974606659403675801e-02,-3.91716316012048082484e+00, 2.01416012229629730335e-02,
-3.90496789023620749504e+00, 2.03857420121848140204e-02,-3.89291954346242619067e+00,
 2.06298831597628677670e-02,-3.88101461552321325144e+00, 2.08740230869859308416e-02,
-3.86924980756850800390e+00, 2.11181638758243077347e-02,-3.85762176162108971766e+00,
 2.13623045155612902024e-02,-3.84612738140447163460e+00, 2.16064455079306959318e-02,
-3.83476360566044149891e+00, 2.18505857219565315097e-02,-3.82352755130295918207e+00,
 2.20947268278799645314e-02,-3.81241630400245412602e+00, 2.23388668272341273102e-02,
-3.80142721056517984834e+00, 2.25830078382944395010e-02,-3.79055752105875853886e+00,
 2.28271481328934447430e-02,-3.77980474376875008602e+00, 2.30712890252300177607e-02,
-3.76916633383507360122e+00, 2.33154292991755926479e-02,-3.75863993588791345246e+00,
 2.35595701000469968933e-02,-3.74822316739209782099e+00, 2.38037112266834792462e-02,
-3.73791377656687018316e+00, 2.40478517569114799701e-02,-3.72770961045694093627e+00,
 2.42919925783059674484e-02,-3.71760850646694507304e+00, 2.45361328895565104780e-02,
-3.70760843607968126889e+00, 2.47802735569855082931e-02,-3.69770736341570227523e+00,
 2.50244144629187779960e-02,-3.68790335213975595252e+00, 2.52685548279013259820e-02,
-3.67819454845755444694e+00, 2.55126956393694481751e-02,-3.66857908250244690862e+00,
 2.57568363163034087793e-02,-3.65905519909631493647e+00, 2.60009761664391908509e-02,
-3.64962119680239993613e+00, 2.62451170662701439851e-02,-3.64027532377076568082e+00,
 2.64892578831275316009e-02,-3.63101599107255879062e+00, 2.67333986645064830556e-02,
-3.62184160897129459045e+00, 2.69775394324082498643e-02,-3.61275063204982416210e+00,
 2.72216800632049289732e-02,-3.60374156201748974127e+00, 2.74658199124513981160e-02,
-3.59481295964019409439e+00, 2.77099612573108791680e-02,-3.58596331825651759573e+00,
 2.79541017838256453110e-02,-3.57719133580758352764e+00, 2.81982417530679749240e-02,
-3.56849565216384689847e+00, 2.84423824390846353982e-02,-3.55987490684242402494e+00,
 2.86865236937321743671e-02,-3.55132782420298553916e+00, 2.89306643337068729316e-02,
-3.54285319676978360803e+00, 2.91748046875002879641e-02,-3.53444979534776848595e+00,
 2.94189450893193940595e-02,-3.52611642137492253823e+00, 2.96630863227642545266e-02,
-3.51785189095073747012e+00, 2.99072269202756806095e-02,-3.50965512477171159844e+00,
 3.01513668010818802756e-02,-3.50152502346725214011e+00, 3.03955081340536829859e-02,
-3.49346044023559443303e+00, 3.06396481961666161675e-02,-3.48546041602401146520e+00,
 3.08837886177621805117e-02,-3.47752387295213338803e+00, 3.11279294408172702013e-02,
-3.46964981004566652700e+00, 3.14941400953491212888e-02,-3.45795377917511359556e+00,
 3.19824212175857866547e-02,-3.44256886407367801439e+00, 3.24707037171354032101e-02,
-3.42741702026160055539e+00, 3.29589843749986538546e-02,-3.41249138828095510689e+00,
 3.34472653082870177710e-02,-3.39778525036024348083e+00, 3.39355475189647981527e-02,
-3.38329221461253837333e+00, 3.44238277007514331673e-02,-3.36900628866544593620e+00,
 3.49121087936623414327e-02,-3.35492155311089845782e+00, 3.54003905544377495129e-02,
-3.34103242629203123215e+00, 3.58886723440119265183e-02,-3.32733356687208470248e+00,
 3.63769536366641971226e-02,-3.31381984670827023365e+00, 3.68652337699117566006e-02,
-3.30048634631793902372e+00, 3.73535163205735709635e-02,-3.28732822670554014621e+00,
 3.78417970801284697657e-02,-3.27434104437941186916e+00, 3.83300774244423603232e-02,
-3.26152037964800456393e+00, 3.88183598325214096469e-02,-3.24886195271290034725e+00,
 3.93066401611306681652e-02,-3.23636181353617757850e+00, 3.97949225572831605247e-02,
-3.22401594876760100306e+00, 4.02832031622070083077e-02,-3.21182069189512731100e+00,
 4.07714839887274790864e-02,-3.19977236377667395573e+00, 4.12597662836584333501e-02,
-3.18786743583257248247e+00, 4.17480462697479501322e-02,-3.17610262471441728849e+00,
 4.22363288475464185301e-02,-3.16447455511433872744e+00, 4.27246100288964023028e-02,
-3.15298017749091741280e+00, 4.32128900029361898150e-02,-3.14161644854084443068e+00,
 4.37011724607619875260e-02,-3.13038034747478377540e+00, 4.41894537252461375165e-02,
-3.11926912187004745647e+00, 4.46777336557711707288e-02,-3.10828002997607200086e+00,
 4.51660163634763911489e-02,-3.09741032529075255297e+00, 4.56542964244464333223e-02,
-3.08665755973356681352e+00, 4.61425774989241838098e-02,-3.07601916522799001541e+00,
 4.66308598865257384336e-02,-3.06549272770303105062e+00, 4.71191399897234880489e-02,
-3.05507599129680329497e+00, 4.76074225628829866830e-02,-3.04476659370652669168e+00,
 4.80957035763506304815e-02,-3.03456242859692792990e+00, 4.85839851336431344242e-02,
-3.02446132637976461410e+00, 4.90722648636232258723e-02,-3.01446127417568732199e+00,
 4.95605461481169254356e-02,-3.00456020234412335412e+00, 5.00488287008052229465e-02,
-2.99475617607606636739e+00, 5.05371087623148404488e-02,-2.98504738557744531136e+00,
 5.10253914211659664879e-02,-2.97543189915120365185e+00, 5.15136727048605760904e-02,
-2.96590801713375640247e+00, 5.20019523196564303902e-02,-2.95647401649671337509e+00,
 5.24902343750008118506e-02,-2.94712813859166544717e+00, 5.29785163890683710730e-02,
-2.93786879875665363215e+00, 5.34667976983075401987e-02,-2.92869442150436043448e+00,
 5.39550777050737812024e-02,-2.91960347298447686626e+00, 5.44433596769196628817e-02,
-2.91059438971365125326e+00, 5.49316403751221385421e-02,-2.90166576906381123990e+00,
 5.54199225781972157345e-02,-2.89281613654941338609e+00, 5.59082035495971005346e-02,
-2.88404415557056381658e+00, 5.63964851727864965869e-02,-2.87534844205151784990e+00,
 5.68847650464581605645e-02,-2.86672772332406111673e+00, 5.73730460209391640580e-02,
-2.85818066746128351951e+00, 5.78613273128146324287e-02,-2.84970603962096902251e+00,
 5.83496088691468975385e-02,-2.84130262345718165307e+00, 5.88378912221743100019e-02,
-2.83296922307920340245e+00, 5.93261710722584809097e-02,-2.82470473690977996739e+00,
 5.98144537989585625692e-02,-2.81650794490712996776e+00, 6.03027337198441398813e-02,
-2.80837784095581355359e+00, 6.07910159903868388409e-02,-2.80031326424409598985e+00,
 6.12792960446383247475e-02,-2.79231324113800383202e+00, 6.17675775942550589481e-02,
-2.78437668658443726599e+00, 6.22558586817859790918e-02,-2.77650263269583774672e+00,
 6.29882806879503431841e-02,-2.76480659072080925398e+00, 6.39648429271016533137e-02,
-2.74942167582310048601e+00, 6.49414065636584741936e-02,-2.73426985310777581617e+00,
 6.59179682524966420454e-02,-2.71934421526827918569e+00, 6.68945305961964886698e-02,
-2.70463807010492018890e+00, 6.78710925507265833145e-02,-2.69014507069857700117e+00,
 6.88476557544217687701e-02,-2.67585910297941609315e+00, 6.98242179648248667556e-02,
-2.66177436714451687294e+00, 7.08007802592148793863e-02,-2.64788525773280980502e+00,
 7.17773450123609657592e-02,-2.63418638179348363337e+00, 7.27539074770596483344e-02,
-2.62067266334804571670e+00, 7.37304700386825773162e-02,-2.60733913186618337932e+00,
 7.47070326145762853010e-02,-2.59418104650126224442e+00, 7.56835951353527908170e-02,
-2.58119385093561959366e+00, 7.66601550968642464490e-02,-2.56837319585326895321e+00,
 7.76367173119365122647e-02,-2.55571480246214832732e+00, 7.86132799006758614668e-02,
-2.54321463833900907048e+00, 7.95898443288603207879e-02,-2.53086877807959353959e+00,
 8.05664068845765024562e-02,-2.51867350438237735943e+00, 8.15429679122149364545e-02,
-2.50662518401679790259e+00, 8.25195324914450717424e-02,-2.49472025619206760894e+00,
 8.34960927677172448647e-02,-2.48295544142115431896e+00, 8.44726573789048862384e-02,
-2.47132737829747384239e+00, 8.54492179911065519171e-02,-2.45983302111710600002e+00,
 8.64257815429549103303e-02,-2.44846925019589889416e+00, 8.74023451647745336368e-02,
-2.43723316413172597095e+00, 8.83789075749761704470e-02,-2.42612193990157720691e+00,
 8.93554701858457200991e-02,-2.41513281724905937153e+00, 9.03320326082727442785e-02,
-2.40426314604462731950e+00, 9.13085933766264179479e-02,-2.39351037339395134040e+00,
 9.22851555069695295153e-02,-2.38287197915121851111e+00, 9.32617202563460340192e-02,
-2.37234554196095359657e+00, 9.42382796668281425001e-02,-2.36192881405418120266e+00,
 9.52148453144239931278e-02,-2.35161941116518846684e+00, 9.61914052375711370324e-02,
-2.34141526794655874255e+00, 9.71679702199125944206e-02,-2.33131414630736344407e+00,
 9.81445325985184358064e-02,-2.32131406436019460671e+00, 9.91210926331757002927e-02,
-2.31141301838488288922e+00, 1.00097656545059823796e-01,-2.30160900407327062567e+00,
 1.01074220389409039078e-01,-2.29190017667417578195e+00, 1.02050781977363433373e-01,
-2.28228472706712182116e+00, 1.03027344344888305283e-01,-2.27276084690925017640e+00,
 1.04003907886698998553e-01,-2.26332680471307412517e+00, 1.04980467151555040273e-01,
-2.25398097325785329303e+00, 1.05957032534405817814e-01,-2.24472162049698908959e+00,
 1.06933592473217167762e-01,-2.23554726828285765805e+00, 1.07910154761203122553e-01,
-2.22645629843827919103e+00, 1.08886719971891393621e-01,-2.21744720347760448931e+00,
 1.09863280255896172966e-01,-2.20851859300353225990e+00, 1.10839845272973080981e-01,
-2.19966895493769243330e+00, 1.11816407061484102137e-01,-2.19089697534786864708e+00,
 1.12792967310562708416e-01,-2.18220128839936933218e+00, 1.13769530581689634618e-01,
-2.17358053846794518904e+00, 1.14746092485066938926e-01,-2.16503348303899656457e+00,
 1.15722657192544237104e-01,-2.15655883687941241078e+00, 1.16699217310301903150e-01,
-2.14815544656471502094e+00, 1.17675782683865221001e-01,-2.13982204048386392969e+00,
 1.18652345417424379126e-01,-2.13155752876582482713e+00, 1.19628907509237034090e-01,
-2.12336076508847781952e+00, 1.20605469772116835614e-01,-2.11523064105654157174e+00,
 1.21582032919824833961e-01,-2.10716607596054927853e+00, 1.22558592662597290457e-01,
-2.09916605590012750326e+00, 1.23535154663287849441e-01,-2.09122951027609360608e+00,
 1.24511720444319934353e-01,-2.08335542739325774164e+00, 1.25976560256825270701e-01,
-2.07165941904406736285e+00, 1.27929689867213314258e-01,-2.05627446389428314433e+00,
 1.29882814044999850278e-01,-2.04112266548236220842e+00, 1.31835936067006387340e-01,
-2.02619703803054562741e+00, 1.33789061152869898086e-01,-2.01149088984038826311e+00,
 1.35742185285735839084e-01,-1.99699788878103845491e+00, 1.37695315100288123711e-01,
-1.98271189633692301868e+00, 1.39648434796177417505e-01,-1.96862719470118463505e+00,
 1.41601562500001859624e-01,-1.95473806317886555917e+00, 1.43554688638290045555e-01,
-1.94103921089140385448e+00, 1.45507815309506571833e-01,-1.92752548034570736490e+00,
 1.47460940232259807292e-01,-1.91419195025582666858e+00, 1.49414065182176925006e-01,
-1.90103386625571602053e+00, 1.51367184986530817215e-01,-1.88804670528531914897e+00,
 1.53320310818882421522e-01,-1.87522601121588650130e+00, 1.55273439648820593684e-01,
-1.86256758954027645814e+00, 1.57226560938703269210e-01,-1.85006745054522525606e+00,
 1.59179685472016413383e-01,-1.83772161753290674824e+00, 1.61132814301210208763e-01,
-1.82552632052045282762e+00, 1.63085935690612499060e-01,-1.81347800427738548024e+00,
 1.65039064831409232337e-01,-1.80157307654997111968e+00, 1.66992188880606434731e-01,
-1.78980824082930212171e+00, 1.68945313875418196004e-01,-1.77818020296047052398e+00,
 1.70898440202511159303e-01,-1.76668581586238815362e+00, 1.72851560464758691182e-01,
-1.75532208480012741347e+00, 1.74804686566071676435e-01,-1.74408600510139422113e+00,
 1.76757809613157151896e-01,-1.73297479066582127416e+00, 1.78710940376110738459e-01,
-1.72198563666438531072e+00, 1.80664064967847814991e-01,-1.71111596686125722044e+00,
 1.82617190339444374070e-01,-1.70036317319625140954e+00, 1.84570309593092479750e-01,
-1.68972480628940502356e+00, 1.86523435583688806938e-01,-1.67919838782665920007e+00,
 1.88476560713050050877e-01,-1.66878162617558700731e+00, 1.90429688724863849236e-01,
-1.65847224060360076692e+00, 1.92382814737144552941e-01,-1.64826806523285540607e+00,
 1.94335940353429159622e-01,-1.63816696619198864937e+00, 1.96289064810789698479e-01,
-1.62816688576799606381e+00, 1.98242186237978373908e-01,-1.61826583292372561118e+00,
 2.00195315064046336362e-01,-1.60846181365332085988e+00, 2.02148439584360406496e-01,
-1.59875300202304515551e+00, 2.04101560156369127297e-01,-1.58913756511731207866e+00,
 2.06054689323313500182e-01,-1.57961366327469932003e+00, 2.08007815835887566180e-01,
-1.57017962385270948289e+00, 2.09960934481931038587e-01,-1.56083379184622117464e+00,
 2.11914064483585101728e-01,-1.55157444269866573450e+00, 2.13867185991603508954e-01,
-1.54240008283591123472e+00, 2.15820310764771122747e-01,-1.53330911212185583281e+00,
 2.17773440750831243085e-01,-1.52430001921175062307e+00, 2.19726559856004838300e-01,
-1.51537141542814879358e+00, 2.21679684224500600509e-01,-1.50652180289387160883e+00,
 2.23632814071700908309e-01,-1.49774979501717098174e+00, 2.25585940045735761084e-01,
-1.48905408379266646079e+00, 2.27539059192649967267e-01,-1.48043336656905921700e+00,
 2.29492186395013675337e-01,-1.47188629627021327906e+00, 2.31445315581153332474e-01,
-1.46341165115165949828e+00, 2.33398433952854561513e-01,-1.45500826886575396202e+00,
 2.35351559207891636305e-01,-1.44667488609684369294e+00, 2.37304684031606732963e-01,
-1.43841037687468542572e+00, 2.39257814848893218818e-01,-1.43021358523731123569e+00,
 2.41210938761362625460e-01,-1.42208346374218308483e+00, 2.43164065102798365725e-01,
-1.41401889843086792098e+00, 2.45117184015952616072e-01,-1.40601888068147218291e+00,
 2.47070311077018212043e-01,-1.39808232263135323592e+00, 2.49023436250571217343e-01,
-1.39020826545834097487e+00, 2.51953122291124820276e-01,-1.37851223142934031785e+00,
 2.55859372611992319513e-01,-1.36312731117163821182e+00, 2.59765620800116503464e-01,
-1.34797551298572670930e+00, 2.63671871452013817372e-01,-1.33304986005714454222e+00,
 2.67578122206616364487e-01,-1.31834370965088965022e+00, 2.71484379010469789595e-01,
-1.30385067713643820753e+00, 2.75390620589041512645e-01,-1.28956475067843712168e+00,
 2.79296870819378373341e-01,-1.27548000974798014795e+00, 2.83203121103925814506e-01,
-1.26159089637610577483e+00, 2.87109379074504122986e-01,-1.24789202406930144207e+00,
 2.91015622431010367865e-01,-1.23437832792171775509e+00, 2.94921875506810426959e-01,
-1.22104478650612691126e+00, 2.98828130253993684562e-01,-1.20788668606508053927e+00,
 3.02734379435562417360e-01,-1.19489949346859702928e+00, 3.06640623918145616145e-01,
-1.18207882321928536840e+00, 3.10546873080715102322e-01,-1.16942042899961506919e+00,
 3.14453130011007275790e-01,-1.15692024411941929429e+00, 3.18359380231136190886e-01,
-1.14457440780120123236e+00, 3.22265629396773944926e-01,-1.13237913749560448196e+00,
 3.26171871155974346657e-01,-1.12033082440802900237e+00, 3.30078120683089115683e-01,
-1.10842592319488630892e+00, 3.33984375000005218048e-01,-1.09666106853683231748e+00,
 3.37890630662039781473e-01,-1.08503301378470951022e+00, 3.41796878326266317938e-01,
-1.07353864138428889419e+00, 3.45703130617572740313e-01,-1.06217487621597683756e+00,
 3.49609378681696947311e-01,-1.05093880866786548722e+00, 3.53515618806027276388e-01,
-1.03982761129475420603e+00, 3.57421869258219393473e-01,-1.02883848826252233266e+00,
 3.61328120643693018188e-01,-1.01796881201755695656e+00, 3.65234370364428462263e-01,
-1.00721602087696737726e+00, 3.69140620796920171376e-01,-9.96577621365986487945e-01,
 3.73046880862486718300e-01,-9.86051181277730393049e-01, 3.76953119787533430518e-01,
-9.75634449962512917409e-01, 3.80859378873238063612e-01,-9.65325056306028272957e-01,
 3.84765629848706214311e-01,-9.55120883699805744982e-01, 3.88671880516789192139e-01,
-9.45019786121065408224e-01, 3.92578128672658843712e-01,-9.35019707625201590417e-01,
 3.96484368944619969266e-01,-9.25118661270403208619e-01, 4.00390627570941681768e-01,
-9.15314639480016012385e-01, 4.04296881041378619592e-01,-9.05605816831211996032e-01,
 4.08203130944030101457e-01,-8.95990358513245155336e-01, 4.12109379559176147723e-01,
-8.86466480500416764698e-01, 4.16015625000012378987e-01,-8.77032459330052915902e-01,
 4.19921880642374900638e-01,-8.67686583475119732611e-01, 4.23828127219428474337e-01,
-8.58427266262424382148e-01, 4.27734368719141833104e-01,-8.49252909907022512925e-01,
 4.31640618968065714611e-01,-8.40161937496191058017e-01, 4.35546878483443478114e-01,
-8.31152845581527799368e-01, 4.39453118679711263184e-01,-8.22224237217255904397e-01,
 4.43359368032235523494e-01,-8.13374623273944052926e-01, 4.47265628008731153109e-01,
-8.04602614758323397837e-01, 4.51171870696748356089e-01,-7.95906924055657327699e-01,
 4.55078119554000015690e-01,-7.87286183440983222681e-01, 4.58984369088840549189e-01,
-7.78739123774130348465e-01, 4.62890628186940544797e-01,-7.70264477019509286038e-01,
 4.66796871065550678104e-01,-7.61861081536608253373e-01, 4.70703123834832981576e-01,
-7.53527694024228367198e-01, 4.74609367819626204454e-01,-7.45263196827977481007e-01,
 4.78515629309174295170e-01,-7.37066405489485987346e-01, 4.82421867973577944699e-01,
-7.28936302976421757194e-01, 4.86328121937636970173e-01,-7.20871734871707303682e-01,
 4.90234370146730102924e-01,-7.12871695807621108543e-01, 4.94140619471375386329e-01,
-7.04935147500350467453e-01, 4.98046870348540282603e-01,-6.97061089220483154349e-01,
 5.03906259479666607248e-01,-6.85365021305528898665e-01, 5.11718741671807308968e-01,
-6.69980137553352506075e-01, 5.19531239866620975221e-01,-6.54828335762658753083e-01,
 5.27343741777559715800e-01,-6.39902681633316205989e-01, 5.35156244243337408939e-01,
-6.25196529408412970952e-01, 5.42968742895167721940e-01,-6.10703524434029532131e-01,
 5.50781256908714822451e-01,-5.96417541557912045391e-01, 5.58593745191563129104e-01,
-5.82332822827765994589e-01, 5.66406238929898164791e-01,-5.68443721603443918866e-01,
 5.74218739399718569594e-01,-5.54744876161180400231e-01, 5.82031242514368440943e-01,
-5.41231151395322740427e-01, 5.89843746273765234811e-01,-5.27897613981963331753e-01,
 5.97656239702385239099e-01,-5.14739540317123100976e-01, 6.05468757525321366053e-01,
-5.01752315131398063386e-01, 6.13281239959553414742e-01,-4.88931655502938156133e-01,
 6.21093740304493158177e-01,-4.76273257869706867496e-01, 6.28906245230470117846e-01,
-4.63773087078948265116e-01, 6.36718753501692269481e-01,-4.51427238173209843808e-01,
 6.44531260162870434094e-01,-4.39231954811134528427e-01, 6.52343741852399516823e-01,
-4.27183644552542318706e-01, 6.60156257917380973055e-01,-4.15278717563296595472e-01,
 6.67968739260340615971e-01,-4.03513904054989325232e-01, 6.75781241239737928694e-01,
-3.91885862944945984054e-01, 6.83593761137851640974e-01,-3.80391454262962724897e-01,
 6.91406255768921229610e-01,-3.69027703561982889369e-01, 6.99218740247065317917e-01,
-3.57791652587138742092e-01, 7.07031245848167344548e-01,-3.46680419085942048696e-01,
 7.14843738508435189161e-01,-3.35691307713773312571e-01, 7.22656242580161034539e-01,
-3.24821629668690547987e-01, 7.30468743396710196158e-01,-3.14068836664773698431e-01,
 7.38281247573092769265e-01,-3.03430432707159525307e-01, 7.46093742908741974773e-01,
-2.92904025937446021999e-01, 7.53906262884932787394e-01,-2.82487238483781766263e-01,
 7.61718759785716348354e-01,-2.72177873068926601263e-01, 7.69531260380107551278e-01,
-2.61973702252703333215e-01, 7.77343760627546620512e-01,-2.51872606083452410530e-01,
 7.85156254726150915069e-01,-2.41872530401110452658e-01, 7.92968740426921891995e-01,
-2.31971477510228396568e-01, 8.00781261196431781713e-01,-2.22167451359268852684e-01,
 8.08593756541109542901e-01,-2.12458643124705304217e-01, 8.16406253653128688441e-01,
-2.02843188040105815162e-01, 8.24218760627765778537e-01,-1.93319298109145165654e-01,
 8.32031247727335609987e-01,-1.83885281501602548815e-01, 8.39843760430642349846e-01,
-1.74539403932158165844e-01, 8.47656237173634474757e-01,-1.65280106070667898166e-01,
 8.55468740561889928742e-01,-1.56105725695738328795e-01, 8.63281242855145070081e-01,
-1.47014751238202734385e-01, 8.71093761011301648267e-01,-1.38005660378667016630e-01,
 8.78906246479578712183e-01,-1.29077046280599461170e-01, 8.86718745554813336796e-01,
-1.20227432011233759113e-01, 8.94531255727883389817e-01,-1.11455434522099494643e-01,
 9.02343756660601292729e-01,-1.02759726576323373592e-01, 9.10156242115593427933e-01,
-9.41389995765575804398e-02, 9.17968742511428348330e-01,-8.55919384931667082128e-02,
 9.25781261611553829560e-01,-7.71172908019934766344e-02, 9.33593738610156709257e-01,
-6.87139047480513909250e-02, 9.41406243185352820113e-01,-6.03805182277028507087e-02,
 9.49218753027978534575e-01,-5.21159979490448677097e-02, 9.57031257760502129806e-01,
-4.39192258259026857559e-02, 9.64843762458745479726e-01,-3.57890949388774970430e-02,
 9.72656263512252539094e-01,-2.77245341227398967021e-02, 9.80468735443280725583e-01,
-1.97245201944724612986e-02, 9.88281264564145001117e-01,-1.17879410151999763912e-02,
 9.96093762169889318514e-01,-3.91388710352202906506e-03, 1.00781251240719083029e+00,
 7.78215275306592957349e-03, 1.02343749698308061902e+00, 2.31670563337047497043e-02,
 1.03906249118153359845e+00, 3.83188558151914548944e-02, 1.05468748157724356851e+00,
 5.32444970513097359799e-02, 1.07031248816185087236e+00, 6.79506508480471893563e-02,
 1.08593748790200428367e+00, 8.24436580704741572845e-02, 1.10156249160902186546e+00,
 9.67296188412092305953e-02, 1.11718752003305499265e+00, 1.10814384271975541196e-01,
 1.13281251530680160400e+00, 1.24703492013168215635e-01, 1.14843748890893770920e+00,
 1.38402313201595461578e-01, 1.16406248149615909604e+00, 1.51916026129924830457e-01,
 1.17968751032788810029e+00, 1.65249581650073196215e-01, 1.19531252015809741351e+00,
 1.78407674337108801410e-01, 1.21093751174476271792e+00, 1.91394862698530232592e-01,
 1.22656248446226467941e+00, 2.04215528760983033862e-01, 1.24218751182243236642e+00,
 2.16873947818044143787e-01, 1.25781248163997516620e+00, 2.29374086468055793375e-01,
 1.27343751496728185835e+00, 2.41719948640593429445e-01, 1.28906252268936594163e+00,
 2.53915227582410807372e-01, 1.30468748289946656627e+00, 2.65963535390142169490e-01,
 1.32031248582116345069e+00, 2.77868440264455784927e-01, 1.33593748174498849757e+00,
 2.89633278918472569785e-01, 1.35156250993076532119e+00, 3.01261337925780026126e-01,
 1.36718748565416658991e+00, 3.12755699510944396025e-01, 1.38281251931152415757e+00,
 3.24119482619607879315e-01, 1.39843747672915519331e+00, 3.35355525280533584009e-01,
 1.41406248777595644484e+00, 3.46466758701581056812e-01, 1.42968752294135992287e+00,
 3.57455904968219329021e-01, 1.44531248842103421381e+00, 3.68325553147315076785e-01,
 1.46093750149440793074e+00, 3.79078353957879699010e-01, 1.47656247952452046590e+00,
 3.89716737273033686506e-01, 1.49218747732647671889e+00, 4.00243148932190706546e-01,
 1.50781251492859502683e+00, 4.10659934886098199058e-01, 1.52343752291366207885e+00,
 4.20969309684892323276e-01, 1.53906247798230921831e+00, 4.31173450512460476602e-01,
 1.55468747961488618081e+00, 4.41274547692842133984e-01, 1.57031247434656928696e+00,
 4.51274627802945360600e-01, 1.58593748798769529351e+00, 4.61175707547909041750e-01,
 1.60156248644776399992e+00, 4.70979706756907035015e-01, 1.61718752177671931669e+00,
 4.80688542811549268663e-01, 1.63281251625628387458e+00, 4.90303998001195395773e-01,
 1.64843752384669883781e+00, 4.99827884022693069710e-01, 1.66406250636281405342e+00,
 5.09261905613470844401e-01, 1.67968748109659626877e+00, 5.18607752953926137707e-01,
 1.69531252192489700903e+00, 5.27867102553500444628e-01, 1.71093749118963378564e+00,
 5.37041460747445853485e-01, 1.72656252427164624841e+00, 5.46132451655921613565e-01,
 1.74218749999999289457e+00, 5.55141507540497514128e-01, 1.75781251538232408116e+00,
 5.64070147035636182409e-01, 1.77343748719091087729e+00, 5.72919746339039193117e-01,
 1.78906248938062795162e+00, 5.81691733698903590266e-01, 1.80468750732626403988e+00,
 5.90387450661751245207e-01, 1.82031249578858300708e+00, 5.99008187332515262824e-01,
 1.83593747354701553043e+00, 6.07555235816107597202e-01, 1.85156250941600220017e+00,
 6.16029882300949793716e-01, 1.86718747198466905601e+00, 6.24433273007866773696e-01,
 1.88281251854803266532e+00, 6.32766679422275046818e-01, 1.89843750925419829656e+00,
 6.41031184295570710674e-01, 1.91406251336923349804e+00, 6.49227953609852193040e-01,
 1.92968747888683922476e+00, 6.57358061767126855557e-01, 1.94531252098439488840e+00,
 6.65422643332249208470e-01, 1.96093747899964632175e+00, 6.73422664502822954091e-01,
 1.97656248201611228765e+00, 6.81359215709335330757e-01, 1.99218752884152139337e+00,
 6.89233295716121574515e-01, 2.01562503971062723096e+00, 7.00929340703396519530e-01,
 2.04687497217855618459e+00, 7.16314226249323837692e-01, 2.07812497140140850505e+00,
 7.31466031100353575312e-01, 2.10937504032632006101e+00, 7.46391714196420252847e-01,
 2.14062497583491539999e+00, 7.61097831179654349398e-01, 2.17187498113030930114e+00,
 7.75590841082816950802e-01, 2.20312495817905595885e+00, 7.89876788035940080590e-01,
 2.23437502912995977766e+00, 8.03961559937420133792e-01, 2.26562495620357839954e+00,
 8.17850639730067996425e-01, 2.29687496507983723859e+00, 8.31549488215728160867e-01,
 2.32812495715711742506e+00, 8.45063204183475136233e-01, 2.35937497502142834449e+00,
 8.58396742868308137986e-01, 2.39062503932786185601e+00, 8.71554854483633789641e-01,
 2.42187501377638847444e+00, 8.84542039247889988474e-01, 2.45312501769688973141e+00,
 8.97362729202654918481e-01, 2.48437504260666974787e+00, 9.10021136010414011253e-01,
 2.51562500964020818728e+00, 9.22521285456923578749e-01, 2.54687503987917640558e+00,
 9.34867133105171888019e-01, 2.57812501315940245661e+00, 9.47062395645161814173e-01,
 2.60937496430218240917e+00, 9.59110715376482403371e-01, 2.64062502454268344110e+00,
 9.71015640857672224229e-01, 2.67187497140907925797e+00, 9.82780462442292446212e-01,
 2.70312504425808741715e+00, 9.94408527511041029356e-01, 2.73437495438762923072e+00,
 1.00590287388274646219e+00, 2.76562500000013766766e+00, 1.01726664921420706378e+00,
 2.79687501777765179511e+00, 1.02850272883733850904e+00, 2.82812496984669614619e+00,
 1.03961393724421213847e+00, 2.85937495431754040709e+00, 1.05060305350536964397e+00,
 2.89062498655182231744e+00, 1.06147273706631040113e+00, 2.92187495636205607497e+00,
 1.07222551856000336556e+00, 2.95312497499041448634e+00, 1.08286392323111613223e+00,
 2.98437502541423205216e+00, 1.09339035320272159879e+00, 3.01562495765420823801e+00,
 1.10380709150308575772e+00, 3.04687505153704263350e+00, 1.11411649211879648647e+00,
 3.07812495371851957415e+00, 1.12432063034270868229e+00, 3.10937503879072352220e+00,
 1.13442175384022903017e+00, 3.14062495409474085051e+00, 1.14442181008280385868e+00,
 3.17187495005669939374e+00, 1.15432287993644422386e+00, 3.20312504793092989175e+00,
 1.16412691074253871193e+00, 3.23437501500604795979e+00, 1.17383571454524826549e+00,
 3.26562504346303139613e+00, 1.18345118191439269140e+00, 3.29687505225279586796e+00,
 1.19297506596558378078e+00, 3.32812498454565064421e+00, 1.20240907770619287476e+00,
 3.35937503339870691121e+00, 1.21175495470993155323e+00, 3.39062503479762877845e+00,
 1.22101428044368276460e+00, 3.42187495275115294291e+00, 1.23018863264894662279e+00,
 3.45312497125021566902e+00, 1.23927960983235152526e+00, 3.48437503373005741381e+00,
 1.24828869778082207631e+00, 3.51562504113336071754e+00, 1.25721733054490414361e+00,
 3.54687503154434047303e+00, 1.26606694301528932556e+00, 3.57812501938674865087e+00,
 1.27483892561269840371e+00, 3.60937505407354297660e+00, 1.28353464214353607531e+00,
 3.64062504877879167253e+00, 1.29215538360449500033e+00, 3.67187496207797625658e+00,
 1.30070242045678696741e+00, 3.70312504289049204687e+00, 1.30917706935770183918e+00,
 3.73437497025255416006e+00, 1.31758046060599554039e+00, 3.76562504481582172033e+00,
 1.32591386203228012164e+00, 3.79687502318689329428e+00, 1.33417836608771267493e+00,
 3.82812502157539080727e+00, 1.34237513282107556734e+00, 3.85937503289095706904e+00,
 1.35050526179065855104e+00, 3.89062502451579517171e+00, 1.35856981940628429584e+00,
 3.92187498828048264343e+00, 1.36656985278386855498e+00, 3.95312505200383812465e+00,
 1.37450641852296939760e+00, 3.98437502731383608179e+00, 1.38238046865399155649e+00,
 4.03125005469345243370e+00, 1.39407651512931352045e+00, 4.09374992224314571132e+00,
 1.40946140140738407354e+00, 4.15624992721350849934e+00, 1.42461320790948775539e+00,
 4.21874999081738799589e+00, 1.43953887346208375675e+00, 4.28125004926801899785e+00,
 1.45424503453625675320e+00, 4.34375002246581676246e+00, 1.46873803550295178866e+00,
 4.40624992903396961452e+00, 1.48302397147267583932e+00, 4.46875001910619662482e+00,
 1.49710873173569325445e+00, 4.53124994046162132832e+00, 1.51099782648134350893e+00,
 4.59374991178148306403e+00, 1.52469666477497867163e+00, 4.65624996950393210682e+00,
 1.53821039659624148044e+00, 4.71874994272904135784e+00, 1.55154392187830580951e+00,
 4.78125007659589229547e+00, 1.56470203461276469170e+00, 4.84375008894072145438e+00,
 1.57768923248147530103e+00, 4.90624991425180390081e+00, 1.59050988507124202442e+00,
 4.96874994122364821436e+00, 1.60316828759130203252e+00, 5.03124993456430402716e+00,
 1.61566844917888374766e+00, 5.09374999080891122105e+00, 1.62801429620265025733e+00,
 5.15624995208009284653e+00, 1.64020956180729626794e+00, 5.21874992581080654475e+00,
 1.65225789540113510157e+00, 5.28124996012553449276e+00, 1.66416280457315224339e+00,
 5.34374996542892954920e+00, 1.67592764723349318956e+00, 5.40624993309582357881e+00,
 1.68755567932271333120e+00, 5.46874993937439146663e+00, 1.69905006003796188452e+00,
 5.53125006144675523956e+00, 1.71041384088312042877e+00, 5.59374993802343833948e+00,
 1.72164989196141959837e+00, 5.65625007017449554780e+00, 1.73276114087263977126e+00,
 5.71875009119658272994e+00, 1.74375026598863769323e+00, 5.78124990194527921261e+00,
 1.75461990531778155855e+00, 5.84374993027079980834e+00, 1.76537270212259045365e+00,
 5.90624990601647237298e+00, 1.77601109634736076259e+00, 5.96875008712475541017e+00,
 1.78653753984372087871e+00, 6.03124998936701928187e+00, 1.79695428434217774338e+00,
 6.09375004551488519411e+00, 1.80726366323312959139e+00, 6.15625009710122661488e+00,
 1.81746784171104991401e+00, 6.21875007375134014609e+00, 1.82756893378427776042e+00,
 6.28124992012524163698e+00, 1.83756899254296968138e+00, 6.34375008018464825454e+00,
 1.84747008888200525689e+00, 6.40624993585989521705e+00, 1.85727406632656766305e+00,
 6.46875007588893069510e+00, 1.86698290219726459327e+00, 6.53125010459241472915e+00,
 1.87659836517923395149e+00, 6.59375003298073902869e+00, 1.88612223567815817660e+00,
 6.65624996392550993818e+00, 1.89555625749005685066e+00, 6.71875005696956328904e+00,
 1.90490213380712702929e+00, 6.78125004511757545345e+00, 1.91416145739401599712e+00,
 6.84374993290853961270e+00, 1.92333581721345581172e+00, 6.90624990255320447829e+00,
 1.93242678460808292584e+00, 6.96875009577413884188e+00, 1.94143588240376629983e+00,
 7.03125009895778330105e+00, 1.95036451347868933226e+00, 7.09374992665843695505e+00,
 1.95921410434277731838e+00, 7.15625003531054382222e+00, 1.96798610568873755255e+00,
 7.21874990123353565963e+00, 1.97668179404013244493e+00, 7.28124992342903976095e+00,
 1.98530254024979058158e+00, 7.34375001884684319720e+00, 1.99384961391081105830e+00,
 7.40625000971154801022e+00, 2.00232423964666850225e+00, 7.46874991202083737818e+00,
 2.01072763735214721237e+00, 7.53125010253679239014e+00, 2.01906104430577215680e+00,
 7.59375005367351363361e+00, 2.02732554760893890489e+00, 7.65625002852042779722e+00,
 2.03552231147011752910e+00, 7.71875003834845063722e+00, 2.04365243879647096037e+00,
 7.78125010738992095582e+00, 2.05171700746609531052e+00, 7.84374991099154428298e+00,
 2.05971702498436570750e+00, 7.90624995140007325034e+00, 2.06765357978076735890e+00,
 7.96874991778560293199e+00, 2.07552763204159873567e+00, 8.06250016518907841601e+00,
 2.08722370261045853113e+00, 8.18749991730411430524e+00, 2.10260859086110940552e+00,
 8.31249999999961275421e+00, 2.11776040598192594189e+00, 8.43750006759882786866e+00,
 2.13268606421036110987e+00, 8.56249989994042870478e+00, 2.14739219190255425929e+00,
 8.68750017070301971955e+00, 2.16188523054017878167e+00, 8.81249988937373984754e+00,
 2.17617115558505247463e+00, 8.93749985499407451073e+00, 2.19025589179568669351e+00,
 9.06250017260934903618e+00, 2.20414503922734184016e+00, 9.18749989982229031682e+00,
 2.21784385363525871071e+00, 9.31249989473215578073e+00, 2.23135757240174825711e+00,
 9.43749987280140523183e+00, 2.24469110109714620194e+00, 9.56249985130819624146e+00,
 2.25784918360318442865e+00, 9.68750016046429962557e+00, 2.27083641124352197949e+00,
 9.81250013735593640263e+00, 2.28365709710658393305e+00, 9.93749981612062782688e+00,
 2.29631546147686549375e+00, 1.00625001807762632922e+01, 2.30881566071002453100e+00,
 1.01875001064603036838e+01, 2.32116148901707219920e+00, 1.03124998754699284120e+01,
 2.33335673958515599224e+00, 1.04374998465447372809e+01, 2.34540507547467313998e+00,
 1.05625000546411484947e+01, 2.35730999785641870403e+00, 1.06874998146544175626e+01,
 2.36907481692060173373e+00, 1.08125000660874288627e+01, 2.38070287837012983800e+00,
 1.09374998135429191137e+01, 2.39219723463622813142e+00, 1.10625001738922996708e+01,
 2.40356102605312571541e+00, 1.11875001595451468717e+01, 2.41479709786199236987e+00,
 1.13124998430717198517e+01, 2.42590829515393124893e+00, 1.14375001814316199500e+01,
 2.43689744646451345034e+00, 1.15624998191289680705e+01, 2.44776708719564339845e+00,
 1.16874999094894853613e+01, 2.45851988687059019867e+00, 1.18125001358197554424e+01,
 2.46915830431782978849e+00, 1.19375000879390356090e+01, 2.47968471317346939031e+00,
 1.20625001217224063765e+01, 2.49010147675608095597e+00, 1.21875001308306494252e+01,
 2.50041084705878802197e+00, 1.23124997977759438328e+01, 2.51061499007391830673e+00,
 1.24375001384567660523e+01, 2.52071611361691338971e+00, 1.25624998694263041443e+01,
 2.53071617542536841938e+00, 1.26875001060564027711e+01, 2.54061726516113139951e+00,
 1.28124997965593330207e+01, 2.55042124102033085364e+00, 1.29375002053768994870e+01,
 2.56013008690013066371e+00, 1.30625001029002039132e+01, 2.56974553760255730595e+00,
 1.31875001075468158973e+01, 2.57926941939149401151e+00, 1.33124999207400716017e+01,
 2.58870343751584641723e+00, 1.34374999116630711171e+01, 2.59804929931397055043e+00,
 1.35625002116033126498e+01, 2.60730864690276575857e+00, 1.36874999999994013677e+01,
 2.61648300757667584548e+00, 1.38124998845031292660e+01, 2.62557397091620714491e+00,
 1.39374999270277761809e+01, 2.63458304398466225393e+00, 1.40624999689059801966e+01,
 2.64351167775350859301e+00, 1.41874998364109536908e+01, 2.65236128371111590241e+00,
 1.43125000600487535252e+01, 2.66113328551000449806e+00, 1.44375001757675605063e+01,
 2.66982900045638871234e+00, 1.45624999079856891626e+01, 2.67844972500734002097e+00,
 1.46874998221265418152e+01, 2.68699677979384432547e+00, 1.48124998440300590374e+01,
 2.69547140836573362321e+00, 1.49375002194337920258e+01, 2.70387484438185765612e+00,
 1.50624999999999111822e+01, 2.71220821125086786196e+00, 1.51875001218394309888e+01,
 2.72047272912311655091e+00, 1.53124999999999875655e+01, 2.72866948830494493450e+00,
 1.54375002229681541621e+01, 2.73679962883147709363e+00, 1.55625001666588858029e+01,
 2.74486418493393102835e+00, 1.56874999233435197254e+01, 2.75286421200553377275e+00,
 1.58124998204550628600e+01, 2.76080075513311840041e+00, 1.59374997936560180989e+01,
 2.76867480997157144884e+00, 1.61250002373038583414e+01, 2.78037087739835442335e+00,
 1.63749997265271431957e+01, 2.79575576482068299811e+00, 1.66249998225723523149e+01,
 2.81090757586957806780e+00, 1.68750001889482916795e+01, 2.82583324795552925934e+00,
 1.71250002043690727760e+01, 2.84053939608224936819e+00, 1.73749996804588562327e+01,
 2.85503237305999846285e+00, 1.76250001336612847069e+01, 2.86931835628195130994e+00,
 1.78750001895202217383e+01, 2.88340309918260118138e+00, 1.81250002311237210506e+01,
 2.89729221349239196570e+00, 1.83749997051896336586e+01, 2.91099102905479867331e+00,
 1.86249997272874452392e+01, 2.92450474962333828444e+00, 1.88750002785230499569e+01,
 2.93783830989127636713e+00, 1.91250002892026316204e+01, 2.95099639483430564724e+00,
 1.93749997170268883906e+01, 2.96398356063434675889e+00, 1.96249998157258076503e+01,
 2.97680425427870432031e+00, 1.98750002732812163231e+01, 2.98946267429039380659e+00,
 2.01250002250250084046e+01, 3.00196283448599388066e+00, 2.03750003000636183970e+01,
 3.01430867385397505132e+00, 2.06249997032550531628e+01, 3.02650391783311079053e+00,
 2.08750003179291354627e+01, 3.03855228596705728705e+00, 2.11249999999995630162e+01,
 3.05045717324321685737e+00, 2.13750003234410499431e+01, 3.06222202995456882491e+00,
 2.16249996567974740458e+01, 3.07385003694730585266e+00, 2.18749997455953462122e+01,
 3.08534442061375102639e+00, 2.21250001585507298785e+01, 3.09670819806012787012e+00,
 2.23750003782183704004e+01, 3.10794428106453324290e+00, 2.26249996548801064478e+01,
 3.11905547433207175345e+00, 2.28749996378158897414e+01, 3.13004459532839973690e+00,
 2.31249996701439037849e+01, 3.14091426913444138336e+00, 2.33750003934210077716e+01,
 3.15166709200559580495e+00, 2.36250000849554133708e+01, 3.16230547697580277600e+00,
 2.38750003332367519704e+01, 3.17283190032435413741e+00, 2.41249996568797726582e+01,
 3.18324863300244942366e+00, 2.43749996762421652363e+01, 3.19355800360153807560e+00,
 2.46250000487733977650e+01, 3.20376218903879816935e+00, 2.48750002596276722500e+01,
 3.21386329348194976774e+00, 2.51249998118203556885e+01, 3.22386335888950270601e+00,
 2.53749995894785023154e+01, 3.23376442118376417056e+00, 2.56249997095792600987e+01,
 3.24356842612507989543e+00, 2.58750002502002267590e+01, 3.25327726125510707789e+00,
 2.61249996893753184679e+01, 3.26289269839503504045e+00, 2.63750002762853732463e+01,
 3.27241660227150532592e+00, 2.66250001751140068507e+01, 3.28185063060664106160e+00,
 2.68750001084056364675e+01, 3.29119649048152496107e+00, 2.71250003663005934129e+01,
 3.30045582536479287228e+00, 2.73749996485453550576e+01, 3.30963017529813896545e+00,
 2.76249996096224883502e+01, 3.31872114570660414756e+00, 2.78749996913755175854e+01,
 3.32773021870855245652e+00, 2.81250002286145956987e+01, 3.33665886865310312714e+00,
 2.83749996402423469988e+01, 3.34550846312288285844e+00, 2.86250001015823016814e+01,
 3.35428046542313040845e+00, 2.88750003041739127241e+01, 3.36297617937611903471e+00,
 2.91250004270154860819e+01, 3.37159692654734044481e+00, 2.93749997994938283341e+01,
 3.38014396563858099398e+00, 2.96249995605638893892e+01, 3.38861858462200871145e+00,
 2.98749997784404683898e+01, 3.39702200283545607817e+00, 3.01250001915207263892e+01,
 3.40535539816835353122e+00, 3.03750002695626619698e+01, 3.41361991053520341666e+00,
 3.06249995524309781558e+01, 3.42181665425039227557e+00, 3.08750003934518773008e+01,
 3.42994680769152182620e+00, 3.11250001941113580983e+01, 3.43801136102138116257e+00,
 3.13749996216052444709e+01, 3.44601138539155726903e+00, 3.16250000000007815970e+01,
 3.45394794704770902172e+00, 3.18750000773922828046e+01, 3.46182200590658339934e+00,
 3.22500001179274988772e+01, 3.47351804689844811946e+00, 3.27499998467818045356e+01,
 3.48890295740284273052e+00, 3.32500005065227242085e+01, 3.50405478233562917012e+00,
 3.37500004609757908725e+01, 3.51898043097708068672e+00, 3.42500004170650456103e+01,
 3.53368657688531584427e+00, 3.47500005921336807546e+01, 3.54817958905061914265e+00,
 3.52499993306367329637e+01, 3.56246551026924862526e+00, 3.57500006774600365134e+01,
 3.57655028808994757128e+00, 3.62499993175142094515e+01, 3.59043936247348938196e+00,
 3.67499998882642415765e+01, 3.60413822261841687578e+00, 3.72500006462765043125e+01,
 3.61765196217527318368e+00, 3.77500006014371436436e+01, 3.63098549162714334670e+00,
 3.82500005615656846203e+01, 3.64414357495400054177e+00, 3.87499995218681902998e+01,
 3.65713074346047051222e+00, 3.92499992736606699850e+01, 3.66995142572295662831e+00,
 3.97499997623556282633e+01, 3.68260983512186612643e+00, 4.02500006739417841573e+01,
 3.69511002060846749373e+00, 4.07499992658592091743e+01, 3.70745582167114662298e+00,
 4.12499993244729807884e+01, 3.71965109640427726845e+00, 4.17500006571579334036e+01,
 3.73169946703717414493e+00, 4.22500007391862695272e+01, 3.74360437129871570505e+00,
 4.27499994640256133493e+01, 3.75536918284535659041e+00, 4.32500005077441045387e+01,
 3.76699724511763633927e+00, 4.37500007636999868055e+01, 3.77849163025962297979e+00,
 4.42499994116789494569e+01, 3.78985535815854746033e+00, 4.47499996819114471691e+01,
 3.80109143761274026474e+00, 4.52500007475566405901e+01, 3.81220268666652373213e+00,
 4.57500007230116381152e+01, 3.82319180752506326826e+00, 4.62500005890746379578e+01,
 3.83406147669518304610e+00, 4.67500006654374189452e+01, 3.84481426996865138790e+00,
 4.72499993411726251225e+01, 3.85545263999631515617e+00, 4.77499997066169967752e+01,
 3.86597906078259256901e+00, 4.82500001382493906021e+01, 3.87639583065026727127e+00,
 4.87499992485481499216e+01, 3.88670518202945913799e+00, 4.92500002541726615846e+01,
 3.89690937277896409796e+00, 4.97500004822119876735e+01, 3.90701047329730499769e+00,
 5.02499998975306851889e+01, 3.91701054489999478392e+00, 5.07500006686205864526e+01,
 3.92691163109668650577e+00, 5.12499998880218186059e+01, 3.93671561583357743075e+00,
 5.17500007492256415276e+01, 3.94642444662326852267e+00, 5.22500007762066402961e+01,
 3.95603990570054975251e+00, 5.27499993476125084158e+01, 3.96556375998864041321e+00,
 5.32499996194679781070e+01, 3.97499779744339321041e+00, 5.37499991438484343576e+01,
 3.98434365107937082939e+00, 5.42500005974988752655e+01, 3.99360300343437302217e+00,
 5.47499995525040787925e+01, 4.00277736052316868154e+00, 5.52499995417688793964e+01,
 4.01186833210408622108e+00, 5.57499991839046060704e+01, 4.02087739570174562687e+00,
 5.62499993741628614430e+01, 4.02980602995853587345e+00, 5.67499992279580567356e+01,
 4.03865564275724864274e+00, 5.72500001161288381013e+01, 4.04742764446280034463e+00,
 5.77500004840416494289e+01, 4.05612335778357646632e+00, 5.82499993045054154095e+01,
 4.06474408050598867703e+00, 5.87500007989451660251e+01, 4.07329116662333490240e+00,
 5.92499994509702787582e+01, 4.08176577074891611119e+00, 5.97499993224992778096e+01,
 4.09016917947269575961e+00, 6.02499994530567590800e+01, 4.09850256329286821710e+00,
 6.07499994246793022512e+01, 4.10676707275035823841e+00, 6.12499991469727405047e+01,
 4.11496383549786060030e+00, 6.17500006555741833836e+01, 4.12309398612467248313e+00,
 6.22500008533554947121e+01, 4.13115854905333890201e+00, 6.27500003821617085009e+01,
 4.13915858410211967566e+00, 6.32499994235490348160e+01, 4.14709511849378031911e+00,
 6.37500005084463197136e+01, 4.15496919201416403666e+00, 6.45000010144042761340e+01,
 4.16666523952892475791e+00, 6.55000009153946365359e+01, 4.18205015661669676774e+00,
 6.65000012383425627149e+01, 4.19720196628350095125e+00, 6.75000010549665887538e+01,
 4.21212761350761866197e+00, 6.85000008408314045028e+01, 4.22683375754309054173e+00,
 6.95000010744837766197e+01, 4.24132676803094454954e+00, 7.04999988697136643623e+01,
 4.25561269378579254408e+00, 7.15000010232526364007e+01, 4.26969746401118754875e+00,
 7.24999991096075717678e+01, 4.28358654957935414131e+00, 7.34999991170870146107e+01,
 4.29728539420636934665e+00, 7.44999998784783059591e+01, 4.31079912375435014837e+00,
 7.54999987221771817758e+01, 4.32413263933017333329e+00, 7.65000010790418514262e+01,
 4.33729075493761229154e+00, 7.74999992741503263005e+01, 4.35027792699349902250e+00,
 7.84999988063749896128e+01, 4.36309860958294848388e+00, 7.94999987775087930686e+01,
 4.37575700628303820849e+00, 8.05000007162134068039e+01, 4.38825719332157859753e+00,
 8.14999992708303153677e+01, 4.40060301129994968505e+00, 8.24999985106456534822e+01,
 4.41279827528785517643e+00, 8.35000013466249413341e+01, 4.42484664798405447073e+00,
 8.45000013990178899803e+01, 4.43675155091955275566e+00, 8.55000007739268994555e+01,
 4.44851638499449109077e+00, 8.65000003896926585867e+01, 4.46014441844295106421e+00,
 8.74999989431682791974e+01, 4.47163878128549185931e+00, 8.84999984704333400032e+01,
 4.48300253473064458376e+00, 8.94999989818597043723e+01, 4.49423861390494039370e+00,
 9.05000000000015489832e+01, 4.50534985070589755196e+00, 9.15000014332025557451e+01,
 4.51633898794489141437e+00, 9.25000010027577985738e+01, 4.52720865535900429677e+00,
 9.34999989823288757407e+01, 4.53796142541045810503e+00, 9.44999991999358712746e+01,
 4.54859982603340995411e+00, 9.54999987372769965077e+01, 4.55912623426445406949e+00,
 9.65000009897477610821e+01, 4.56954301860139366909e+00, 9.74999998096137119319e+01,
 4.57985237605112160963e+00, 9.85000006671633201449e+01, 4.59005655495127484755e+00,
 9.95000008645123301676e+01, 4.60015765285311317712e+00, 1.00499998470700148800e+02,
 4.61015771228221638012e+00, 1.01500000918823687357e+02, 4.62005880753429210017e+00,
 1.02499998724223871704e+02, 4.62986278613186641451e+00, 1.03499999285656400616e+02,
 4.63957160580355321144e+00, 1.04499999482174644072e+02, 4.64918706644959911500e+00,
 1.05499998535447659265e+02, 4.65871093903410837811e+00, 1.06500000792753027667e+02,
 4.66814499259317017277e+00, 1.07500001606394491205e+02, 4.67749086251092194289e+00,
 1.08500000817527777031e+02, 4.68675018051533243124e+00, 1.09500001667182928600e+02,
 4.69592456448197026475e+00, 1.10500001557067378144e+02, 4.70501553504891489865e+00,
 1.11499999176461074057e+02, 4.71402458351417408267e+00, 1.12499998469950213575e+02,
 4.72295320804403218062e+00, 1.13499998344024007224e+02, 4.73180282233136040304e+00,
 1.14499999999999516831e+02, 4.74057482299429011618e+00, 1.15500000663559873715e+02,
 4.74927053570695534290e+00, 1.16499999000340153543e+02, 4.75789126442498400138e+00,
 1.17500001082404153863e+02, 4.76643834279616385174e+00, 1.18499998345167767866e+02,
 4.77491294661035681202e+00, 1.19500001713525534797e+02, 4.78331638571069106547e+00,
 1.20500001362604109545e+02, 4.79164976423862754018e+00, 1.21500001239572554823e+02,
 4.79991427298284634162e+00, 1.22499998366538306982e+02, 4.80811101665040041553e+00,
 1.23500001007112388152e+02, 4.81624116422278802929e+00, 1.24500001318374685866e+02,
 4.82430572649411715247e+00, 1.25499998419163517838e+02, 4.83230574597553186322e+00,
 1.26500001669624879241e+02, 4.84024232136619048106e+00, 1.27499998575683619606e+02,
 4.84811635342737190513e+00, 1.28999999142948269082e+02, 4.85981239771786022175e+00,
 1.30999997472650591135e+02, 4.87519730390840777545e+00, 1.33000000814921151004e+02,
 4.89034913434898044216e+00, 1.35000002359433693755e+02, 4.90527479591571591300e+00,
 1.37000001714067053626e+02, 4.91998093833956318122e+00, 1.38999998078227719134e+02,
 4.93447391930499179580e+00, 1.41000001766533188174e+02, 4.94875990290677236771e+00,
 1.42999998949253551928e+02, 4.96284462291202999040e+00, 1.44999997315785634555e+02,
 4.97673372390875101701e+00, 1.46999997527868174529e+02, 4.99043256996151285421e+00,
 1.49000001407856387914e+02, 5.00394631539415968291e+00, 1.50999997288146147412e+02,
 5.01727981885562712705e+00, 1.53000002085698497467e+02, 5.03043793502445168997e+00,
 1.54999999199722253707e+02, 5.04342511175616436958e+00, 1.57000001744228683265e+02,
 5.05624581645804482832e+00, 1.59000001449780739904e+02, 5.06890421133834934153e+00,
 1.61000002465417168196e+02, 5.08140438029761298822e+00, 1.62999997766668627719e+02,
 5.09375018710534277488e+00, 1.64999998923320418953e+02, 5.10594546737524979108e+00,
 1.66999997226155954877e+02, 5.11799379580691038427e+00, 1.69000002648529545013e+02,
 5.12989873059484580153e+00, 1.71000000544814412251e+02, 5.14166355968870902871e+00,
 1.73000002651005189591e+02, 5.15329160982150824566e+00, 1.74999999999992610356e+02,
 5.16478597392347182904e+00, 1.76999998203850054779e+02, 5.17614972242608928354e+00,
 1.79000002604839892228e+02, 5.18738582039293305570e+00, 1.81000001084520675931e+02,
 5.19849703725765266427e+00, 1.83000002860902611701e+02, 5.20948616847476841940e+00,
 1.85000001615781258124e+02, 5.22035583381227752398e+00, 1.86999998850400714900e+02,
 5.23110861070699684205e+00, 1.89000003007214075978e+02, 5.24174703097082783643e+00,
 1.90999999194980148332e+02, 5.25227342383186623920e+00, 1.93000002767713510821e+02,
 5.26269020324536995759e+00, 1.94999998427880399277e+02, 5.27299955050159496750e+00,
 1.97000001631514805922e+02, 5.28320373701978951431e+00, 1.99000001547888530240e+02,
 5.29330483250282668450e+00, 2.00999997306160622657e+02, 5.30330489465688970085e+00,
 2.02999999999993292477e+02, 5.31320597904175428994e+00, 2.05000001145878599118e+02,
 5.32300998472805986239e+00, 2.06999997351200420326e+02, 5.33271878046923575312e+00,
 2.08999997869926914973e+02, 5.34233424177307369973e+00, 2.10999996759016738679e+02,
 5.35185811811595613108e+00, 2.13000001693567213579e+02, 5.36129217366044485971e+00,
 2.15000003012713904127e+02, 5.37063804214028550632e+00, 2.17000000000011795009e+02,
 5.37989735354051390459e+00, 2.19000002565770529372e+02, 5.38907174153234791447e+00,
 2.21000001863787360890e+02, 5.39816270995117974962e+00, 2.22999997271335843152e+02,
 5.40717175922395654197e+00, 2.24999999999993804067e+02, 5.41610040220439259429e+00,
 2.26999999999991388222e+02, 5.42495001748136473907e+00, 2.29000000000012477130e+02,
 5.43372200355429413321e+00, 2.31000000000000341061e+02, 5.44241771052179501567e+00,
 2.33000000505073131762e+02, 5.45103845573339640396e+00, 2.34999996461911194956e+02,
 5.45958549908846180188e+00, 2.37000001900791602338e+02, 5.46806014915534888132e+00,
 2.38999998400102668938e+02, 5.47646354523737954878e+00, 2.40999996827141274025e+02,
 5.48479692032526600087e+00, 2.43000002645080542152e+02, 5.49306145422565350600e+00,
 2.45000003114972685125e+02, 5.50125822325890112552e+00, 2.46999998887487095089e+02,
 5.50938833212387635285e+00, 2.49000001501187654185e+02, 5.51745290249357367429e+00,
 2.51000001840596723923e+02, 5.52545294646483853285e+00, 2.53000001613383346921e+02,
 5.53338949510452948743e+00, 2.55000003477524387563e+02, 5.54126355879577658925e+00,
 2.58000004329371677159e+02, 5.55295960170212765661e+00, 2.62000002811905460476e+02,
 5.56834451449356038211e+00, 2.66000003319284815007e+02, 5.58349632126021333534e+00,
 2.69999994830099694809e+02, 5.59842193985059566330e+00, 2.74000003491727738947e+02,
 5.61312811913160203403e+00, 2.78000003327664444441e+02, 5.62762112566065297869e+00,
 2.82000002127009111064e+02, 5.64190707848069905594e+00, 2.86000004959261218573e+02,
 5.65599182815992662654e+00, 2.89999996537899392024e+02, 5.66988091104224167793e+00,
 2.93999998269364198222e+02, 5.68357976145216525765e+00, 2.97999996047752972572e+02,
 5.69709347324283044145e+00, 3.02000005719085777400e+02, 5.71042703631223957927e+00,
 3.05999995978556853515e+02, 5.72358508881040961569e+00, 3.09999994348712050396e+02,
 5.73657227924923063256e+00, 3.14000001242128689682e+02, 5.74939298986407720804e+00,
 3.18000005390336923483e+02, 5.76205139973092173022e+00, 3.22000001952311492914e+02,
 5.77455155160748745402e+00, 3.25999994396890031112e+02, 5.78689736417925360001e+00,
 3.29999996638473817256e+02, 5.79909264427408288611e+00, 3.33999994295661508659e+02,
 5.81114097589784250886e+00, 3.38000004980140033695e+02, 5.82304591021716078103e+00,
 3.41999994372196113090e+02, 5.83481072060703809257e+00, 3.46000003802201831604e+02,
 5.84643878604674682720e+00, 3.49999995573165392670e+02, 5.85793314183536040218e+00,
 3.53999994650135135998e+02, 5.86929689802116172359e+00, 3.57999993860333574958e+02,
 5.88053296925079393986e+00, 3.61999996841060067254e+02, 5.89164420309941760934e+00,
 3.66000005710800223824e+02, 5.90263334900464542443e+00, 3.70000002190370821609e+02,
 5.91350301155819124688e+00, 3.74000006043282269275e+02, 5.92425581357304054109e+00,
 3.78000004373932199542e+02, 5.93489420719083682343e+00, 3.81999995193611141531e+02,
 5.94542059602440531307e+00, 3.85999998686655089841e+02, 5.95583736606238289824e+00,
 3.89999995619111928136e+02, 5.96614672789064570679e+00, 3.93999996235315563808e+02,
 5.97635089974289712700e+00, 3.98000002685090123578e+02, 5.98645201203089527553e+00,
 4.01999995457559634815e+02, 5.99645207731941809470e+00, 4.05999994468157410665e+02,
 6.00635314597650449286e+00, 4.09999994425560487343e+02, 6.01615714610215945868e+00,
 4.13999999052391387977e+02, 6.02586597153640468605e+00, 4.17999994342520494683e+02,
 6.03548141899011625355e+00, 4.21999998681842384940e+02, 6.04500531091241555259e+00,
 4.26000003590228914163e+02, 6.05443935469713778730e+00, 4.30000005596728613000e+02,
 6.06378522170325595653e+00, 4.33999993914640072035e+02, 6.07304452007883810438e+00,
 4.37999997137553748416e+02, 6.08221890384118069051e+00, 4.42000006264802607348e+02,
 6.09130989625145957689e+00, 4.46000000000004831691e+02, 6.10031895202007490298e+00,
 4.49999996671757969580e+02, 6.10924757536827200965e+00, 4.54000001928818051056e+02,
 6.11809720228984588175e+00, 4.57999999999999772626e+02, 6.12686918411418446340e+00,
 4.62000006608196713387e+02, 6.13556490538519483380e+00, 4.65999993752602392760e+02,
 6.14418562071921314782e+00, 4.70000005288256488711e+02, 6.15273270595571375452e+00,
 4.73999993756915785070e+02, 6.16120730852401266020e+00, 4.78000005276492345274e+02,
 6.16961074353014282678e+00, 4.81999992870493770170e+02, 6.17794409925909349113e+00,
 4.86000005602417559203e+02, 6.18620863542810184299e+00, 4.89999994089511346829e+02,
 6.19440537904245047685e+00, 4.93999992940638719574e+02, 6.20253550289771737880e+00,
 4.98000006843576784377e+02, 6.21060009076677488338e+00, 5.02000006647477391653e+02,
 6.21860013293371594045e+00, 5.05999994190742143019e+02, 6.22653665780671872199e+00,
 5.09999997488753137986e+02, 6.23441072079435798514e+00,
};

/* For j=0 to 31, the entries of _TBL_jlog_**[4j+i] are:
*   _TBL_jlog_p2[4j,4j+1,4j+2,4j+3]={2**(-j),(j*ln2_h),(j*ln2_l),2**(j)}
*   _TBL_jlog_p1[4j,4j+1,4j+2,4j+3]={2**(-32*j),(32*j)*ln2_h,(32*j)*ln2_l,2**(32*j)}
*   _TBL_jlog_n2[4j,4j+1,4j+2,4j+3]={2**(j), -j*ln2_h, -j*ln2_l,2**(-j)}
*   _TBL_jlog_n1[4j,4j+1,4j+2,4j+3]={2**(32*j),-(32*j)*ln2_h,-(32*j*ln2_l),2**(-32*j)}
*  For _TBL_jlog_n1, j may go up to 32 and 33 with the following values:
*   _TBL_jlog_n1[4j,4j+1,4j+2,4j+3] =
*       {2**(32*(j-2)),-(32*j)*ln2_h, -(32*j*ln2_l),2**(-32*(j-2))}
*/

const double _TBL_jlog_p2[] = {
 1.00000000000000000000e+00, 0.00000000000000000000e+00, 0.00000000000000000000e+00,
 1.00000000000000000000e+00, 5.00000000000000000000e-01, 6.93147180559890330187e-01,
 5.49792301870837115524e-14, 2.00000000000000000000e+00, 2.50000000000000000000e-01,
 1.38629436111978066037e+00, 1.09958460374167423105e-13, 4.00000000000000000000e+00,
 1.25000000000000000000e-01, 2.07944154167978467740e+00, 5.12508528396351049179e-14,
 8.00000000000000000000e+00, 6.25000000000000000000e-02, 2.77258872223967500759e+00,
 1.06230083026718816470e-13, 1.60000000000000000000e+01, 3.12500000000000000000e-02,
 3.46573590279967902461e+00, 4.75224754921864982833e-14, 3.20000000000000000000e+01,
 1.56250000000000000000e-02, 4.15888308335956935480e+00, 1.02501705679270209836e-13,
 6.40000000000000000000e+01, 7.81250000000000000000e-03, 4.85203026391957337182e+00,
 4.37940981447378916488e-14, 1.28000000000000000000e+02, 3.90625000000000000000e-03,
 5.54517744447946370201e+00, 9.87733283318216032012e-14, 2.56000000000000000000e+02,
 1.95312500000000000000e-03, 6.23832462503946771903e+00, 4.00657207972892850143e-14,
 5.12000000000000000000e+02, 9.76562500000000000000e-04, 6.93147180559935804922e+00,
 9.50449509843729965667e-14, 1.02400000000000000000e+03, 4.88281250000000000000e-04,
 7.62461898615936206625e+00, 3.63373434498406783797e-14, 2.04800000000000000000e+03,
 2.44140625000000000000e-04, 8.31776616671925239643e+00, 9.13165736369243899321e-14,
 4.09600000000000000000e+03, 1.22070312500000000000e-04, 9.01091334727925641346e+00,
 3.26089661023920717452e-14, 8.19200000000000000000e+03, 6.10351562500000000000e-05,
 9.70406052783914674364e+00, 8.75881962894757832976e-14, 1.63840000000000000000e+04,
 3.05175781250000000000e-05, 1.03972077083991507607e+01, 2.88805887549434651106e-14,
 3.27680000000000000000e+04, 1.52587890625000000000e-05, 1.10903548889590410909e+01,
 8.38598189420271766631e-14, 6.55360000000000000000e+04, 7.62939453125000000000e-06,
 1.17835020695190451079e+01, 2.51522114074948616315e-14, 1.31072000000000000000e+05,
 3.81469726562500000000e-06, 1.24766492500789354381e+01, 8.01314415945785700285e-14,
 2.62144000000000000000e+05, 1.90734863281250000000e-06, 1.31697964306389394551e+01,
 2.14238340600462549970e-14, 5.24288000000000000000e+05, 9.53674316406250000000e-07,
 1.38629436111988297853e+01, 7.64030642471299633940e-14, 1.04857600000000000000e+06,
 4.76837158203125000000e-07, 1.45560907917588338023e+01, 1.76954567125976483625e-14,
 2.09715200000000000000e+06, 2.38418579101562500000e-07, 1.52492379723187241325e+01,
 7.26746868996813567594e-14, 4.19430400000000000000e+06, 1.19209289550781250000e-07,
 1.59423851528787281495e+01, 1.39670793651490433057e-14, 8.38860800000000000000e+06,
 5.96046447753906250000e-08, 1.66355323334386184797e+01, 6.89463095522327501249e-14,
 1.67772160000000000000e+07, 2.98023223876953125000e-08, 1.73286795139986224967e+01,
 1.02387020177004366711e-14, 3.35544320000000000000e+07, 1.49011611938476562500e-08,
 1.80218266945585128269e+01, 6.52179322047841434904e-14, 6.71088640000000000000e+07,
 7.45058059692382812500e-09, 1.87149738751185168439e+01, 6.51032467025183082543e-15,
 1.34217728000000000000e+08, 3.72529029846191406250e-09, 1.94081210556784071741e+01,
 6.14895548573355368558e-14, 2.68435456000000000000e+08, 1.86264514923095703125e-09,
 2.01012682362384111912e+01, 2.78194732280322419089e-15, 5.36870912000000000000e+08,
 9.31322574615478515625e-10, 2.07944154167983015213e+01, 5.77611775098869302213e-14,
 1.07374182400000000000e+09, 4.65661287307739257812e-10, 2.14875625973583055384e+01,
-9.46430024645381852000e-16, 2.14748364800000000000e+09,
};
const double _TBL_jlog_p1[] = {
 1.00000000000000000000e+00, 0.00000000000000000000e+00, 0.00000000000000000000e+00,
 1.00000000000000000000e+00, 2.32830643653869628906e-10, 2.21807097779181958686e+01,
 5.40328001624383298976e-14, 4.29496729600000000000e+09, 5.42101086242752217004e-20,
 4.43614195558363917371e+01, 1.08065600324876659795e-13, 1.84467440737095516160e+19,
 1.26217744835361888866e-29, 6.65421293337547012925e+01, 4.84115627656989599535e-14,
 7.92281625142643375935e+28, 2.93873587705571876992e-39, 8.87228391116728971610e+01,
 1.02444362928137289851e-13, 3.40282366920938463463e+38, 6.84227765783602085412e-49,
 1.10903548889591206716e+02, 4.27903253689595900094e-14, 1.46150163733090291820e+48,
 1.59309191113245227703e-58, 1.33084258667509402585e+02, 9.68231255313979199071e-14,
 6.27710173538668076384e+57, 3.70920615068742138573e-68, 1.55264968445427712140e+02,
 3.71690879722202200653e-14, 2.69599466671506397947e+67, 8.63616855509444462539e-78,
 1.77445678223345908009e+02, 9.12018881346585499629e-14, 1.15792089237316195424e+77,
 2.01076468338594879615e-87, 1.99626388001264217564e+02, 3.15478505754808501212e-14,
 4.97323236409786642155e+86, 4.68167635469219832716e-97, 2.21807097779182413433e+02,
 8.55806507379191800188e-14, 2.13598703592091008240e+96, 1.09003771904208658430e-106,
 2.43987807557100722988e+02, 2.59266131787414801771e-14, 9.17399446396028604644e+105,
 2.53794183731564922327e-116, 2.66168517335018918857e+02, 7.99594133411798100747e-14,
 3.94020061963944792123e+115, 5.90910631538287089969e-126, 2.88349227112937228412e+02,
 2.03053757820021102330e-14, 1.69230328010303641332e+125, 1.37582102682973977637e-135,
 3.10529936890855424281e+02, 7.43381759444404401306e-14, 7.26838724295606890549e+134,
 3.20333295229296147909e-145, 3.32710646668773733836e+02, 1.46841383852627402889e-14,
 3.12174855031599223138e+144, 7.45834073120020674329e-155, 3.54891356446691929705e+02,
 6.87169385477010701865e-14, 1.34078079299425970996e+154, 1.73653027303521678394e-164,
 3.77072066224610239260e+02, 9.06290098852336876704e-15, 5.75860965701529137000e+163,
 4.04317461195219490663e-174, 3.99252776002528435129e+02, 6.30957011509617002424e-14,
 2.47330401473104534061e+173, 9.41374947305814108492e-184, 4.21433485780446744684e+02,
 3.44166359178399882294e-15, 1.06227598563353419738e+183, 2.19180934900840303975e-193,
 4.43614195558364940553e+02, 5.74744637542223302983e-14, 4.56244061762219521864e+192,
 5.10320381496195457818e-203, 4.65794905336283250108e+02,-2.17957380495537112117e-15,
 1.95955332426293697478e+202, 1.18818222893447488970e-212, 4.87975615114201445977e+02,
 5.18532263574829603542e-14, 8.41621744247739761159e+211, 2.76645233140903266542e-222,
 5.10156324892119755532e+02,-7.80081120169474106528e-15, 3.61473786714651839609e+221,
 6.44114876959713330823e-232, 5.32337034670037951400e+02, 4.62319889607435904101e-14,
 1.55251809230070893515e+231, 1.49969681389563095482e-241, 5.54517744447956260956e+02,
-1.34220485984341110094e-14, 6.66801443287985427408e+240, 3.49175374464977289275e-251,
 5.76698454225874456824e+02, 4.06107515640042204660e-14, 2.86389039184749612044e+250,
 8.12987271847616157668e-261, 5.98879164003792766380e+02,-1.90432859951734825312e-14,
 1.23003155723136208568e+260, 1.89288349786683953756e-270, 6.21059873781710962248e+02,
 3.49895141672648505219e-14, 5.28294531135665246352e+269, 4.40721283170124408220e-280,
 6.43240583559629271804e+02,-2.46645233919128524753e-14, 2.26900773388333597229e+279,
 1.02613420032459406233e-289, 6.65421293337547467672e+02, 2.93682767705254805778e-14,
 9.74531401139999908035e+288, 2.38915486336824033021e-299, 6.87602003115465777228e+02,
-3.02857607886522192640e-14, 4.18558049682135672245e+298,
};
const double _TBL_jlog_n2[] = {
 1.00000000000000000000e+00, 0.00000000000000000000e+00, 0.00000000000000000000e+00,
 1.00000000000000000000e+00, 2.00000000000000000000e+00,-6.93147180559890330187e-01,
-5.49792301870837115524e-14, 5.00000000000000000000e-01, 4.00000000000000000000e+00,
-1.38629436111978066037e+00,-1.09958460374167423105e-13, 2.50000000000000000000e-01,
 8.00000000000000000000e+00,-2.07944154167978467740e+00,-5.12508528396351049179e-14,
 1.25000000000000000000e-01, 1.60000000000000000000e+01,-2.77258872223967500759e+00,
-1.06230083026718816470e-13, 6.25000000000000000000e-02, 3.20000000000000000000e+01,
-3.46573590279967902461e+00,-4.75224754921864982833e-14, 3.12500000000000000000e-02,
 6.40000000000000000000e+01,-4.15888308335956935480e+00,-1.02501705679270209836e-13,
 1.56250000000000000000e-02, 1.28000000000000000000e+02,-4.85203026391957337182e+00,
-4.37940981447378916488e-14, 7.81250000000000000000e-03, 2.56000000000000000000e+02,
-5.54517744447946370201e+00,-9.87733283318216032012e-14, 3.90625000000000000000e-03,
 5.12000000000000000000e+02,-6.23832462503946771903e+00,-4.00657207972892850143e-14,
 1.95312500000000000000e-03, 1.02400000000000000000e+03,-6.93147180559935804922e+00,
-9.50449509843729965667e-14, 9.76562500000000000000e-04, 2.04800000000000000000e+03,
-7.62461898615936206625e+00,-3.63373434498406783797e-14, 4.88281250000000000000e-04,
 4.09600000000000000000e+03,-8.31776616671925239643e+00,-9.13165736369243899321e-14,
 2.44140625000000000000e-04, 8.19200000000000000000e+03,-9.01091334727925641346e+00,
-3.26089661023920717452e-14, 1.22070312500000000000e-04, 1.63840000000000000000e+04,
-9.70406052783914674364e+00,-8.75881962894757832976e-14, 6.10351562500000000000e-05,
 3.27680000000000000000e+04,-1.03972077083991507607e+01,-2.88805887549434651106e-14,
 3.05175781250000000000e-05, 6.55360000000000000000e+04,-1.10903548889590410909e+01,
-8.38598189420271766631e-14, 1.52587890625000000000e-05, 1.31072000000000000000e+05,
-1.17835020695190451079e+01,-2.51522114074948616315e-14, 7.62939453125000000000e-06,
 2.62144000000000000000e+05,-1.24766492500789354381e+01,-8.01314415945785700285e-14,
 3.81469726562500000000e-06, 5.24288000000000000000e+05,-1.31697964306389394551e+01,
-2.14238340600462549970e-14, 1.90734863281250000000e-06, 1.04857600000000000000e+06,
-1.38629436111988297853e+01,-7.64030642471299633940e-14, 9.53674316406250000000e-07,
 2.09715200000000000000e+06,-1.45560907917588338023e+01,-1.76954567125976483625e-14,
 4.76837158203125000000e-07, 4.19430400000000000000e+06,-1.52492379723187241325e+01,
-7.26746868996813567594e-14, 2.38418579101562500000e-07, 8.38860800000000000000e+06,
-1.59423851528787281495e+01,-1.39670793651490433057e-14, 1.19209289550781250000e-07,
 1.67772160000000000000e+07,-1.66355323334386184797e+01,-6.89463095522327501249e-14,
 5.96046447753906250000e-08, 3.35544320000000000000e+07,-1.73286795139986224967e+01,
-1.02387020177004366711e-14, 2.98023223876953125000e-08, 6.71088640000000000000e+07,
-1.80218266945585128269e+01,-6.52179322047841434904e-14, 1.49011611938476562500e-08,
 1.34217728000000000000e+08,-1.87149738751185168439e+01,-6.51032467025183082543e-15,
 7.45058059692382812500e-09, 2.68435456000000000000e+08,-1.94081210556784071741e+01,
-6.14895548573355368558e-14, 3.72529029846191406250e-09, 5.36870912000000000000e+08,
-2.01012682362384111912e+01,-2.78194732280322419089e-15, 1.86264514923095703125e-09,
 1.07374182400000000000e+09,-2.07944154167983015213e+01,-5.77611775098869302213e-14,
 9.31322574615478515625e-10, 2.14748364800000000000e+09,-2.14875625973583055384e+01,
 9.46430024645381852000e-16, 4.65661287307739257812e-10,
};
const double _TBL_jlog_n1[] = {
 1.00000000000000000000e+00, 0.00000000000000000000e+00, 0.00000000000000000000e+00,
 1.00000000000000000000e+00, 4.29496729600000000000e+09,-2.21807097779181958686e+01,
-5.40328001624383298976e-14, 2.32830643653869628906e-10, 1.84467440737095516160e+19,
-4.43614195558363917371e+01,-1.08065600324876659795e-13, 5.42101086242752217004e-20,
 7.92281625142643375935e+28,-6.65421293337547012925e+01,-4.84115627656989599535e-14,
 1.26217744835361888866e-29, 3.40282366920938463463e+38,-8.87228391116728971610e+01,
-1.02444362928137289851e-13, 2.93873587705571876992e-39, 1.46150163733090291820e+48,
-1.10903548889591206716e+02,-4.27903253689595900094e-14, 6.84227765783602085412e-49,
 6.27710173538668076384e+57,-1.33084258667509402585e+02,-9.68231255313979199071e-14,
 1.59309191113245227703e-58, 2.69599466671506397947e+67,-1.55264968445427712140e+02,
-3.71690879722202200653e-14, 3.70920615068742138573e-68, 1.15792089237316195424e+77,
-1.77445678223345908009e+02,-9.12018881346585499629e-14, 8.63616855509444462539e-78,
 4.97323236409786642155e+86,-1.99626388001264217564e+02,-3.15478505754808501212e-14,
 2.01076468338594879615e-87, 2.13598703592091008240e+96,-2.21807097779182413433e+02,
-8.55806507379191800188e-14, 4.68167635469219832716e-97, 9.17399446396028604644e+105,
-2.43987807557100722988e+02,-2.59266131787414801771e-14, 1.09003771904208658430e-106,
 3.94020061963944792123e+115,-2.66168517335018918857e+02,-7.99594133411798100747e-14,
 2.53794183731564922327e-116, 1.69230328010303641332e+125,-2.88349227112937228412e+02,
-2.03053757820021102330e-14, 5.90910631538287089969e-126, 7.26838724295606890549e+134,
-3.10529936890855424281e+02,-7.43381759444404401306e-14, 1.37582102682973977637e-135,
 3.12174855031599223138e+144,-3.32710646668773733836e+02,-1.46841383852627402889e-14,
 3.20333295229296147909e-145, 1.34078079299425970996e+154,-3.54891356446691929705e+02,
-6.87169385477010701865e-14, 7.45834073120020674329e-155, 5.75860965701529137000e+163,
-3.77072066224610239260e+02,-9.06290098852336876704e-15, 1.73653027303521678394e-164,
 2.47330401473104534061e+173,-3.99252776002528435129e+02,-6.30957011509617002424e-14,
 4.04317461195219490663e-174, 1.06227598563353419738e+183,-4.21433485780446744684e+02,
-3.44166359178399882294e-15, 9.41374947305814108492e-184, 4.56244061762219521864e+192,
-4.43614195558364940553e+02,-5.74744637542223302983e-14, 2.19180934900840303975e-193,
 1.95955332426293697478e+202,-4.65794905336283250108e+02, 2.17957380495537112117e-15,
 5.10320381496195457818e-203, 8.41621744247739761159e+211,-4.87975615114201445977e+02,
-5.18532263574829603542e-14, 1.18818222893447488970e-212, 3.61473786714651839609e+221,
-5.10156324892119755532e+02, 7.80081120169474106528e-15, 2.76645233140903266542e-222,
 1.55251809230070893515e+231,-5.32337034670037951400e+02,-4.62319889607435904101e-14,
 6.44114876959713330823e-232, 6.66801443287985427408e+240,-5.54517744447956260956e+02,
 1.34220485984341110094e-14, 1.49969681389563095482e-241, 2.86389039184749612044e+250,
-5.76698454225874456824e+02,-4.06107515640042204660e-14, 3.49175374464977289275e-251,
 1.23003155723136208568e+260,-5.98879164003792766380e+02, 1.90432859951734825312e-14,
 8.12987271847616157668e-261, 5.28294531135665246352e+269,-6.21059873781710962248e+02,
-3.49895141672648505219e-14, 1.89288349786683953756e-270, 2.26900773388333597229e+279,
-6.43240583559629271804e+02, 2.46645233919128524753e-14, 4.40721283170124408220e-280,
 9.74531401139999908035e+288,-6.65421293337547467672e+02,-2.93682767705254805778e-14,
 1.02613420032459406233e-289, 4.18558049682135672245e+298,-6.87602003115465777228e+02,
 3.02857607886522192640e-14, 2.38915486336824033021e-299, 9.74531401139999908035e+288,
-7.09782712893383973096e+02,-2.37470393737861074782e-14, 1.02613420032459406233e-289,
 4.18558049682135672245e+298,-7.31963422671302282652e+02, 3.59069981853915892081e-14,
 2.38915486336824033021e-299,
};

#define HI(x) *(int *) &x
#define LO(x) *(1 + (int *) &x)

static const double pg[] = {
/* HALFN= */-0.5,
    /* log((1-v)/(1+v)) ~ 2s + (B1~2/3)s^3 + (B2~2/5)s^5
     * where  |s| < eps+2**-8, accuracy 2**-54 */
/* B1 = */     6.66666666635156294315870287075892538030194089695e-0001,
/* B2 = */     4.00006365019244575372057638905621724693243663268e-0001,
    /* log(1+v) ~ v - 0.5v*v + v**3 * (P0+v*(P1+...v*P7)),
     * where -2**-5 <= v < 2**-4 */
/* P0 = */   3.333333333334637790472065599997758646679e-0001,
/* P1 = */  -2.499999999994901940082091169112973250636e-0001,
/* P2 = */   1.999999993155754745200823224715488765236e-0001,
/* P3 = */  -1.666666608342163007934626596255065013041e-0001,
/* P4 = */   1.428581602580156830502063380923372556513e-0001,
/* P5 = */  -1.250187234703896538644519426046236477061e-0001,
/* P6 = */   1.107261912277397395930805862196691829590e-0001,
/* P7 = */  -8.755007345249893821202869541556059506802e-0002,
    /* Other constant */
/* ZERO       = */ 0.0,
/* TWO64      = */ 1.84467440737095516160e+19,  /* 2**64 */
/* ONE        = */ 1.0,
/* TWO        = */ 2.0,
};

#define HALFN           pg[0]
#define B1              pg[1]
#define B2              pg[2]
#define P0              pg[3]
#define P1              pg[4]
#define P2              pg[5]
#define P3              pg[6]
#define P4              pg[7]
#define P5              pg[8]
#define P6              pg[9]
#define P7              pg[10]
#define ZERO            pg[11]
#define TWO64           pg[12]
#define ONE             pg[13]
#define TWO             pg[14]
/* INDENT ON */

double
__log(double x) {
        double v, w, z;
        const double *tb, *T1, *T2;
        int ii, k, hx, lx;

        hx = HI(x);
        lx = LO(x);
        ii = hx >> 12;
        /* -2**-5 <= x-1 < 2**-4 */
        if (((ii + 0x10) >> 5) == 0x1ff8) {
                v = x - ONE;
                if (((hx - 0x3ff00000) | lx) == 0)
                        return ZERO;
                if (((hx + 0x20) >> 6) == 0xffc000) {   /* |x-1|<2**-15 */
                        w = (v * v) * (HALFN + P0 * v) + ((v * v) * (v * v)) *
                                P1;
                        return v + w;
                }
                else {
                        w = ((v * v) * (HALFN + P0 * v) +
                                ((v * v) * (v * v)) * (P1 + P2 * v)) +
                                ((v * v) * ((v * v) * (v * v))) *
                                (((P3 + P4 * v) + (v * v) * (P5 + P6 * v)) +
                                ((v * v) * (v * v)) * P7);
                        return v + w;
                }
        }
        /* 2**-7 <= x < 2**8 */
        k = hx >> 20;
        if (((k - 0x3f8) | (0x407 - k)) >= 0) {
                tb = _TBL_xlog + (((hx - 0x3f800000) >> 14) << 1);
                z = (x - *tb) / (x + *tb);
                return *(tb + 1) + (((z + z) + (z * z) * (z * B1)) +
                        ((z * z) * ((z * z) * (z * B2))));
        }
        /* subnormal,0,negative,inf,nan */
        if ((hx + 0x100000) < 0x200000) {
                if (k >= 0x7ff || x != x) {
                        return x + x;   /* x is +inf/+-nan */
                }
                if (((hx << 1) | lx) == 0)
                        return __kernel_standard (x, x, 16);  /* log(+-0) is -inf */
                if (hx < 0)
                        return __kernel_standard (x, x, 17);  /* log(-#) is NaN */
                /* now x must be +subnormal */
                x *= TWO64;
                hx = HI(x);
                k = 1087 - (hx >> 20);  /* 1023 <= k <= 1076 */
                T1 = _TBL_jlog_n1 + ((k >> 5) << 2);
                T2 = _TBL_jlog_n2 + ((k & 31) << 2);
                v = x * (*T1);
                if (k == 1023)
                        w = TWO64 * (*(T2+3));
                else
                        w = *(T2 + 3);
        }
        else {
                if (k < 0x3ff) {
                        k = 1023 - k;
                        T1 = _TBL_jlog_n1 + ((k >> 5) << 2);
                        T2 = _TBL_jlog_n2 + ((k & 31) << 2);
                        v = x * (*T1);
                        w = *(T2 + 3);
                }
                else {
                        k -= 1023;
                        T1 = _TBL_jlog_p1 + ((k >> 5) << 2);
                        T2 = _TBL_jlog_p2 + ((k & 31) << 2);
                        v = x * (*T1);
                        w = *(T2 + 3);
                }
        }
        /* otherwise */
        tb = _TBL_xlog + ((((hx & (~0xfff00000)) >> 14) | 0x1c0) << 1);
        w *= *tb;
        z = (v - w) / (v + w);
        return ((*(T1 + 1) + *(T2 + 1)) + *(tb + 1)) + ((((*(T1 + 2) +
                *(T2 + 2)) + (z + z)) + ((B1 * z) * (z * z))) +
                (z * z) * ((B2 * z) * (z * z)));
}

strong_alias (__log, __log_finite)
weak_alias (__log, log)
weak_alias (__log, __ieee754_log)
