/*
 * Copyright 08/20/03 Sun Microsystems, Inc.  All Rights Reserved.
 */

/*
 * fast pow return x**y
 *
 * Method:
 *
 *      x**y = exp(y*log(x)) = 2**(y*log2(x))
 *
 * Quick algorithm
 * ---------------
 *   Let
 *	x = 2**n * (1+f), where	sqrt(0.5)<f<sqrt(2), or  1/sqrt2-1<f<sqrt2-1
 *   then
 *	log2(x) = n + log2(1+f)
 *   Let
 *	X=1+f 			... range of X is [0x3fe6a09e - 0x3ff6a09e]
 *   Let Xi = X rounded to 8 bits
 *	Xi=(X+0x800)&0xfffff000 ... range of Xi is [0x3fe6a000 -- 0x3ff6a000]
 *   where
 *	i = (Xi-0x3fe6a000)>>12 ... range of i is 0,...0x100 (0,...,256)
 *   then
 * (1)	log2(x) = n + log2(Xi) + log2(X/Xi)
 *
 *   To compute (1), we break log2(Xi) into log2_XH+log2_XL, where
 *	log2_XH =(log2(Xi) + s*2**36)-s*2**36	... 16 bits after binary point
 *	log2_XL = log2(Xi) - log2_XH		... always less than 2**-16
 *   where s = sign(log2(Xi)).
 *   Note that Xi = 0x3fe6a000 + (i<<12)
 *   For log2(X/Xi), let v = (X-Xi)*(1/Xi), then  |v|<=2**-9 and
 *	log2(X/Xi) = log2(1+v) = w = v*(C0+v*(C1+...C5)...) where the polynomial
 *   approximation has relative error less than 2**-62.26 in absolute value.
 *
 *   A picture below show the lenght and size of the crucial elements
 *
 *		n:	[--].			... 11 bits for integer n
 *		log2_XH	    [-----]		... 16 bits (after binary pt.)
 *		log2_XL		  [-----...]  	... less than 2**-16
 *		w=log2(X/Xi)	[------...]   	... less than 2**-8
 *
 *   Thus, let 	nx = n+log2_XH, wx = log2_XL+w,  then nx has at most 27
 *   significant bits, |wx|<= 2**-8, and
 * (2)	log2(x) = nx + wx
 *
 *   The next step is to compute
 * (3)	y*log2(X) = k + (j/256) + r 	... j,k integer, |r|<=1/512.
 *   First break y = yH+yL, where yH = y rounded to 26 bits. Then
 *   compute z = zH+zL = y*log2(x) by
 *	zH= yH*nx,
 *	zL= yL*nx+y*wx
 *	z = zH+zL.
 *   To determine j,k, let m= z*256+0.5*sign(z) (z*256 rounded to an integer),
 *   then
 *	k=m>>8
 *	j=m&0xff
 *   and the remainder
 *	r = (zH-m*(1/256.0))+zL
 *
 *   From (3) we have
 * (4)	x**y = 2**k * 2**(j/256) * 2**(r),
 *   The computation of 2**(j/256) is by table lookup
 *	2**(j/256) = exp2_zH[j]+exp2_zL[j]
 *   where
 *	exp2_zH[j] = 2**(j/256) rounded, and
 *	exp2_zL[j] = 2**(j/256)-exp2_zH[j]
 *   The computation of 2**r is by 1 + (2**r - 1), where
 *	2**r - 1 = p = r*(D1+r*(D2+r*(D3+r*D4)))	(accuracy 2**-60)
 *
 *   Finally (4) is computed by
 *	2**k * (exp2_zH[j] + (exp2_zL[j]+exp2_zH[j]*p))
 *
 *   Note 1:
 *      This scheme guarantees less than 1ulp accuracy if (j/256+r) have 3 extra
 * 	bits accuracy. Since we are using an 8-bit table lookup Xi, log2(x)
 *	has about log2(n)+8 extra bits (assume n!=0). Therefore, k can go up
 *	to log2(n)+5 bits, or k < 32n. Since y*log2(X)~y*n, we have |y|<32.
 *	In other word, as long as |y|<32 and n!=0, one can use the Quick
 *	algorithm safely. Also, when n>=2**6, it is also safe to use the
 *	Quick algorithm, for if |y|>=32, y*n >= 2**11 and the result will
 *	over/underflow. Thus the Quick algorithm applies to
 *		x in  [0,0.707] U [1.42,oo) and y in [0,32).
 *	This range will cover a majority of data in current spec program,
 *	including DODUC.
 *
 *   Note 2:
 *	When |n|<2**6 and |y|>=32, 6 more bits is needed in log2(X).
 *	(A quick way to compute |n|: i=(n>>31), |n|=(n^i)-i.)
 *	That is,
 *	log2(X) will need 14 extra bit so that when k reaches it maximum 11
 *	bit, we still have 3 bits left for the r. Here is a way to add in 6
 *	more bits. Let Y  = X*(1/Xi). Since both X and Xi have the same
 *	exponent, we may assume that they are normalized. Xi is
 *	obtained by rounding X to 8 bits; thus we have X = Xi + eps,
 *	where -2**-9<=eps< 2**-9.  Thus X/Xi = (Xi+eps)/Xi
 *	= 1 + eps/Xi = 1 + eps'. Since Xi>=1, we have -2**-9 <= eps < 2**-9.
 *	Therefore, Y is between
 *		0.1111111110000000000... and 1.0000000001111111111...
 *	or Hi(Y) rounded is between
 *		0x3feff000, 0x3ff00800
 *	Let
 *		Yj = (Y+0x20)&0xffffffc0,	...rounded to 14 bits
 *		j  = (Yi-0x3feff000)>>6		...j=0,...,96
 *	Then for j=0,...,96
 *		Yj = 0x3feff000 + (i<<6) 	...[0x3feff000,0x3ff007c0]
 *	We have
 *		log2(X) = log2(Xi) + log2(Yi) + log2(X/Xi/Yj)
 *	Here
 *		log2(X/Xi/Yj) = log2(X/(Xi*Yj))
 *			      = log2(1+u)
 *			      = u*(G0+u*(G1+u*(G2+u*(G3+u*G4))))
 *	where u = (X-Xi*Yj)*(1/Xi)*(1/Yj), |u|<=2**-15. The approximation
 *	accuracy is 2**-80.
 *	For the value of log2(Yj), we break it into
 *		log2_YH[j] + log2_YL[j] where log2_YH[j] is 16 bits after binary
 *	That is (s= sign(log2(Yj)))
 *		log2_YH[j] = (log2(Yj)+s*2**36)-s*2**36
 *		log2_YL[j] = log2(Yj) - log2_YL[j]
 *
 *	Step (2) of the Quick algorithm is then modified to be
 *		nx = n + log2_XH[i]+log2_YH[j]
 *		wx = log2_XL[j]+log2_YL[j]+log2(1+u)
 *	The rest is the same as the Quick algorithm.
 *
 *   Note 3: n=0.
 *      Case 1: -2**-8 <= x-1 < 2**-7
 *	------
 *	If (Hi(x)+0x2000)&0x7fffc000 == 0x3ff00000, then we have
 *		-2**-8 <= x-1 < 2**-7
 *	In this case, we directly use the following method to compute
 *	log(x): let r = x-1,
 *		log(1+r) = r - 0.5*r*r + r*r*r*Poly(r),
 *	where
 *		Poly(r) = P0+r*(P1+...r*P5)
 *   	with precision = 2**(-69)
 *
 *	If one compute r-0.5*r*r carefully, then the third term of log(1+r)
 *	will have 14 bits shift right and thus guarantee 14 bits extra accuracy.
 *	Let
 *		rH = r rounded to 26 bits,
 *		rL = r-rH
 *		r2 = r*r
 *		t  = r*Poly(r)
 *		w  = r+r2*(t-0.5);
 *		wH = w rounded to 26 bits
 *		wL = (((r-wH)-(0.5*rH*rH))+r2*t)-rL*(rH+0.5*rL)
 *	then
 *		log(1+r) = wH + wL
 *
 *      Case 2 Otherwise
 *	------
 *	Use three 7 bits tables: let
 *		Ai = (X+0x1000)&0x7fffe000	... rounded to 7 bits
 *		i  = (Ai-0x3fe6a000)>>13 	... i = 0,...,128
 *		B  =  X*(1/Ai)			... Ai=0x3fe6a000+(i<<13)
 *		Bj = (B+0x20)&0x7fffffc0	... rounded to 14 bits
 *		j  = (Bj-0x3fefe000)>>6		... j = 0,...,192
 *		C  =  B*(1/Bj)			... Bj=0x3fefe000+(i<<6)
 *		Lo(Ck) = (Lo(Ck)+0x40000000)&0x80000000
 *		TEMP = (((unsigned)Lo(C))>>30)+1
 *		Hi(Ck) = Hi(C)+(TEMP>>2) 	... add carry
 *		Lo(Ck) = (TEMP<<30)&0x80000000
 *		k  = ((Ck-0x3fefffc0)<<1)+((Lo(Ck)>>31)&1)... k = 0,...,192
 *						... Ck=0x3fefffc0+(k>>1)
 *						... Lo(Ck)= (K&1)<<30
 *	then
 *		log(x)=log(Ai)+log(Bj)+log(Ck)+log(X/Ai/Bj/Ck)
 *	Here we choose the High part of the table value as follows
 *	so that the sum would be exact.
 *		log_AH    .[-----------]	... 26 bits (after binary pt.)
 *		log_BH    .   [--------]	... 19 bits
 *		log_CH    .      [-----]	... 12 bits
 *	That is,
 *	log_AH[i] = (log(Ai)+s*2**26)-s*2**26, log_AL[i]=log(Ai)-log_AiH[i]
 *	log_BH[j] = (log(Bj)+s*2**26)-s*2**26, log_BL[j]=log(Bj)-log_BjH[j]
 *	log_CH[k] = (log(Ck)+s*2**26)-s*2**26, log_CL[k]=log(Ck)-log_CkH[k]
 *	where s is the sign of log(#).
 *	Then
 *		v  = (X-Ai*Bj*Ck)*(1/Ai)*(1/Bj)*(1/Ck)	... |v|<=2**-22
 *		wH = log_AH[i]+log_BH[j]+log_CH[k]
 *		wL = (log_AL[i]+log_BL[j]+log_CL[k])+log(1+v)
 *	where
 *		log(1+v)= v*(1.0+v*(-0.5+v*Q3))
 *	with Precision   =  2**(-68.00)
 *	Then
 *		log(1+r) = wH + wL
 *
 *      Now we proceed to compute x**y by exp(y*log(x)) as follow:
 *
 *		y = yH+yL, where yH = y rounded to 26 bits
 *		zH = yH*wH
 *		zL = yL*wH+y*wL
 *		z  = zH+zL
 *		m  = z*(256/ln2)+0.5*sign(z)
 *		k  = m>>8
 *		i  = m&0xff
 *		ln2_256H = ln2/256 chopped to 32 bits
 *						...so that m*ln2_256H exact
 *		ln2_256L = ln2/256 - ln2_256H
 *		r = (zH-m*ln2_256H)+(zL-m*ln2_256L)
 *	Now z = k*ln2 + (i/256)*ln2 + r, with |r|<0.5*(ln2/256)=1.3538..e-3
 *	We have
 *		exp(z) = 2**k * (2**(1/256) * (1+p)
 *	where
 *		p = exp(r)-1 = r*(1.0+r*(B1+r*(B2+r*B3));
 *	with precision = 2**-57.45
 *	Finally, we compute exp(z) as follow
 *		2**k * (exp2_zH[j] + (exp2_zL[j]+exp2_zH[j]*p))
 *
 * Special cases:
 * --------------
 *	1.  (anything) ** 0  is 1
 *	1'. 1 ** (anything)  is 1	(C99; 1 ** +-INF/NAN used to be NAN)
 *	2.  (anything) ** 1  is itself
 *	3.  (anything) ** NAN is NAN
 *	4.  NAN ** (anything except 0) is NAN
 *	5.  +-(|x| > 1) **  +INF is +INF
 *	6.  +-(|x| > 1) **  -INF is +0
 *	7.  +-(|x| < 1) **  +INF is +0
 *	8.  +-(|x| < 1) **  -INF is +INF
 *	9.  -1          ** +-INF is 1	(C99; -1 ** +-INF used to be NAN)
 *	10. +0 ** (+anything except 0, NAN)               is +0
 *	11. -0 ** (+anything except 0, NAN, odd integer)  is +0
 *	12. +0 ** (-anything except 0, NAN)               is +INF
 *	13. -0 ** (-anything except 0, NAN, odd integer)  is +INF
 *	14. -0 ** (odd integer)  = -(+0 ** (odd integer)), hence
 *	    -0 ** (+odd integer) = -0
 *	    -0 ** (-odd integer) = -INF
 *	15. +INF ** (+anything except 0,NAN) is +INF
 *	16. +INF ** (-anything except 0,NAN) is +0
 *	17. -INF ** (anything)  = -0 ** (-anything), hence
 *	    -INF ** (+anything except 0,NaN)  = +INF
 *	    -INF ** (-anything except 0,NaN,odd integer)  = +0
 *	    -INF ** (+odd integer) = -INF
 *	    -INF ** (-odd integer) = -0
 *	18. (-anything) ** (integer) is ((-1)**(integer)) * (+anything**integer)
 *	19. (-anything except 0 and inf) ** (non-integer) is NAN
 *
 * Accuracy:
 *	pow(x,y) returns x**y nearly rounded. In particular
 *			pow(integer,integer)
 *	always returns the correct integer provided it is
 *	representable.
 */
/* Copyright (C) 2011-2014 Free Software Foundation, Inc.
   This file is part of the GNU C Library.
   Contributed by Ulrich Drepper <drepper@gmail.com>, 2011.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <http://www.gnu.org/licenses/>.  */

#include <math.h>
#include <math_private.h>
#include <math-svid-compat.h>

/* Total size: 3124 double = 6k words = 12k bytes 
 * _TBL_log2_X: (size 772 double)
 * 	log2(Xi)=log2_X[3i]+log2_X[3i+1], where
 * 	Xi=0x3fe6a000+(i<<12), i=0,...,257, and
 * 	1/(Xi)=log2_X[3i+2] 
 *
 * _TBL_log2_Y: (size 292 double)
 * 	log2(Yi)=log2_Y[3i]+log2_Y[3i+1], where
 * 	Yi=0x3feff000+(i<<6), i=0,...,97, and
 * 	1/(Yi)=log2_Y[3i+2]
 *
 * _TBL_log_A: (size 388 double)
 *	log(Ai)=log_A[3i]+log_A[3i+1],Ai=0x3fe6a000+(i<<13),i=0--129
 *	log_A[3i+2]=1/Ai 
 *
 * _TBL_log_B: (size 580 double)
 *	log(Bi)=log_B[3i]+log_B[3i+1],Bi=0x3fefffc0+(i<<6),i=0--193
 *	log_B[3i+2]=1/Bi
 *
 * _TBL_log_C: (size 580 double)
 *	log(Ci)=log_C[3i]+log_C[3i+1],Ci=0x3fefffc0+(i>>1),Lo(Ci)=(i<<31),i=0--193
 *	log_C[3i+2]=1/Ci
 */

const double _TBL_log2_X[] = {
-5.00152587890625000e-01,-1.52502616963946643e-06, 1.41436464088397784e+00,
-4.96170043945312500e-01,-4.21805893680614711e-06, 1.41046831955922869e+00,
-4.92202758789062500e-01,-2.60101224123216114e-06, 1.40659340659340670e+00,
-4.88250732421875000e-01, 3.38618925457773413e-06, 1.40273972602739727e+00,
-4.84298706054687500e-01,-1.45566127004207403e-06, 1.39890710382513661e+00,
-4.80361938476562500e-01,-1.80868022472966339e-06, 1.39509536784741139e+00,
-4.76440429687500000e-01, 2.38574451287229399e-06, 1.39130434782608692e+00,
-4.72518920898437500e-01,-4.07304116644392599e-06, 1.38753387533875339e+00,
-4.68612670898437500e-01,-5.86858525037027165e-06, 1.38378378378378386e+00,
-4.64721679687500000e-01,-2.94369169671327694e-06, 1.38005390835579522e+00,
-4.60845947265625000e-01, 4.75837365639025962e-06, 1.37634408602150549e+00,
-4.56970214843750000e-01, 2.03509898777703545e-06, 1.37265415549597858e+00,
-4.53109741210937500e-01, 4.20109857416445358e-06, 1.36898395721925126e+00,
-4.49249267578125000e-01,-3.94703863177493490e-06, 1.36533333333333329e+00,
-4.45404052734375000e-01,-7.09558798762782932e-06, 1.36170212765957444e+00,
-4.41574096679687500e-01,-5.19005164821888307e-06, 1.35809018567639250e+00,
-4.37759399414062500e-01, 1.82363513515180320e-06, 1.35449735449735442e+00,
-4.33944702148437500e-01,-1.25968047080064569e-06, 1.35092348284960417e+00,
-4.30145263671875000e-01, 8.72002822841663863e-07, 1.34736842105263155e+00,
-4.26345825195312500e-01,-6.98731136546526634e-06, 1.34383202099737531e+00,
-4.22576904296875000e-01, 5.73233262368925826e-06, 1.34031413612565453e+00,
-4.18792724609375000e-01,-6.69346566791516280e-06, 1.33681462140992169e+00,
-4.15039062500000000e-01, 1.56322115618145373e-06, 1.33333333333333326e+00,
-4.11285400390625000e-01, 3.59728887115116507e-08, 1.32987012987012987e+00,
-4.07546997070312500e-01, 4.03433839291963722e-06, 1.32642487046632129e+00,
-4.03808593750000000e-01,-1.65010558969820834e-06, 1.32299741602067189e+00,
-4.00085449218750000e-01,-1.70859412232017369e-06, 1.31958762886597936e+00,
-3.96377563476562500e-01, 3.90846275443256377e-06, 1.31619537275064258e+00,
-3.92669677734375000e-01,-8.51601431027912818e-09, 1.31282051282051282e+00,
-3.88977050781250000e-01, 1.84808860228054820e-06, 1.30946291560102313e+00,
-3.85284423828125000e-01,-5.73205666678511555e-06, 1.30612244897959173e+00,
-3.81607055664062500e-01,-7.44207733103952881e-06, 1.30279898218829526e+00,
-3.77944946289062500e-01,-3.23425456123321604e-06, 1.29949238578680193e+00,
-3.74298095703125000e-01, 6.93876759027229729e-06, 1.29620253164556964e+00,
-3.70635986328125000e-01,-7.39359226538089296e-06, 1.29292929292929304e+00,
-3.67004394531250000e-01,-4.08325792214004607e-07, 1.28967254408060450e+00,
-3.63372802734375000e-01,-2.57672197616144228e-06, 1.28643216080402012e+00,
-3.59756469726562500e-01, 1.40594890828268933e-06, 1.28320802005012524e+00,
-3.56140136718750000e-01,-3.67350652530425923e-06, 1.28000000000000003e+00,
-3.52539062500000000e-01,-2.51104507970215380e-06, 1.27680798004987528e+00,
-3.48953247070312500e-01, 4.93824924111438312e-06, 1.27363184079601988e+00,
-3.45367431640625000e-01, 3.46016859236920294e-06, 1.27047146401985112e+00,
-3.41781616210937500e-01,-6.90103726776282798e-06, 1.26732673267326734e+00,
-3.38226318359375000e-01, 4.41613136207368509e-06, 1.26419753086419751e+00,
-3.34671020507812500e-01, 6.93769298872816210e-06, 1.26108374384236455e+00,
-3.31115722656250000e-01, 7.06922497038057123e-07, 1.25798525798525795e+00,
-3.27575683593750000e-01, 1.02556524558970773e-06, 1.25490196078431371e+00,
-3.24035644531250000e-01,-7.32252700117774795e-06, 1.25183374083129584e+00,
-3.20526123046875000e-01, 6.22255232104103587e-06, 1.24878048780487805e+00,
-3.17001342773437500e-01,-4.07354487960275024e-06, 1.24574209245742096e+00,
-3.13507080078125000e-01, 7.60726134338106070e-06, 1.24271844660194164e+00,
-3.09997558593750000e-01,-4.46998680463375044e-06, 1.23970944309927367e+00,
-3.06518554687500000e-01, 5.51218682523520103e-06, 1.23671497584541057e+00,
-3.03039550781250000e-01, 7.07701553710423005e-06, 1.23373493975903625e+00,
-2.99560546875000000e-01, 2.65016092160396791e-07, 1.23076923076923084e+00,
-2.96096801757812500e-01, 3.75202476112443471e-07, 1.22781774580335723e+00,
-2.92648315429687500e-01, 7.44751057024999274e-06, 1.22488038277511957e+00,
-2.89199829101562500e-01, 6.26280091409693119e-06, 1.22195704057279242e+00,
-2.85751342773437500e-01,-3.13956043986323387e-06, 1.21904761904761916e+00,
-2.82318115234375000e-01,-5.46169922892020124e-06, 1.21615201900237535e+00,
-2.78900146484375000e-01,-6.64808439865841086e-07, 1.21327014218009488e+00,
-2.75482177734375000e-01,-3.96914567526492186e-06, 1.21040189125295505e+00,
-2.72079467773437500e-01,-7.76633633207192236e-08, 1.20754716981132071e+00,
-2.68676757812500000e-01,-4.21116243589600481e-06, 1.20470588235294107e+00,
-2.65289306640625000e-01,-1.07313353675713950e-06, 1.20187793427230050e+00,
-2.61901855468750000e-01,-5.88491075961608596e-06, 1.19906323185011709e+00,
-2.58529663085937500e-01,-3.35051291555213535e-06, 1.19626168224299056e+00,
-2.55172729492187500e-01, 6.56699173309804952e-06, 1.19347319347319347e+00,
-2.51800537109375000e-01,-6.61330116471324520e-06, 1.19069767441860463e+00,
-2.48458862304687500e-01, 2.92139378563598654e-06, 1.18793503480278417e+00,
-2.45117187500000000e-01, 4.68966346854436119e-06, 1.18518518518518512e+00,
-2.41775512695312500e-01,-1.27257796259461268e-06, 1.18244803695150114e+00,
-2.38449096679687500e-01, 3.29124166816248113e-07, 1.17972350230414746e+00,
-2.35122680664062500e-01,-5.72859984684995623e-06, 1.17701149425287355e+00,
-2.31811523437500000e-01,-4.15178557364152060e-06, 1.17431192660550465e+00,
-2.28515625000000000e-01, 5.09450059836608758e-06, 1.17162471395881007e+00,
-2.25219726562500000e-01, 6.78616367341131769e-06, 1.16894977168949765e+00,
-2.21923828125000000e-01, 9.57660358184112076e-07, 1.16628701594533024e+00,
-2.18643188476562500e-01, 2.90200122210406976e-06, 1.16363636363636358e+00,
-2.15362548828125000e-01,-2.60561435442220851e-06, 1.16099773242630389e+00,
-2.12097167968750000e-01,-2.72639818431349106e-07, 1.15837104072398200e+00,
-2.08831787109375000e-01,-5.32433560669304128e-06, 1.15575620767494347e+00,
-2.05581665039062500e-01,-2.46861083153668809e-06, 1.15315315315315314e+00,
-2.02331542968750000e-01,-6.93117748989716522e-06, 1.15056179775280909e+00,
-1.99096679687500000e-01,-3.42039219526696706e-06, 1.14798206278026904e+00,
-1.95861816406250000e-01,-7.16241043224693463e-06, 1.14541387024608499e+00,
-1.92642211914062500e-01,-2.86602833339255777e-06, 1.14285714285714279e+00,
-1.89422607421875000e-01,-5.75783697805759173e-06, 1.14031180400890864e+00,
-1.86218261718750000e-01,-5.47064212941351875e-07, 1.13777777777777778e+00,
-1.83013916015625000e-01,-2.46072899405063472e-06, 1.13525498891352550e+00,
-1.79824829101562500e-01, 3.79151675020831846e-06, 1.13274336283185839e+00,
-1.76635742187500000e-01, 2.98223373508799665e-06, 1.13024282560706402e+00,
-1.73446655273437500e-01,-4.85743564751621798e-06, 1.12775330396475781e+00,
-1.70272827148437500e-01,-4.43776550388429054e-06, 1.12527472527472527e+00,
-1.67114257812500000e-01, 4.27197724167524689e-06, 1.12280701754385959e+00,
-1.63955688476562500e-01, 6.04353463220044027e-06, 1.12035010940919033e+00,
-1.60797119140625000e-01, 9.07237568938703705e-07, 1.11790393013100431e+00,
-1.57653808593750000e-01, 4.15200755795261519e-06, 1.11546840958605675e+00,
-1.54510498046875000e-01, 5.48991250220164486e-07, 1.11304347826086958e+00,
-1.51382446289062500e-01, 5.38671840047707634e-06, 1.11062906724511934e+00,
-1.48254394531250000e-01, 3.43594730754509469e-06, 1.10822510822510822e+00,
-1.45126342773437500e-01,-5.27396632611360052e-06, 1.10583153347732188e+00,
-1.42013549804687500e-01,-5.45506774037928013e-06, 1.10344827586206895e+00,
-1.38916015625000000e-01, 2.92162039373812979e-06, 1.10107526881720430e+00,
-1.35818481445312500e-01, 4.62609959273701846e-06, 1.09871244635193133e+00,
-1.32720947265625000e-01,-3.13024713086733662e-07, 1.09635974304068529e+00,
-1.29638671875000000e-01, 3.39145840452330409e-06, 1.09401709401709413e+00,
-1.26556396484375000e-01, 5.08999751540371195e-07, 1.09168443496801704e+00,
-1.23489379882812500e-01, 6.32644781222004085e-06, 1.08936170212765959e+00,
-1.20422363281250000e-01, 5.61289403318786337e-06, 1.08704883227176219e+00,
-1.17355346679687500e-01,-1.60395847124119267e-06, 1.08474576271186440e+00,
-1.14303588867187500e-01,-3.77934173049164922e-08, 1.08245243128964064e+00,
-1.11251831054687500e-01,-4.92004705339411915e-06, 1.08016877637130793e+00,
-1.08215332031250000e-01,-9.64750439810465857e-07, 1.07789473684210524e+00,
-1.05178833007812500e-01,-3.40368424398430378e-06, 1.07563025210084029e+00,
-1.02157592773437500e-01, 3.04877894904218829e-06, 1.07337526205450740e+00,
-9.91363525390625000e-02, 3.16051981105834240e-06, 1.07112970711297062e+00,
-9.61151123046875000e-02,-3.04195913225615699e-06, 1.06889352818371597e+00,
-9.31091308593750000e-02,-2.73532106470675937e-07, 1.06666666666666665e+00,
-9.01031494140625000e-02,-3.76681589555774519e-06, 1.06444906444906451e+00,
-8.71124267578125000e-02, 1.76298777411898379e-06, 1.06224066390041494e+00,
-8.41217041015625000e-02, 1.08293733566118995e-06, 1.06004140786749490e+00,
-8.11309814453125000e-02,-5.78128009298760084e-06, 1.05785123966942152e+00,
-7.81555175781250000e-02,-3.54534738497230330e-06, 1.05567010309278353e+00,
-7.51800537109375000e-02,-7.44268328159273135e-06, 1.05349794238683137e+00,
-7.22198486328125000e-02,-2.18928484533806051e-06, 1.05133470225872694e+00,
-6.92596435546875000e-02,-3.01888242622352776e-06, 1.04918032786885251e+00,
-6.63146972656250000e-02, 5.35221785873401711e-06, 1.04703476482617597e+00,
-6.33544921875000000e-02,-7.56880992943724537e-06, 1.04489795918367356e+00,
-6.04248046875000000e-02, 4.01900219306347727e-06, 1.04276985743380846e+00,
-5.74798583984375000e-02,-5.63626232262537972e-06, 1.04065040650406515e+00,
-5.45501708984375000e-02,-5.99272365097102613e-06, 1.03853955375253548e+00,
-5.16357421875000000e-02, 2.97377217765419022e-06, 1.03643724696356276e+00,
-4.87213134765625000e-02, 6.02844353446697721e-06, 1.03434343434343434e+00,
-4.58068847656250000e-02, 3.19515250020880589e-06, 1.03225806451612900e+00,
-4.28924560546875000e-02,-5.50238302633115100e-06, 1.03018108651911477e+00,
-3.99932861328125000e-02,-4.78179910656218640e-06, 1.02811244979919669e+00,
-3.71093750000000000e-02, 5.38033726053433890e-06, 1.02605210420841675e+00,
-3.42102050781250000e-02,-5.51025978795638863e-06, 1.02400000000000002e+00,
-3.13262939453125000e-02,-6.91285947909407528e-06, 1.02195608782435121e+00,
-2.84576416015625000e-02, 1.19555233449097707e-06, 1.01992031872509958e+00,
-2.55889892578125000e-02, 3.57906333959179509e-06, 1.01789264413518876e+00,
-2.27203369140625000e-02, 2.60413978970349421e-07, 1.01587301587301582e+00,
-1.98669433593750000e-02, 6.52099853208504220e-06, 1.01386138613861387e+00,
-1.70135498046875000e-02, 7.12449899762849315e-06, 1.01185770750988135e+00,
-1.41601562500000000e-02, 2.09325334050224699e-06, 1.00986193293885607e+00,
-1.13220214843750000e-02, 6.70825654085327993e-06, 1.00787401574803148e+00,
-8.48388671875000000e-03, 5.73279444529548115e-06, 1.00589390962671898e+00,
-5.64575195312500000e-03,-8.11188017062421888e-07, 1.00392156862745097e+00,
-2.82287597656250000e-03, 2.35691418383730618e-06, 1.00195694716242656e+00,
 0.00000000000000000e+00, 0.00000000000000000e+00, 1.00000000000000000e+00,
 5.63049316406250000e-03,-5.94397018439307963e-06, 9.96108949416342426e-01,
 1.12304687500000000e-02,-3.21332674587966207e-06, 9.92248062015503862e-01,
 1.68151855468750000e-02,-6.89786032111069960e-06, 9.88416988416988440e-01,
 2.23693847656250000e-02,-1.57173717049173277e-06, 9.84615384615384670e-01,
 2.79083251953125000e-02,-2.32862542801637267e-06, 9.80842911877394585e-01,
 3.34167480468750000e-02, 6.25349057527901746e-06, 9.77099236641221336e-01,
 3.89251708984375000e-02,-6.18160613514908134e-06, 9.73384030418250945e-01,
 4.43878173828125000e-02, 6.30197564093765331e-06, 9.69696969696969724e-01,
 4.98504638671875000e-02,-1.91441662597284900e-06, 9.66037735849056611e-01,
 5.52825927734375000e-02,-1.57272247898764508e-07, 9.62406015037593932e-01,
 6.06994628906250000e-02,-3.53120307106358167e-06, 9.58801498127340834e-01,
 6.60858154296875000e-02, 3.37502808493292939e-06, 9.55223880597014907e-01,
 7.14569091796875000e-02, 5.45337693664462716e-06, 9.51672862453531554e-01,
 7.68127441406250000e-02, 2.85291020589223136e-06, 9.48148148148148184e-01,
 8.21533203125000000e-02,-4.27895862843930208e-06, 9.44649446494464917e-01,
 8.74633789062500000e-02,-5.37655910591745897e-07, 9.41176470588235281e-01,
 9.27581787109375000e-02,-1.03779108505070741e-06, 9.37728937728937728e-01,
 9.80377197265625000e-02,-5.63676603578420397e-06, 9.34306569343065663e-01,
 1.03286743164062500e-01, 1.06524795945193993e-06, 9.30909090909090908e-01,
 1.08520507812500000e-01, 3.94896566905374730e-06, 9.27536231884057982e-01,
 1.13739013671875000e-01, 3.15237731332991495e-06, 9.24187725631768986e-01,
 1.18942260742187500e-01,-1.18801868006901021e-06, 9.20863309352518034e-01,
 1.24114990234375000e-01, 6.32159481257171335e-06, 9.17562724014336917e-01,
 1.29287719726562500e-01,-4.70278159604468760e-06, 9.14285714285714257e-01,
 1.34429931640625000e-01,-3.61141969890130483e-06, 9.11032028469750843e-01,
 1.39556884765625000e-01,-5.53236683144637559e-06, 9.07801418439716290e-01,
 1.44653320312500000e-01, 4.92251938232081637e-06, 9.04593639575971720e-01,
 1.49749755859375000e-01,-2.63635469293859323e-06, 9.01408450704225372e-01,
 1.54815673828125000e-01, 2.43522397902311749e-06, 8.98245614035087736e-01,
 1.59866333007812500e-01, 5.00377057691659579e-06, 8.95104895104895104e-01,
 1.64901733398437500e-01, 5.19327725030060793e-06, 8.91986062717770034e-01,
 1.69921875000000000e-01, 3.12644231236290746e-06, 8.88888888888888840e-01,
 1.74926757812500000e-01,-1.07531182118349179e-06, 8.85813148788927363e-01,
 1.79916381835937500e-01,-7.29182100303140996e-06, 8.82758620689655160e-01,
 1.84875488281250000e-01,-1.45372966138720038e-07, 8.79725085910652904e-01,
 1.89819335937500000e-01, 5.22294251722986396e-06, 8.76712328767123239e-01,
 1.94763183593750000e-01,-6.32917150212546979e-06, 8.73720136518771340e-01,
 1.99676513671875000e-01,-4.16883551060366182e-06, 8.70748299319727859e-01,
 2.04574584960937500e-01,-3.44071173389332207e-06, 8.67796610169491500e-01,
 2.09457397460937500e-01,-4.03183198771814182e-06, 8.64864864864864913e-01,
 2.14324951171875000e-01,-5.83037110919943923e-06, 8.61952861952861915e-01,
 2.19161987304687500e-01, 6.53315747407161164e-06, 8.59060402684563740e-01,
 2.23999023437500000e-01, 2.65076060503269063e-06, 8.56187290969899650e-01,
 2.28820800781250000e-01,-2.11028536912280550e-06, 8.53333333333333277e-01,
 2.33612060546875000e-01, 7.61621282704632600e-06, 8.50498338870431914e-01,
 2.38403320312500000e-01, 1.41901257890654292e-06, 8.47682119205298013e-01,
 2.43179321289062500e-01,-5.33781611158137425e-06, 8.44884488448844895e-01,
 2.47924804687500000e-01, 2.70875608549379316e-06, 8.42105263157894690e-01,
 2.52670288085937500e-01,-4.85563568887565717e-06, 8.39344262295081966e-01,
 2.57385253906250000e-01, 2.58878640177116146e-06, 8.36601307189542509e-01,
 2.62100219726562500e-01,-5.37435638311187486e-06, 8.33876221498371373e-01,
 2.66784667968750000e-01, 1.87272615136364138e-06, 8.31168831168831224e-01,
 2.71469116210937500e-01,-6.08830656293748557e-06, 8.28478964401294538e-01,
 2.76123046875000000e-01, 1.35839923755667648e-06, 8.25806451612903225e-01,
 2.80776977539062500e-01,-6.20740845995884589e-06, 8.23151125401929251e-01,
 2.85400390625000000e-01, 1.82823724834185057e-06, 8.20512820512820484e-01,
 2.90023803710937500e-01,-4.95677831916581214e-06, 8.17891373801916899e-01,
 2.94616699218750000e-01, 4.04967287700640964e-06, 8.15286624203821697e-01,
 2.99209594726562500e-01,-1.57633928368178014e-06, 8.12698412698412698e-01,
 3.03787231445312500e-01,-6.48326820957557288e-06, 8.10126582278481000e-01,
 3.08334350585937500e-01, 4.67955346975435117e-06, 8.07570977917981048e-01,
 3.12881469726562500e-01, 1.48555779286073456e-06, 8.05031446540880546e-01,
 3.17413330078125000e-01,-7.16313255623080866e-07, 8.02507836990595580e-01,
 3.21929931640625000e-01,-1.83675326265212961e-06, 8.00000000000000044e-01,
 3.26431274414062500e-01,-1.78729175937068162e-06, 7.97507788161993747e-01,
 3.30917358398437500e-01,-4.80283820520263885e-07, 7.95031055900621064e-01,
 3.35388183593750000e-01, 2.17110017490204758e-06, 7.92569659442724506e-01,
 3.39843750000000000e-01, 6.25288462472581492e-06, 7.90123456790123413e-01,
 3.44299316406250000e-01,-3.40849043314386260e-06, 7.87692307692307692e-01,
 3.48724365234375000e-01, 3.78899670255256338e-06, 7.85276073619631920e-01,
 3.53149414062500000e-01,-2.58856441746006729e-06, 7.82874617737003065e-01,
 3.57559204101562500e-01,-7.19948347880683345e-06, 7.80487804878048808e-01,
 3.61938476562500000e-01, 5.29717274147961291e-06, 7.78115501519756836e-01,
 3.66317749023437500e-01, 4.46522237828552348e-06, 7.75757575757575757e-01,
 3.70681762695312500e-01, 5.64411190519761043e-06, 7.73413897280966767e-01,
 3.75045776367187500e-01,-6.34502026274364013e-06, 7.71084337349397630e-01,
 3.79379272460937500e-01,-9.05389675355234681e-07, 7.68768768768768762e-01,
 3.83697509765625000e-01, 6.78270842722447099e-06, 7.66467065868263520e-01,
 3.88015747070312500e-01, 1.53827482228079957e-06, 7.64179104477611948e-01,
 3.92318725585937500e-01,-1.30280717721110426e-06, 7.61904761904761862e-01,
 3.96606445312500000e-01,-1.66413064153488771e-06, 7.59643916913946615e-01,
 4.00878906250000000e-01, 5.30032184320793582e-07, 7.57396449704141994e-01,
 4.05136108398437500e-01, 5.35473790638977219e-06, 7.55162241887905594e-01,
 4.09393310546875000e-01,-2.37440917324387541e-06, 7.52941176470588225e-01,
 4.13635253906250000e-01,-7.32488207753499453e-06, 7.50733137829912023e-01,
 4.17846679687500000e-01, 5.83519839785670062e-06, 7.48538011695906391e-01,
 4.22058105468750000e-01, 6.66070406232232583e-06, 7.46355685131195323e-01,
 4.26269531250000000e-01,-4.77654790206111623e-06, 7.44186046511627897e-01,
 4.30450439453125000e-01, 2.11221240640161832e-06, 7.42028985507246386e-01,
 4.34631347656250000e-01,-3.12001952536727876e-06, 7.39884393063583778e-01,
 4.38796997070312500e-01,-5.14449205159247637e-06, 7.37752161383285254e-01,
 4.42947387695312500e-01,-3.89184658419782640e-06, 7.35632183908045967e-01,
 4.47082519531250000e-01, 7.06678402230607631e-07, 7.33524355300859576e-01,
 4.51217651367187500e-01,-6.53953485869681743e-06, 7.31428571428571428e-01,
 4.55322265625000000e-01, 4.95467956070475825e-06, 7.29344729344729381e-01,
 4.59426879882812500e-01, 4.73875448475619873e-06, 7.27272727272727293e-01,
 4.63531494140625000e-01,-7.12086944470976998e-06, 7.25212464589235162e-01,
 4.67605590820312500e-01,-4.07373150597388521e-08, 7.23163841807909602e-01,
 4.71679687500000000e-01,-4.47310795559072306e-06, 7.21126760563380320e-01,
 4.75738525390625000e-01,-5.09442422724503540e-06, 7.19101123595505598e-01,
 4.79782104492187500e-01,-1.84046308780285026e-06, 7.17086834733893563e-01,
 4.83810424804687500e-01, 5.35245956892409164e-06, 7.15083798882681587e-01,
 4.87838745117187500e-01, 1.28870586384535077e-06, 7.13091922005571033e-01,
 4.91851806640625000e-01, 1.28968904971077785e-06, 7.11111111111111138e-01,
 4.95849609375000000e-01, 5.41751217098758717e-06, 7.09141274238227148e-01,
 4.99847412109375000e-01,-1.52502616963946643e-06, 7.07182320441988921e-01,
};

const double _TBL_log2_Y[] = {
-2.82287597656250000e-03, 2.35691418383730618e-06, 1.00195694716242656e+00,
-2.77709960937500000e-03, 6.93590797690835398e-07, 1.00192631096162676e+00,
-2.73132324218750000e-03,-9.71081384340371007e-07, 1.00189567663425660e+00,
-2.68554687500000000e-03,-2.63710227977754663e-06, 1.00186504418014488e+00,
-2.63977050781250000e-03,-4.30447180614949077e-06, 1.00183441359911951e+00,
-2.59399414062500000e-03,-5.97318988099256715e-06, 1.00180378489100863e+00,
-2.56347656250000000e-03, 7.61553264064929988e-06, 1.00177315805564038e+00,
-2.51770019531250000e-03, 5.94411771622462030e-06, 1.00174253309284333e+00,
-2.47192382812500000e-03, 4.27135449067434428e-06, 1.00171191000244564e+00,
-2.42614746093750000e-03, 2.59724304643186141e-06, 1.00168128878427543e+00,
-2.38037109375000000e-03, 9.21783465923001594e-07, 1.00165066943816106e+00,
-2.33459472656250000e-03,-7.55024168433964174e-07, 1.00162005196393089e+00,
-2.28881835937500000e-03,-2.43317977422832234e-06, 1.00158943636141329e+00,
-2.24304199218750000e-03,-4.11268326905691617e-06, 1.00155882263043683e+00,
-2.19726562500000000e-03,-5.79353457052414518e-06, 1.00152821077082943e+00,
-2.15148925781250000e-03,-7.47573359624196233e-06, 1.00149760078241989e+00,
-2.12097167968750000e-03, 6.09950879867012222e-06, 1.00146699266503658e+00,
-2.07519531250000000e-03, 4.41461457158504869e-06, 1.00143638641850807e+00,
-2.02941894531250000e-03, 2.72837286736820304e-06, 1.00140578204266251e+00,
-1.98364257812500000e-03, 1.04078376837742077e-06, 1.00137517953732846e+00,
-1.93786621093750000e-03,-6.48152643037013377e-07, 1.00134457890233475e+00,
-1.89208984375000000e-03,-2.33843628453236385e-06, 1.00131398013750950e+00,
-1.84631347656250000e-03,-4.03006707377344363e-06, 1.00128338324268173e+00,
-1.80053710937500000e-03,-5.72304492843261320e-06, 1.00125278821767960e+00,
-1.75476074218750000e-03,-7.41736976618978042e-06, 1.00122219506233190e+00,
-1.72424316406250000e-03, 6.14574755776760234e-06, 1.00119160377646743e+00,
-1.67846679687500000e-03, 4.44872900074453733e-06, 1.00116101435991456e+00,
-1.63269042968750000e-03, 2.75036370753848311e-06, 1.00113042681250186e+00,
-1.58691406250000000e-03, 1.05065176043935523e-06, 1.00109984113405837e+00,
-1.54113769531250000e-03,-6.50406758270473506e-07, 1.00106925732441265e+00,
-1.49536132812500000e-03,-2.35281176631617057e-06, 1.00103867538339331e+00,
-1.44958496093750000e-03,-4.05656318143044393e-06, 1.00100809531082935e+00,
-1.40380859375000000e-03,-5.76166092135354080e-06, 1.00097751710654936e+00,
-1.35803222656250000e-03,-7.46810490383324687e-06, 1.00094694077038215e+00,
-1.32751464843750000e-03, 6.08289401587511529e-06, 1.00091636630215652e+00,
-1.28173828125000000e-03, 4.37375779500868617e-06, 1.00088579370170128e+00,
-1.23596191406250000e-03, 2.66327557829707024e-06, 1.00085522296884544e+00,
-1.19018554687500000e-03, 9.51447447962338129e-07, 1.00082465410341781e+00,
-1.14440917968750000e-03,-7.61726513780973889e-07, 1.00079408710524698e+00,
-1.09863281250000000e-03,-2.47624622472586199e-06, 1.00076352197416241e+00,
-1.05285644531250000e-03,-4.19211160267285461e-06, 1.00073295870999268e+00,
-1.00708007812500000e-03,-5.90932256543001059e-06, 1.00070239731256683e+00,
-9.61303710937500000e-04,-7.62787903081292019e-06, 1.00067183778171387e+00,
-9.30786132812500000e-04, 5.91100814585530002e-06, 1.00064128011726261e+00,
-8.85009765625000000e-04, 4.18976092174400167e-06, 1.00061072431904230e+00,
-8.39233398437500000e-04, 2.46716844151501132e-06, 1.00058017038688196e+00,
-7.93457031250000000e-04, 7.43230787322628512e-07, 1.00054961832061062e+00,
-7.47680664062500000e-04,-9.82051958686372282e-07, 1.00051906812005731e+00,
-7.01904296875000000e-04,-2.70867971437274140e-06, 1.00048851978505127e+00,
-6.56127929687500000e-04,-4.43665239760475296e-06, 1.00045797331542152e+00,
-6.10351562500000000e-04,-6.16596992625820475e-06, 1.00042742871099710e+00,
-5.79833984375000000e-04, 7.36215684428358679e-06, 1.00039688597160747e+00,
-5.34057617187500000e-04, 5.63014987112978054e-06, 1.00036634509708144e+00,
-4.88281250000000000e-04, 3.89679829888201718e-06, 1.00033580608724848e+00,
-4.42504882812500000e-04, 2.16210220963441993e-06, 1.00030526894193783e+00,
-3.96728515625000000e-04, 4.26061685473590854e-07, 1.00027473366097874e+00,
-3.50952148437500000e-04,-1.31132319152138351e-06, 1.00024420024420024e+00,
-3.05175781250000000e-04,-3.05005233927893286e-06, 1.00021366869143180e+00,
-2.59399414062500000e-04,-4.79012567573500284e-06, 1.00018313900250289e+00,
-2.13623046875000000e-04,-6.53154311883305350e-06, 1.00015261117724252e+00,
-1.83105468750000000e-04, 6.98448447597594237e-06, 1.00012208521548041e+00,
-1.37329101562500000e-04, 5.24037906573349879e-06, 1.00009156111704556e+00,
-9.15527343750000000e-05, 3.49492979497361785e-06, 1.00006103888176767e+00,
-4.57763671875000000e-05, 1.74813674572279103e-06, 1.00003051850947600e+00,
 0.00000000000000000e+00, 0.00000000000000000e+00, 1.00000000000000000e+00,
 9.15527343750000000e-05,-3.50030425282309150e-06, 9.99938968568812925e-01,
 1.83105468750000000e-04,-7.00598230749396485e-06, 9.99877944586842426e-01,
 2.59399414062500000e-04, 4.74175555436897051e-06, 9.99816928052724707e-01,
 3.50952148437500000e-04, 1.22533186352723778e-06, 9.99755918965096413e-01,
 4.42504882812500000e-04,-2.29646366187768215e-06, 9.99694917322594412e-01,
 5.34057617187500000e-04,-5.82363036632431861e-06, 9.99633923123856016e-01,
 6.10351562500000000e-04, 5.90262146808881689e-06, 9.99572936367518761e-01,
 7.01904296875000000e-04, 2.36471437164325813e-06, 9.99511957052220623e-01,
 7.93457031250000000e-04,-1.17856193799938021e-06, 9.99450985176599804e-01,
 8.85009765625000000e-04,-4.72720680579738183e-06, 9.99390020739294838e-01,
 9.61303710937500000e-04, 6.97756948567110681e-06, 9.99329063738944812e-01,
 1.05285644531250000e-03, 3.41818946620810398e-06, 9.99268114174188815e-01,
 1.14440917968750000e-03,-1.46557147004180409e-07, 9.99207172043666492e-01,
 1.23596191406250000e-03,-3.71666969940331200e-06, 9.99146237346017818e-01,
 1.32751464843750000e-03,-7.29214753654660528e-06, 9.99085310079882882e-01,
 1.40380859375000000e-03, 4.38579905838890689e-06, 9.99024390243902438e-01,
 1.49536132812500000e-03, 7.99592614606501777e-07, 9.98963477836717240e-01,
 1.58691406250000000e-03,-2.79197715131020245e-06, 9.98902572856968707e-01,
 1.67846679687500000e-03,-6.38890958539722148e-06, 9.98841675303298149e-01,
 1.75476074218750000e-03, 5.26758502868983007e-06, 9.98780785174347763e-01,
 1.84631347656250000e-03, 1.65992921967576160e-06, 9.98719902468759524e-01,
 1.93786621093750000e-03,-1.95308729633415795e-06, 9.98659027185176185e-01,
 2.02941894531250000e-03,-5.57146386585417729e-06, 9.98598159322240497e-01,
 2.10571289062500000e-03, 6.06358922698197159e-06, 9.98537298878595769e-01,
 2.19726562500000000e-03, 2.43449451042109982e-06, 9.98476445852885641e-01,
 2.28881835937500000e-03,-1.19995829990940907e-06, 9.98415600243753754e-01,
 2.38037109375000000e-03,-4.83976855100156931e-06, 9.98354762049844635e-01,
 2.45666503906250000e-03, 6.77385347253323461e-06, 9.98293931269802592e-01,
 2.54821777343750000e-03, 3.12333029846428144e-06, 9.98233107902272598e-01,
 2.63977050781250000e-03,-5.32548358058463657e-07, 9.98172291945899848e-01,
 2.73132324218750000e-03,-4.19378184450431683e-06, 9.98111483399329869e-01,
 2.80761718750000000e-03, 7.39841955403815435e-06, 9.98050682261208522e-01,
};

const double _TBL_log_A[] = {
-3.46680417656898499e-01, 4.44316177003638645e-09, 1.41436464088397784e+00,
-3.41170758008956909e-01, 6.06189784417902794e-10, 1.40659340659340670e+00,
-3.35691288113594055e-01,-3.52454748001544154e-09, 1.39890710382513661e+00,
-3.30241680145263672e-01,-6.72531318440440745e-09, 1.39130434782608692e+00,
-3.24821621179580688e-01, 1.77834303210756060e-09, 1.38378378378378386e+00,
-3.19430768489837646e-01,-2.27652358211184022e-09, 1.37634408602150549e+00,
-3.14068824052810669e-01,-3.57216518208106619e-09, 1.36898395721925126e+00,
-3.08735474944114685e-01,-6.70549858462384857e-09, 1.36170212765957444e+00,
-3.03430423140525818e-01,-6.27939427817567446e-09, 1.35449735449735442e+00,
-2.98153370618820190e-01,-1.70025614088115068e-09, 1.34736842105263155e+00,
-2.92904019355773926e-01, 2.92284132329334299e-09, 1.34031413612565453e+00,
-2.87682071328163147e-01,-1.12361778046656267e-09, 1.33333333333333326e+00,
-2.82487258315086365e-01, 2.74040944126316770e-09, 1.32642487046632129e+00,
-2.77319282293319702e-01,-3.12291464165546584e-09, 1.31958762886597936e+00,
-2.72177889943122864e-01, 4.02730719048116591e-09, 1.31282051282051282e+00,
-2.67062783241271973e-01,-2.00777327363643722e-09, 1.30612244897959173e+00,
-2.61973708868026733e-01,-6.87354723515962176e-09, 1.29949238578680193e+00,
-2.56910413503646851e-01,-2.81380388482253283e-10, 1.29292929292929304e+00,
-2.51872614026069641e-01,-5.72900043881445400e-09, 1.28643216080402012e+00,
-2.46860072016716003e-01,-5.91480979446667301e-09, 1.28000000000000003e+00,
-2.41872534155845642e-01,-2.26464108218269596e-09, 1.27363184079601988e+00,
-2.36909747123718262e-01, 4.53605466823234164e-11, 1.26732673267326734e+00,
-2.31971472501754761e-01, 7.06397961769295543e-09, 1.26108374384236455e+00,
-2.27057456970214844e-01, 6.33486875889138723e-09, 1.25490196078431371e+00,
-2.22167462110519409e-01,-3.23063488769064667e-09, 1.24878048780487805e+00,
-2.17301279306411743e-01, 3.61643034801203993e-09, 1.24271844660194164e+00,
-2.12458655238151550e-01, 4.02395814855235541e-09, 1.23671497584541057e+00,
-2.07639366388320923e-01, 1.61007642123612140e-09, 1.23076923076923084e+00,
-2.02843189239501953e-01,-3.27524951814188617e-09, 1.22488038277511957e+00,
-1.98069915175437927e-01, 1.41334413242682632e-09, 1.21904761904761916e+00,
-1.93319305777549744e-01,-5.22594623594355708e-09, 1.21327014218009488e+00,
-1.88591167330741882e-01,-2.47680814003470502e-09, 1.20754716981132071e+00,
-1.83885276317596436e-01,-2.45254092706628205e-09, 1.20187793427230050e+00,
-1.79201424121856689e-01,-5.33585430316310109e-09, 1.19626168224299056e+00,
-1.74539417028427124e-01, 6.76527446759182412e-10, 1.19069767441860463e+00,
-1.69899031519889832e-01,-5.27550764135745627e-09, 1.18518518518518512e+00,
-1.65280088782310486e-01,-2.15679243846349595e-09, 1.17972350230414746e+00,
-1.60682380199432373e-01,-1.49104109249643331e-09, 1.17431192660550465e+00,
-1.56105712056159973e-01,-2.60690168170597171e-09, 1.16894977168949765e+00,
-1.51549905538558960e-01, 7.41135802212024734e-09, 1.16363636363636358e+00,
-1.47014737129211426e-01,-5.83259823325358474e-09, 1.15837104072398200e+00,
-1.42500057816505432e-01,-4.79077759802837772e-09, 1.15315315315315314e+00,
-1.38005673885345459e-01, 8.65901742282623279e-10, 1.14798206278026904e+00,
-1.33531391620635986e-01,-1.00388663681821861e-09, 1.14285714285714279e+00,
-1.29077047109603882e-01, 4.83446153849008985e-09, 1.13777777777777778e+00,
-1.24642446637153625e-01, 1.42987702814978788e-09, 1.13274336283185839e+00,
-1.20227426290512085e-01,-7.07647715363537801e-10, 1.12775330396475781e+00,
-1.15831822156906128e-01, 6.63178442283056725e-09, 1.12280701754385959e+00,
-1.11455440521240234e-01,-4.04082592521621347e-10, 1.11790393013100431e+00,
-1.07098132371902466e-01,-3.18446463469280006e-09, 1.11304347826086958e+00,
-1.02759733796119690e-01,-1.61649244833909166e-10, 1.10822510822510822e+00,
-9.84400659799575806e-02,-6.83329493933648262e-09, 1.10344827586206895e+00,
-9.41389948129653931e-02, 3.89910348303077415e-09, 1.09871244635193133e+00,
-8.98563265800476074e-02,-2.54181343965477199e-09, 1.09401709401709413e+00,
-8.55919271707534790e-02,-3.16465003491224077e-09, 1.08936170212765959e+00,
-8.13456326723098755e-02,-6.78164253039906136e-09, 1.08474576271186440e+00,
-7.71173089742660522e-02, 5.62983476247642727e-09, 1.08016877637130793e+00,
-7.29067772626876831e-02, 6.45459990253973109e-09, 1.07563025210084029e+00,
-6.87138885259628296e-02,-4.02208897878484947e-09, 1.07112970711297062e+00,
-6.45385235548019409e-02, 2.41723076924504492e-09, 1.06666666666666665e+00,
-6.03805184364318848e-02, 7.44752440489417922e-09, 1.06224066390041494e+00,
-5.62397241592407227e-02, 5.83636464485951227e-09, 1.05785123966942152e+00,
-5.21160066127777100e-02, 5.47376369159930659e-09, 1.05349794238683137e+00,
-4.80092167854309082e-02,-2.40092969954887866e-09, 1.04918032786885251e+00,
-4.39192354679107666e-02, 1.53307527607517038e-09, 1.04489795918367356e+00,
-3.98459136486053467e-02, 5.10140567602107126e-09, 1.04065040650406515e+00,
-3.57891023159027100e-02,-5.53568256931440472e-09, 1.03643724696356276e+00,
-3.17486971616744995e-02,-1.15290580164527755e-09, 1.03225806451612900e+00,
-2.77245491743087769e-02, 1.15945391638832911e-09, 1.02811244979919669e+00,
-2.37165242433547974e-02,-2.37396124475506562e-09, 1.02400000000000002e+00,
-1.97245031595230103e-02,-2.18825557886536567e-09, 1.01992031872509958e+00,
-1.57483518123626709e-02,-5.15577649770911158e-09, 1.01587301587301582e+00,
-1.17879509925842285e-02,-4.75945801195353596e-09, 1.01185770750988135e+00,
-7.84318149089813232e-03, 4.02987223945103447e-09, 1.00787401574803148e+00,
-3.91389429569244385e-03,-5.02544388524466163e-09, 1.00392156862745097e+00,
 0.00000000000000000e+00, 0.00000000000000000e+00, 1.00000000000000000e+00,
 7.78214633464813232e-03,-5.89259318337675549e-09, 9.92248062015503862e-01,
 1.55041813850402832e-02, 5.15092497094772879e-09, 9.84615384615384670e-01,
 2.31670588254928589e-02, 4.56041519342080401e-10, 9.77099236641221336e-01,
 3.07716578245162964e-02, 8.42237391984309442e-10, 9.69696969696969724e-01,
 3.83188575506210327e-02, 6.75151556647891194e-09, 9.62406015037593932e-01,
 4.58095371723175049e-02,-1.14102330171613308e-09, 9.55223880597014907e-01,
 5.32445162534713745e-02,-1.73465909164584847e-09, 9.48148148148148184e-01,
 6.06246292591094971e-02,-7.44267465448970614e-09, 9.41176470588235281e-01,
 6.79506659507751465e-02,-4.04226739708980980e-09, 9.34306569343065663e-01,
 7.52234160900115967e-02, 5.14757592901891746e-09, 9.27536231884057982e-01,
 8.24436694383621216e-02,-2.27287530313871183e-10, 9.20863309352518034e-01,
 8.96121561527252197e-02, 2.53696191289338878e-09, 9.14285714285714257e-01,
 9.67296212911605835e-02, 5.16739052879947717e-09, 9.07801418439716290e-01,
 1.03796795010566711e-01,-1.32892314659971938e-09, 9.01408450704225372e-01,
 1.10814362764358521e-01, 3.57593159368699372e-09, 8.95104895104895104e-01,
 1.17783039808273315e-01,-4.15188986089089339e-09, 8.88888888888888840e-01,
 1.24703481793403625e-01,-3.29244638962487482e-09, 8.82758620689655160e-01,
 1.31576359272003174e-01,-1.48328390123940883e-09, 8.76712328767123239e-01,
 1.38402327895164490e-01,-5.03604535406076773e-09, 8.70748299319727859e-01,
 1.45182013511657715e-01,-3.66715981756181484e-09, 8.64864864864864913e-01,
 1.51916041970252991e-01, 5.55889843491471725e-11, 8.59060402684563740e-01,
 1.58605024218559265e-01, 5.95807931895665273e-09, 8.53333333333333277e-01,
 1.65249571204185486e-01, 1.69112167703576755e-09, 8.47682119205298013e-01,
 1.71850264072418213e-01,-7.14575899055052612e-09, 8.42105263157894690e-01,
 1.78407654166221619e-01, 3.30659667846705631e-09, 8.36601307189542509e-01,
 1.84922337532043457e-01, 9.61968535632653506e-10, 8.31168831168831224e-01,
 1.91394850611686707e-01, 2.38794274806633005e-09, 8.25806451612903225e-01,
 1.97825744748115540e-01,-1.41819565918820911e-09, 8.20512820512820484e-01,
 2.04215541481971741e-01,-5.32808492188358669e-11, 8.15286624203821697e-01,
 2.10564762353897095e-01, 6.75345254294299015e-09, 8.10126582278481000e-01,
 2.16873943805694580e-01,-5.50508022045903553e-09, 8.05031446540880546e-01,
 2.23143547773361206e-01, 3.54084854971160739e-09, 8.00000000000000044e-01,
 2.29374095797538757e-01, 5.26730707266726214e-09, 7.95031055900621064e-01,
 2.35566064715385437e-01, 6.59738147206586947e-09, 7.90123456790123413e-01,
 2.41719931364059448e-01, 5.52308571990212030e-09, 7.85276073619631920e-01,
 2.47836157679557800e-01, 6.22502345648763414e-09, 7.80487804878048808e-01,
 2.53915205597877502e-01, 4.38308594169591683e-09, 7.75757575757575757e-01,
 2.59957522153854370e-01, 2.28307169685489199e-09, 7.71084337349397630e-01,
 2.65963554382324219e-01,-5.88518627741087381e-09, 7.66467065868263520e-01,
 2.71933719515800476e-01,-4.03215871724254953e-09, 7.61904761904761862e-01,
 2.77868449687957764e-01, 1.31549854251447497e-09, 7.57396449704141994e-01,
 2.83768177032470703e-01,-3.90182610477809875e-09, 7.52941176470588225e-01,
 2.89633288979530334e-01, 3.60351234240623674e-09, 7.48538011695906391e-01,
 2.95464217662811279e-01,-4.76897540291019344e-09, 7.44186046511627897e-01,
 3.01261335611343384e-01,-5.03318160277618717e-09, 7.39884393063583778e-01,
 3.07025030255317688e-01, 5.03959417408684312e-09, 7.35632183908045967e-01,
 3.12755703926086426e-01, 6.07781046260499659e-09, 7.31428571428571428e-01,
 3.18453729152679443e-01, 1.96585517245087232e-09, 7.27272727272727293e-01,
 3.24119463562965393e-01, 5.09124658302426438e-09, 7.23163841807909602e-01,
 3.29753279685974121e-01, 6.68649386072067322e-09, 7.19101123595505598e-01,
 3.35355535149574280e-01, 6.77156355047202320e-09, 7.15083798882681587e-01,
 3.40926587581634521e-01,-6.11041311179285793e-10, 7.11111111111111138e-01,
 3.46466764807701111e-01, 2.53850747007861840e-09, 7.07182320441988921e-01,
};

const double _TBL_log_B[] = {
-3.91389429569244385e-03,-5.02544388524466163e-09, 1.00392156862745097e+00,
-3.88325750827789307e-03,-5.02726758362722641e-09, 1.00389081216874487e+00,
-3.85262072086334229e-03,-5.96767515823008967e-09, 1.00386005759451025e+00,
-3.82199883460998535e-03, 7.05455209165671353e-09, 1.00382930490457367e+00,
-3.79136204719543457e-03, 4.23714927991518679e-09, 1.00379855409876240e+00,
-3.76072525978088379e-03, 4.81335096685962298e-10, 1.00376780517690301e+00,
-3.73008847236633301e-03,-4.21283296702168204e-09, 1.00373705813882252e+00,
-3.69946658611297607e-03, 5.05586376836614044e-09, 1.00370631298434776e+00,
-3.66882979869842529e-03,-1.51483960440205131e-09, 1.00367556971330552e+00,
-3.63820791244506836e-03, 5.87743677753113062e-09, 1.00364482832552304e+00,
-3.60757112503051758e-03,-2.56957200364865487e-09, 1.00361408882082692e+00,
-3.57694923877716064e-03, 2.94651390435441398e-09, 1.00358335119904440e+00,
-3.54631245136260986e-03,-7.37657042683427676e-09, 1.00355261546000252e+00,
-3.51569056510925293e-03,-3.73644515547788280e-09, 1.00352188160352807e+00,
-3.48506867885589600e-03,-1.03421402666104684e-09, 1.00349114962944808e+00,
-3.45444679260253906e-03, 7.30180403101370996e-10, 1.00346041953759002e+00,
-3.42382490634918213e-03, 1.55679557201728564e-09, 1.00342969132778048e+00,
-3.39320302009582520e-03, 1.44568891301803144e-09, 1.00339896499984693e+00,
-3.36258113384246826e-03, 3.96917853759009846e-10, 1.00336824055361618e+00,
-3.33195924758911133e-03,-1.58946018337966475e-09, 1.00333751798891568e+00,
-3.30133736133575439e-03,-4.51338778129251965e-09, 1.00330679730557248e+00,
-3.27073037624359131e-03, 6.52635366569957973e-09, 1.00327607850341383e+00,
-3.24010848999023438e-03, 1.72749917645944958e-09, 1.00324536158226696e+00,
-3.20948660373687744e-03,-4.00873265387982830e-09, 1.00321464654195869e+00,
-3.17887961864471436e-03, 4.21887676454276990e-09, 1.00318393338231693e+00,
-3.14825773239135742e-03,-3.39193756522611187e-09, 1.00315322210316848e+00,
-3.11765074729919434e-03, 2.96120412998002696e-09, 1.00312251270434083e+00,
-3.08702886104583740e-03,-6.52396315733305793e-09, 1.00309180518566143e+00,
-3.05642187595367432e-03,-2.04505966453846220e-09, 1.00306109954695732e+00,
-3.02581489086151123e-03, 1.49681078417891985e-09, 1.00303039578805597e+00,
-2.99520790576934814e-03, 4.10170555321366339e-09, 1.00299969390878485e+00,
-2.96460092067718506e-03, 5.76968200169280454e-09, 1.00296899390897121e+00,
-2.93399393558502197e-03, 6.50079748347648137e-09, 1.00293829578844274e+00,
-2.90338695049285889e-03, 6.29510934715858454e-09, 1.00290759954702668e+00,
-2.87277996540069580e-03, 5.15267493606739460e-09, 1.00287690518455030e+00,
-2.84217298030853271e-03, 3.07355158826623424e-09, 1.00284621270084173e+00,
-2.81156599521636963e-03, 5.77966365541068827e-11, 1.00281552209572777e+00,
-2.78095901012420654e-03,-3.89453259153365608e-09, 1.00278483336903634e+00,
-2.75036692619323730e-03, 6.11778242012290536e-09, 1.00275414652059491e+00,
-2.71975994110107422e-03, 2.92476600838397708e-10, 1.00272346155023095e+00,
-2.68915295600891113e-03,-6.46923154379134371e-09, 1.00269277845777238e+00,
-2.65856087207794189e-03, 7.33876486568420995e-10, 1.00266209724304645e+00,
-2.62796878814697266e-03, 7.00069679929666677e-09, 1.00263141790588084e+00,
-2.59736180305480957e-03,-2.56987450348743909e-09, 1.00260074044610348e+00,
-2.56676971912384033e-03, 1.82454225661907474e-09, 1.00257006486354183e+00,
-2.53617763519287109e-03, 5.28284317121768946e-09, 1.00253939115802360e+00,
-2.50557065010070801e-03,-7.09607567334798965e-09, 1.00250871932937646e+00,
-2.47497856616973877e-03,-5.50983461444862309e-09, 1.00247804937742835e+00,
-2.44438648223876953e-03,-4.85953757625443140e-09, 1.00244738130200695e+00,
-2.41379439830780029e-03,-5.14512729434392505e-09, 1.00241671510293973e+00,
-2.38320231437683105e-03,-6.36654650955091875e-09, 1.00238605078005505e+00,
-2.35262513160705566e-03, 6.37742322588376667e-09, 1.00235538833318039e+00,
-2.32203304767608643e-03, 3.28451677292232615e-09, 1.00232472776214365e+00,
-2.29144096374511719e-03,-7.44047431183455013e-10, 1.00229406906677276e+00,
-2.26084887981414795e-03,-5.70821214828217866e-09, 1.00226341224689541e+00,
-2.23027169704437256e-03, 3.29324104837311980e-09, 1.00223275730233974e+00,
-2.19967961311340332e-03,-3.54195300126501174e-09, 1.00220210423293365e+00,
-2.16910243034362793e-03, 3.58858531289579462e-09, 1.00217145303850508e+00,
-2.13851034641265869e-03,-5.11740917969288074e-09, 1.00214080371888192e+00,
-2.10793316364288330e-03, 1.42443120561648236e-10, 1.00211015627389211e+00,
-2.07735598087310791e-03, 4.46703822646021921e-09, 1.00207951070336398e+00,
-2.04676389694213867e-03,-7.04472785444456599e-09, 1.00204886700712525e+00,
-2.01618671417236328e-03,-4.59047553830473018e-09, 1.00201822518500405e+00,
-1.98560953140258789e-03,-3.07130882806221793e-09, 1.00198758523682852e+00,
-1.95503234863281250e-03,-2.48717053805762744e-09, 1.00195694716242656e+00,
-1.92445516586303711e-03,-2.83800348787722703e-09, 1.00192631096162676e+00,
-1.89387798309326172e-03,-4.12375050235231069e-09, 1.00189567663425660e+00,
-1.86330080032348633e-03,-6.34435441155856039e-09, 1.00186504418014488e+00,
-1.83273851871490479e-03, 5.40140314303225667e-09, 1.00183441359911951e+00,
-1.80216133594512939e-03, 1.31125693316229768e-09, 1.00180378489100863e+00,
-1.77158415317535400e-03,-3.71357469312577040e-09, 1.00177315805564038e+00,
-1.74102187156677246e-03, 5.22812660696889695e-09, 1.00174253309284333e+00,
-1.71044468879699707e-03,-1.66590441053700504e-09, 1.00171191000244564e+00,
-1.67988240718841553e-03, 5.40671178052330620e-09, 1.00168128878427543e+00,
-1.64930522441864014e-03,-3.35629007431391016e-09, 1.00165066943816106e+00,
-1.61874294281005859e-03, 1.84746954063897102e-09, 1.00162005196393089e+00,
-1.58818066120147705e-03, 6.11688655428798983e-09, 1.00158943636141329e+00,
-1.55760347843170166e-03,-5.44914310969879010e-09, 1.00155882263043683e+00,
-1.52704119682312012e-03,-3.04823995134767346e-09, 1.00152821077082943e+00,
-1.49647891521453857e-03,-1.58150805746462783e-09, 1.00149760078241989e+00,
-1.46591663360595703e-03,-1.04889032624402172e-09, 1.00146699266503658e+00,
-1.43535435199737549e-03,-1.45032966111563900e-09, 1.00143638641850807e+00,
-1.40479207038879395e-03,-2.78576897074403919e-09, 1.00140578204266251e+00,
-1.37422978878021240e-03,-5.05515116902791668e-09, 1.00137517953732846e+00,
-1.34368240833282471e-03, 6.64274201874819249e-09, 1.00134457890233475e+00,
-1.31312012672424316e-03, 2.50564528052393040e-09, 1.00131398013750950e+00,
-1.28255784511566162e-03,-2.56522311945031222e-09, 1.00128338324268173e+00,
-1.25201046466827393e-03, 6.33135507784428041e-09, 1.00125278821767960e+00,
-1.22144818305969238e-03,-6.06885455347382231e-10, 1.00122219506233190e+00,
-1.19090080261230469e-03, 6.42243472337993407e-09, 1.00119160377646743e+00,
-1.16033852100372314e-03,-2.38294972418881814e-09, 1.00116101435991456e+00,
-1.12979114055633545e-03, 2.77934063389291644e-09, 1.00113042681250186e+00,
-1.09924376010894775e-03, 7.00820164280034523e-09, 1.00109984113405837e+00,
-1.06868147850036621e-03,-4.59747085751906857e-09, 1.00106925732441265e+00,
-1.03813409805297852e-03,-2.23529745080199497e-09, 1.00103867538339331e+00,
-1.00758671760559082e-03,-8.06382307554537002e-10, 1.00100809531082935e+00,
-9.77039337158203125e-04,-3.10668409660968056e-10, 1.00097751710654936e+00,
-9.46491956710815430e-04,-7.48098744230748732e-10, 1.00094694077038215e+00,
-9.15944576263427734e-04,-2.11861630359788859e-09, 1.00091636630215652e+00,
-8.85397195816040039e-04,-4.42216408532030776e-09, 1.00088579370170128e+00,
-8.54864716529846191e-04, 7.24247610166845720e-09, 1.00085522296884544e+00,
-8.24317336082458496e-04, 3.07303886166926729e-09, 1.00082465410341781e+00,
-7.93769955635070801e-04,-2.02925762469604437e-09, 1.00079408710524698e+00,
-7.63237476348876953e-04, 6.83680481797299698e-09, 1.00076352197416241e+00,
-7.32690095901489258e-04,-1.31039221686822619e-10, 1.00073295870999268e+00,
-7.02157616615295410e-04, 6.86958961513182539e-09, 1.00070239731256683e+00,
-6.71610236167907715e-04,-1.96357409337380202e-09, 1.00067183778171387e+00,
-6.41077756881713867e-04, 3.17184900116537222e-09, 1.00064128011726261e+00,
-6.10545277595520020e-04, 7.37475466035728420e-09, 1.00061072431904230e+00,
-5.79997897148132324e-04,-4.25596135940766022e-09, 1.00058017038688196e+00,
-5.49465417861938477e-04,-1.91791972541298506e-09, 1.00054961832061062e+00,
-5.18932938575744629e-04,-5.12224691701438681e-10, 1.00051906812005731e+00,
-4.88400459289550781e-04,-3.88193236837349622e-11, 1.00048851978505127e+00,
-4.57867980003356934e-04,-4.97646691985572069e-10, 1.00045797331542152e+00,
-4.27335500717163086e-04,-1.88864987244699560e-09, 1.00042742871099710e+00,
-3.96803021430969238e-04,-4.21177194612176124e-09, 1.00039688597160747e+00,
-3.66285443305969238e-04, 7.43420519457095787e-09, 1.00036634509708144e+00,
-3.35752964019775391e-04, 3.24701607045658208e-09, 1.00033580608724848e+00,
-3.05220484733581543e-04,-1.87212122130830029e-09, 1.00030526894193783e+00,
-2.74702906608581543e-04, 6.97801141122173569e-09, 1.00027473366097874e+00,
-2.44170427322387695e-04,-4.85152676138296546e-12, 1.00024420024420024e+00,
-2.13652849197387695e-04, 6.98166924011500444e-09, 1.00021366869143180e+00,
-1.83120369911193848e-04,-1.86469179337632132e-09, 1.00018313900250289e+00,
-1.52602791786193848e-04, 3.25844463771942393e-09, 1.00015261117724252e+00,
-1.22085213661193848e-04, 7.44997421160607332e-09, 1.00012208521548041e+00,
-9.15527343750000000e-05,-4.19120739871988842e-09, 1.00009156111704556e+00,
-6.10351562500000000e-05,-1.86272094392572102e-09, 1.00006103888176767e+00,
-3.05175781250000000e-05,-4.65670761427728399e-10, 1.00003051850947600e+00,
 0.00000000000000000e+00, 0.00000000000000000e+00, 1.00000000000000000e+00,
 6.10351562500000000e-05,-1.86256936147508694e-09, 9.99938968568812925e-01,
 1.22070312500000000e-04,-7.44997432262837661e-09, 9.99877944586842426e-01,
 1.83090567588806152e-04,-1.86059906713601158e-09, 9.99816928052724707e-01,
 2.44110822677612305e-04, 4.84975040447297940e-12, 9.99755918965096413e-01,
 3.05131077766418457e-04,-1.85317341396257296e-09, 9.99694917322594412e-01,
 3.66151332855224609e-04,-7.43421418737826135e-09, 9.99633923123856016e-01,
 4.27156686782836914e-04,-1.83665708630120917e-09, 9.99572936367518761e-01,
 4.88162040710449219e-04, 3.87909019697870470e-11, 9.99511957052220623e-01,
 5.49167394638061523e-04,-1.80741609914099066e-09, 9.99450985176599804e-01,
 6.10172748565673828e-04,-7.37482404931355675e-09, 9.99390020739294838e-01,
 6.71163201332092285e-04,-1.76181779746428832e-09, 9.99329063738944812e-01,
 7.32153654098510742e-04, 1.30895336731373947e-10, 9.99268114174188815e-01,
 7.93144106864929199e-04,-1.69623085559783903e-09, 9.99207172043666492e-01,
 8.54134559631347656e-04,-7.24274266634642539e-09, 9.99146237346017818e-01,
 9.15110111236572266e-04,-1.60702527656420466e-09, 9.99085310079882882e-01,
 9.76085662841796875e-04, 3.10213662020960823e-10, 9.99024390243902438e-01,
 1.03706121444702148e-03,-1.49057239143362854e-09, 9.98963477836717240e-01,
 1.09803676605224609e-03,-7.00893006071293205e-09, 9.98902572856968707e-01,
 1.15899741649627686e-03,-1.34324485867628618e-09, 9.98841675303298149e-01,
 1.21995806694030762e-03, 6.05775231219848419e-10, 9.98780785174347763e-01,
 1.28091871738433838e-03,-1.16141666351493105e-09, 9.98719902468759524e-01,
 1.34187936782836914e-03,-6.64436749823241709e-09, 9.98659027185176185e-01,
 1.40282511711120605e-03,-9.41463117277934116e-10, 9.98598159322240497e-01,
 1.46377086639404297e-03, 1.04658816448688973e-09, 9.98537298878595769e-01,
 1.52471661567687988e-03,-6.79760856752719950e-10, 9.98476445852885641e-01,
 1.58566236495971680e-03,-6.12005746759215516e-09, 9.98415600243753754e-01,
 1.64659321308135986e-03,-3.72687843539593911e-10, 9.98354762049844635e-01,
 1.70752406120300293e-03, 1.66163936946120510e-09, 9.98293931269802592e-01,
 1.76845490932464600e-03,-1.66233634059653482e-11, 9.98233107902272598e-01,
 1.82938575744628906e-03,-5.40702365965725852e-09, 9.98172291945899848e-01,
 1.89030170440673828e-03, 3.92051974359037094e-10, 9.98111483399329869e-01,
 1.95121765136718750e-03, 2.47989456193967386e-09, 9.98050682261208522e-01,
 2.01213359832763672e-03, 8.56956237589556411e-10, 9.97989888530182112e-01,
 2.07304954528808594e-03,-4.47631094680578221e-09, 9.97929102204897056e-01,
 2.13395059108734131e-03, 1.38170617188760535e-09, 9.97868323284000214e-01,
 2.19485163688659668e-03, 3.53029828652923466e-09, 9.97807551766138889e-01,
 2.25575268268585205e-03, 1.96991720126727264e-09, 9.97746787649960387e-01,
 2.31665372848510742e-03,-3.29898536228900729e-09, 9.97686030934112789e-01,
 2.37753987312316895e-03, 2.62520342879853523e-09, 9.97625281617244064e-01,
 2.43842601776123047e-03, 4.84177393727403425e-09, 9.97564539698002961e-01,
 2.49931216239929199e-03, 3.35117763725068060e-09, 9.97503805175038005e-01,
 2.56019830703735352e-03,-1.84613407961685003e-09, 9.97443078046998610e-01,
 2.62106955051422119e-03, 4.15145128973533728e-09, 9.97382358312534190e-01,
 2.68194079399108887e-03, 6.44322377825746369e-09, 9.97321645970294601e-01,
 2.74281203746795654e-03, 5.02963453034911059e-09, 9.97260941018929925e-01,
 2.80368328094482422e-03,-8.88653919683747801e-11, 9.97200243457090685e-01,
 2.86453962326049805e-03, 5.98933618481583215e-09, 9.97139553283427627e-01,
 2.92541086673736572e-03,-6.53763222966854470e-09, 9.97078870496592051e-01,
 2.98625230789184570e-03, 7.03416376112841439e-09, 9.97018195095235149e-01,
 3.04710865020751953e-03, 2.00169130837263856e-09, 9.96957527078008998e-01,
 3.10796499252319336e-03,-6.73343774365733744e-09, 9.96896866443565566e-01,
 3.16880643367767334e-03,-4.26961163294097628e-09, 9.96836213190557263e-01,
 3.22964787483215332e-03,-5.50754106739102871e-09, 9.96775567317637057e-01,
 3.29047441482543945e-03, 4.45438555055709972e-09, 9.96714928823457802e-01,
 3.35131585597991943e-03,-4.18570384527292348e-09, 9.96654297706673131e-01,
 3.41214239597320557e-03,-1.62503662784476487e-09, 9.96593673965936788e-01,
 3.47296893596649170e-03,-2.76432383382302585e-09, 9.96533057599902627e-01,
 3.53378057479858398e-03, 7.29804580568502866e-09, 9.96472448607225392e-01,
 3.59460711479187012e-03,-1.23980010408883135e-09, 9.96411846986559602e-01,
 3.65541875362396240e-03, 1.42491073537977088e-09, 9.96351252736560444e-01,
 3.71623039245605469e-03, 3.91466958994193504e-10, 9.96290665855883217e-01,
 3.77704203128814697e-03,-4.33968168655263622e-09, 9.96230086343183774e-01,
 3.83783876895904541e-03, 2.13307565724173765e-09, 9.96169514197117967e-01,
 3.89863550662994385e-03, 4.90902737916628080e-09, 9.96108949416342426e-01,
};

const double _TBL_log_C[] = {
-3.05175781250000000e-05,-4.65670761427728399e-10, 1.00003051850947600e+00,
-3.02791595458984375e-05,-4.58423005201879013e-10, 1.00003028007640116e+00,
-3.00407409667968750e-05,-4.51232095837388619e-10, 1.00003004164344000e+00,
-2.98023223876953125e-05,-4.44098033307149783e-10, 1.00002980321059254e+00,
-2.95639038085937500e-05,-4.37020817584055021e-10, 1.00002956477785876e+00,
-2.93254852294921875e-05,-4.30000448640996850e-10, 1.00002932634523889e+00,
-2.90870666503906250e-05,-4.23036926450867785e-10, 1.00002908791273248e+00,
-2.88486480712890625e-05,-4.16130250986560497e-10, 1.00002884948033977e+00,
-2.86102294921875000e-05,-4.09280422220967502e-10, 1.00002861104806073e+00,
-2.83718109130859375e-05,-4.02487440126981420e-10, 1.00002837261589561e+00,
-2.81333923339843750e-05,-3.95751304677494920e-10, 1.00002813418384395e+00,
-2.78949737548828125e-05,-3.89072015845400571e-10, 1.00002789575190620e+00,
-2.76565551757812500e-05,-3.82449573603591096e-10, 1.00002765732008192e+00,
-2.74181365966796875e-05,-3.75883977924959216e-10, 1.00002741888837154e+00,
-2.71797180175781250e-05,-3.69375228782397552e-10, 1.00002718045677463e+00,
-2.69412994384765625e-05,-3.62923326148798877e-10, 1.00002694202529163e+00,
-2.67028808593750000e-05,-3.56528269997055914e-10, 1.00002670359392232e+00,
-2.64644622802734375e-05,-3.50190060300061438e-10, 1.00002646516266647e+00,
-2.62260437011718750e-05,-3.43908697030708274e-10, 1.00002622673152453e+00,
-2.59876251220703125e-05,-3.37684180161889146e-10, 1.00002598830049627e+00,
-2.57492065429687500e-05,-3.31516509666496982e-10, 1.00002574986958170e+00,
-2.55107879638671875e-05,-3.25405685517424506e-10, 1.00002551143878082e+00,
-2.52723693847656250e-05,-3.19351707687564646e-10, 1.00002527300809363e+00,
-2.50339508056640625e-05,-3.13354576149810333e-10, 1.00002503457752012e+00,
-2.47955322265625000e-05,-3.07414290877054339e-10, 1.00002479614706030e+00,
-2.45571136474609375e-05,-3.01530851842189648e-10, 1.00002455771671417e+00,
-2.43186950683593750e-05,-2.95704259018109239e-10, 1.00002431928648172e+00,
-2.40802764892578125e-05,-2.89934512377706043e-10, 1.00002408085636296e+00,
-2.38418579101562500e-05,-2.84221611893872988e-10, 1.00002384242635789e+00,
-2.36034393310546875e-05,-2.78565557539503159e-10, 1.00002360399646650e+00,
-2.33650207519531250e-05,-2.72966349287489486e-10, 1.00002336556668880e+00,
-2.31266021728515625e-05,-2.67423987110725053e-10, 1.00002312713702501e+00,
-2.28881835937500000e-05,-2.61938470982102893e-10, 1.00002288870747469e+00,
-2.26497650146484375e-05,-2.56509800874516142e-10, 1.00002265027803805e+00,
-2.24113464355468750e-05,-2.51137976760857781e-10, 1.00002241184871532e+00,
-2.21729278564453125e-05,-2.45822998614021050e-10, 1.00002217341950606e+00,
-2.19345092773437500e-05,-2.40564866406898983e-10, 1.00002193499041070e+00,
-2.16960906982421875e-05,-2.35363580112384817e-10, 1.00002169656142881e+00,
-2.14576721191406250e-05,-2.30219139703371639e-10, 1.00002145813256060e+00,
-2.12192535400390625e-05,-2.25131545152752687e-10, 1.00002121970380631e+00,
-2.09808349609375000e-05,-2.20100796433421149e-10, 1.00002098127516570e+00,
-2.07424163818359375e-05,-2.15126893518270292e-10, 1.00002074284663856e+00,
-2.05039978027343750e-05,-2.10209836380193328e-10, 1.00002050441822532e+00,
-2.02655792236328125e-05,-2.05349624992083549e-10, 1.00002026598992555e+00,
-2.00271606445312500e-05,-2.00546259326834221e-10, 1.00002002756173969e+00,
-1.97887420654296875e-05,-1.95799739357338661e-10, 1.00001978913366751e+00,
-1.95503234863281250e-05,-1.91110065056490211e-10, 1.00001955070570903e+00,
-1.93119049072265625e-05,-1.86477236397182215e-10, 1.00001931227786400e+00,
-1.90734863281250000e-05,-1.81901253352308016e-10, 1.00001907385013289e+00,
-1.88350677490234375e-05,-1.77382115894761009e-10, 1.00001883542251546e+00,
-1.85966491699218750e-05,-1.72919823997434613e-10, 1.00001859699501172e+00,
-1.83582305908203125e-05,-1.68514377633222225e-10, 1.00001835856762167e+00,
-1.81198120117187500e-05,-1.64165776775017315e-10, 1.00001812014034530e+00,
-1.78813934326171875e-05,-1.59874021395713331e-10, 1.00001788171318262e+00,
-1.76429748535156250e-05,-1.55639111468203745e-10, 1.00001764328613363e+00,
-1.74045562744140625e-05,-1.51461046965382081e-10, 1.00001740485919832e+00,
-1.71661376953125000e-05,-1.47339827860141862e-10, 1.00001716643237670e+00,
-1.69277191162109375e-05,-1.43275454125376587e-10, 1.00001692800566877e+00,
-1.66893005371093750e-05,-1.39267925733979858e-10, 1.00001668957907452e+00,
-1.64508819580078125e-05,-1.35317242658845250e-10, 1.00001645115259397e+00,
-1.62124633789062500e-05,-1.31423404872866338e-10, 1.00001621272622709e+00,
-1.59740447998046875e-05,-1.27586412348936751e-10, 1.00001597429997391e+00,
-1.57356262207031250e-05,-1.23806265059950140e-10, 1.00001573587383463e+00,
-1.54972076416015625e-05,-1.20082962978800135e-10, 1.00001549744780882e+00,
-1.52587890625000000e-05,-1.16416506078380439e-10, 1.00001525902189670e+00,
-1.50203704833984375e-05,-1.12806894331584732e-10, 1.00001502059609826e+00,
-1.47819519042968750e-05,-1.09254127711306731e-10, 1.00001478217041373e+00,
-1.45435333251953125e-05,-1.05758206190440154e-10, 1.00001454374484267e+00,
-1.43051147460937500e-05,-1.02319129741878785e-10, 1.00001430531938529e+00,
-1.40666961669921875e-05,-9.89368983385163785e-11, 1.00001406689404182e+00,
-1.38282775878906250e-05,-9.56115119532467311e-11, 1.00001382846881182e+00,
-1.35898590087890625e-05,-9.23429705589636508e-11, 1.00001359004369550e+00,
-1.33514404296875000e-05,-8.91312741285609718e-11, 1.00001335161869309e+00,
-1.31130218505859375e-05,-8.59764226349325412e-11, 1.00001311319380415e+00,
-1.28746032714843750e-05,-8.28784160509722320e-11, 1.00001287476902911e+00,
-1.26361846923828125e-05,-7.98372543495739428e-11, 1.00001263634436754e+00,
-1.23977661132812500e-05,-7.68529375036315596e-11, 1.00001239791981988e+00,
-1.21593475341796875e-05,-7.39254654860390200e-11, 1.00001215949538569e+00,
-1.19209289550781250e-05,-7.10548382696902743e-11, 1.00001192107106540e+00,
-1.16825103759765625e-05,-6.82410558274792990e-11, 1.00001168264685858e+00,
-1.14440917968750000e-05,-6.54841181323000575e-11, 1.00001144422276567e+00,
-1.12056732177734375e-05,-6.27840251570465649e-11, 1.00001120579878622e+00,
-1.09672546386718750e-05,-6.01407768746128621e-11, 1.00001096737492068e+00,
-1.07288360595703125e-05,-5.75543732578929706e-11, 1.00001072895116883e+00,
-1.04904174804687500e-05,-5.50248142797809574e-11, 1.00001049052753044e+00,
-1.02519989013671875e-05,-5.25520999131709151e-11, 1.00001025210400596e+00,
-1.00135803222656250e-05,-5.01362301309569428e-11, 1.00001001368059517e+00,
-9.77516174316406250e-06,-4.77772049060331590e-11, 1.00000977525729784e+00,
-9.53674316406250000e-06,-4.54750242112937080e-11, 1.00000953683411442e+00,
-9.29832458496093750e-06,-4.32296880196327471e-11, 1.00000929841104469e+00,
-9.05990600585937500e-06,-4.10411963039444596e-11, 1.00000905998808842e+00,
-8.82148742675781250e-06,-3.89095490371230413e-11, 1.00000882156524606e+00,
-8.58306884765625000e-06,-3.68347461920627143e-11, 1.00000858314251739e+00,
-8.34465026855468750e-06,-3.48167877416577198e-11, 1.00000834471990241e+00,
-8.10623168945312500e-06,-3.28556736588023055e-11, 1.00000810629740089e+00,
-7.86781311035156250e-06,-3.09514039163907517e-11, 1.00000786787501328e+00,
-7.62939453125000000e-06,-2.91039784873173576e-11, 1.00000762945273936e+00,
-7.39097595214843750e-06,-2.73133973444764389e-11, 1.00000739103057912e+00,
-7.15255737304687500e-06,-2.55796604607623273e-11, 1.00000715260853257e+00,
-6.91413879394531250e-06,-2.39027678090693806e-11, 1.00000691418659948e+00,
-6.67572021484375000e-06,-2.22827193622919692e-11, 1.00000667576478031e+00,
-6.43730163574218750e-06,-2.07195150933244894e-11, 1.00000643734307482e+00,
-6.19888305664062500e-06,-1.92131549750613507e-11, 1.00000619892148301e+00,
-5.96046447753906250e-06,-1.77636389803969882e-11, 1.00000596050000490e+00,
-5.72204589843750000e-06,-1.63709670822258500e-11, 1.00000572207864047e+00,
-5.48362731933593750e-06,-1.50351392534424099e-11, 1.00000548365738973e+00,
-5.24520874023437500e-06,-1.37561554669411566e-11, 1.00000524523625267e+00,
-5.00679016113281250e-06,-1.25340156956165994e-11, 1.00000500681522930e+00,
-4.76837158203125000e-06,-1.13687199123632672e-11, 1.00000476839431940e+00,
-4.52995300292968750e-06,-1.02602680900757067e-11, 1.00000452997352340e+00,
-4.29153442382812500e-06,-9.20866020164848874e-12, 1.00000429155284110e+00,
-4.05311584472656250e-06,-8.21389621997619873e-12, 1.00000405313227247e+00,
-3.81469726562500000e-06,-7.27597611795344309e-12, 1.00000381471181754e+00,
-3.57627868652343750e-06,-6.39489986847484680e-12, 1.00000357629147629e+00,
-3.33786010742187500e-06,-5.57066744443505668e-12, 1.00000333787124873e+00,
-3.09944152832031250e-06,-4.80327881872873567e-12, 1.00000309945113486e+00,
-2.86102294921875000e-06,-4.09273396425056937e-12, 1.00000286103113467e+00,
-2.62260437011718750e-06,-3.43903285389525991e-12, 1.00000262261124817e+00,
-2.38418579101562500e-06,-2.84217546055753084e-12, 1.00000238419147536e+00,
-2.14576721191406250e-06,-2.30216175713212346e-12, 1.00000214577181623e+00,
-1.90734863281250000e-06,-1.81899171651379970e-12, 1.00000190735227079e+00,
-1.66893005371093750e-06,-1.39266531159734065e-12, 1.00000166893283904e+00,
-1.43051147460937500e-06,-1.02318251527754641e-12, 1.00000143051352097e+00,
-1.19209289550781250e-06,-7.10543300449236526e-13, 1.00000119209431659e+00,
-9.53674316406250000e-07,-4.54747640007250277e-13, 1.00000095367522590e+00,
-7.15255737304687500e-07,-2.55795506846445903e-13, 1.00000071525624890e+00,
-4.76837158203125000e-07,-1.13686873861701362e-13, 1.00000047683738558e+00,
-2.38418579101562500e-07,-2.84217139479138694e-14, 1.00000023841863594e+00,
 0.00000000000000000e+00, 0.00000000000000000e+00, 1.00000000000000000e+00,
 4.76837158203125000e-07,-1.13686801581556534e-13, 9.99999523163069171e-01,
 9.53674316406250000e-07,-4.54747061766091551e-13, 9.99999046326593088e-01,
 1.43051147460937500e-06,-1.02318056371363593e-12, 9.99998569490571754e-01,
 1.90734863281250000e-06,-1.81898709058453050e-12, 9.99998092655005166e-01,
 2.38418579101562500e-06,-2.84216642553942652e-12, 9.99997615819893326e-01,
 2.86102294921875000e-06,-4.09271835173928558e-12, 9.99997138985236234e-01,
 3.33786010742187500e-06,-5.57064265234537863e-12, 9.99996662151033888e-01,
 3.81469726562500000e-06,-7.27593911051928784e-12, 9.99996185317286290e-01,
 4.29153442382812500e-06,-9.20860750942290595e-12, 9.99995708483993329e-01,
 4.76837158203125000e-06,-1.13686476322184327e-11, 9.99995231651155225e-01,
 5.24520874023437500e-06,-1.37560592620683836e-11, 9.99994754818771869e-01,
 5.72204589843750000e-06,-1.63708421821355796e-11, 9.99994277986843150e-01,
 6.19888305664062500e-06,-1.92129961755831519e-11, 9.99993801155369288e-01,
 6.67572021484375000e-06,-2.22825210255745448e-11, 9.99993324324350064e-01,
 7.15255737304687500e-06,-2.55794165152735116e-11, 9.99992847493785697e-01,
 7.62939453125000000e-06,-2.91036824278441156e-11, 9.99992370663675967e-01,
 8.10623168945312500e-06,-3.28553185464507269e-11, 9.99991893834020984e-01,
 8.58306884765625000e-06,-3.68343246542580390e-11, 9.99991417004820748e-01,
 9.05990600585937500e-06,-4.10407005344310328e-11, 9.99990940176075260e-01,
 9.53674316406250000e-06,-4.54744459701350189e-11, 9.99990463347784519e-01,
 1.00135803222656250e-05,-5.01355607445356115e-11, 9.99989986519948526e-01,
 1.04904174804687500e-05,-5.50240446407987350e-11, 9.99989509692567280e-01,
 1.09672546386718750e-05,-6.01398974420906177e-11, 9.99989032865640670e-01,
 1.14440917968750000e-05,-6.54831189315778173e-11, 9.99988556039168808e-01,
 1.19209289550781250e-05,-7.10537088924271824e-11, 9.99988079213151804e-01,
 1.23977661132812500e-05,-7.68516671078058653e-11, 9.99987602387589436e-01,
 1.28746032714843750e-05,-8.28769933608813673e-11, 9.99987125562481816e-01,
 1.33514404296875000e-05,-8.91296874348214480e-11, 9.99986648737828943e-01,
 1.38282775878906250e-05,-9.56097491127942163e-11, 9.99986171913630706e-01,
 1.43051147460937500e-05,-1.02317178177968078e-10, 9.99985695089887328e-01,
 1.47819519042968750e-05,-1.09251974413511737e-10, 9.99985218266598586e-01,
 1.52587890625000000e-05,-1.16414137602594244e-10, 9.99984741443764591e-01,
 1.57356262207031250e-05,-1.23803667528384912e-10, 9.99984264621385344e-01,
 1.62124633789062500e-05,-1.31420563974053399e-10, 9.99983787799460844e-01,
 1.66893005371093750e-05,-1.39264826722769662e-10, 9.99983310977990980e-01,
 1.71661376953125000e-05,-1.47336455557703943e-10, 9.99982834156975864e-01,
 1.76429748535156250e-05,-1.55635450262026871e-10, 9.99982357336415606e-01,
 1.81198120117187500e-05,-1.64161810618909281e-10, 9.99981880516309873e-01,
 1.85966491699218750e-05,-1.72915536411522397e-10, 9.99981403696658999e-01,
 1.90734863281250000e-05,-1.81896627423037728e-10, 9.99980926877462872e-01,
 1.95503234863281250e-05,-1.91105083436627091e-10, 9.99980450058721382e-01,
 2.00271606445312500e-05,-2.00540904235462588e-10, 9.99979973240434639e-01,
 2.05039978027343750e-05,-2.10204089602716685e-10, 9.99979496422602532e-01,
 2.09808349609375000e-05,-2.20094639321562129e-10, 9.99979019605225283e-01,
 2.14576721191406250e-05,-2.30212553175171954e-10, 9.99978542788302671e-01,
 2.19345092773437500e-05,-2.40557830946719580e-10, 9.99978065971834806e-01,
 2.24113464355468750e-05,-2.51130472419378610e-10, 9.99977589155821689e-01,
 2.28881835937500000e-05,-2.61930477376323110e-10, 9.99977112340263208e-01,
 2.33650207519531250e-05,-2.72957845600727328e-10, 9.99976635525159474e-01,
 2.38418579101562500e-05,-2.84212576875765874e-10, 9.99976158710510488e-01,
 2.43186950683593750e-05,-2.95694670984613669e-10, 9.99975681896316138e-01,
 2.47955322265625000e-05,-3.07404127710445993e-10, 9.99975205082576646e-01,
 2.52723693847656250e-05,-3.19340946836438336e-10, 9.99974728269291790e-01,
 2.57492065429687500e-05,-3.31505128145766549e-10, 9.99974251456461571e-01,
 2.62260437011718750e-05,-3.43896671421606844e-10, 9.99973774644086211e-01,
 2.67028808593750000e-05,-3.56515576447135588e-10, 9.99973297832165375e-01,
 2.71797180175781250e-05,-3.69361843005529666e-10, 9.99972821020699398e-01,
 2.76565551757812500e-05,-3.82435470879966118e-10, 9.99972344209688058e-01,
 2.81333923339843750e-05,-3.95736459853622345e-10, 9.99971867399131464e-01,
 2.86102294921875000e-05,-4.09264809709676060e-10, 9.99971390589029618e-01,
 2.90870666503906250e-05,-4.23020520231305283e-10, 9.99970913779382409e-01,
 2.95639038085937500e-05,-4.37003591201688349e-10, 9.99970436970189946e-01,
 3.00407409667968750e-05,-4.51214022404003949e-10, 9.99969960161452232e-01,
 3.05175781250000000e-05,-4.65651813621430934e-10, 9.99969483353169153e-01,
};

#define	HI(x)	((int *) &x)[0]
#define	LO(x)	((int *) &x)[1]

extern const double _TBL_exp2_z[];

static const double pw[] = {
	/* New coef */
	/* HALF[0]=HALFP= */ +0.5,
	/* HALF[1]=HALFN= */ -0.5,
	/* C0 = */ +1.442695040888963407331361492316472503815e+0000,
	/* C1 = */ -7.213475204444814575554476780172262488228e-0001,
	/* C2 = */ +4.808983469632258749713343059085267288370e-0001,
	/* C3 = */ -3.606737610360722710599944793620089890892e-0001,
	/* C4 = */ +2.885389358423653791898594336639455625917e-0001,
	/* C5 = */ -2.398470400700468245916239605675441387580e-0001,
	/*
	 * exp2(v) ~ 1+v*(D1+v*(D2+v*(D3+v*D4))), where |v| <= 2**-9,
	 * pre=2**-58
	 */
	/* D1 = */ +6.931471805599391590463305694993282535894e-0001,
	/* D2 = */ +2.402265069591005782229760567511770004119e-0001,
	/* D3 = */ +5.550411506821709784853486883814916103240e-0002,
	/* D4 = */ +9.618129532167256903944369680704221601893e-0003,
	/* log2(1+v) ~ v*(G0+v*(G1+...+v*G4)), where |v|<= 2**-15 */
	/* G0 = */ +1.442695040888963407359924681012302448154e+0000,
	/* G1 = */ -7.213475204444817036147677701060111806049e-0001,
	/* G2 = */ +4.808983469629878023080174701018243890719e-0001,
	/* G3 = */ -3.606737605022049618082188829310410926123e-0001,
	/* G4 = */ +2.885390085137496134885036857056202159666e-0001,
	/*
	 * log(1+v) ~ v - 0.5v*v + v**3 * (P0+v*(P1+...+v*P5)), where -2**-8
	 * <= v < 2**-7
	 */
	/* P0 = */ +3.333333333333326371079496441227474269633e-0001,
	/* P1 = */ -2.499999999998000401798393776697019451768e-0001,
	/* P2 = */ +2.000000001325024869043804177434354070950e-0001,
	/* P3 = */ -1.666667034995311995725105619037478489835e-0001,
	/* P4 = */ +1.428531441624770270895219577998287677288e-0001,
	/* P5 = */ -1.233312360776102606766361691245797111675e-0001,
	/*
	 * log(1+v) ~ v + v**2*(-0.5 + v*(T1+v*(T2+v*T3))), precision 2**-78
	 * where -2**-15 <= v < 2**-14
	 */
	/* T1 = */ +3.333333333333441929606485081336932257220e-0001,
	/* T2 = */ -2.500000000628671926540148030643307976401e-0001,
	/* T3 = */ +1.999881443890404668688455760175850279731e-0001,
	/*
	 * exp(r) ~ 1+ r*(1.0+r*(B1+r*(B2+r*B3))) for |r|<=ln2*2**-9 =
	 * 1.35e-3
	 */
	/* B1 = */ +4.999999999999994329226023553116941526055e-0001,
	/* B2 = */ +1.666666800631238978802265642974644968200e-0001,
	/* B3 = */ +4.166666954215025418085995936151722790326e-0002,
	/* log(1+v)~ v*(1.0+v*(-0.5+v*Q3)) , where |v| <= 2**-22 */
	/* Q3 = */ +3.333333333333447021526155426274053579896e-0001,
	/* Other constant */
	/* ln2_256H ~ log2/256 chopped to 32 bits */
	/* INVLN2_256 */	3.693299304675746322841407183364843871812e+0002,
	/* LN2_256H */		2.70760617331688990816e-03,
	/* LN2_256L */		7.45396456746323320321e-13,
	/* ZERO */		0.0,
	/* TWO54 */		1.80143985094819840000e+16,	/* 2**54 */
	/* ONE */		1.0,
	/* TWOM100 */		7.88860905221011805412e-31,	/* 2**-100 */
	/* C256 */		256.0,
	/* INV256 */		0.00390625,
};

#define	HALF	pw
#define	HALFP	pw[0]
#define	HALFN	pw[1]
#define	C0	pw[2]
#define	C1	pw[3]
#define	C2	pw[4]
#define	C3	pw[5]
#define	C4	pw[6]
#define	C5	pw[7]
#define	D1	pw[8]
#define	D2	pw[9]
#define	D3	pw[10]
#define	D4	pw[11]
#define	G0	pw[12]
#define	G1	pw[13]
#define	G2	pw[14]
#define	G3	pw[15]
#define	G4	pw[16]
#define	P0	pw[17]
#define	P1	pw[18]
#define	P2	pw[19]
#define	P3	pw[20]
#define	P4	pw[21]
#define	P5	pw[22]
#define	T1	pw[23]
#define	T2	pw[24]
#define	T3	pw[25]
#define	B1	pw[26]
#define	B2	pw[27]
#define	B3	pw[28]
#define	Q3	pw[29]
#define	INVLN2_256	pw[30]
#define	LN2_256H	pw[31]
#define	LN2_256L	pw[32]
#define	ZERO	pw[33]
#define	TWO54	pw[34]
#define	ONE	pw[35]
#define	TWOM100	pw[36]
#define	C256	pw[37]
#define	INV256	pw[38]

double
__pow(double x, double y) {
	double yH, yL, zH, zL, p, q, r, s, t, u, v, w, z, nx, wx, X, Xi, Y, Yj;
	double r2, rH, rL, wH, wL, Ai, B, Bj, C, Ck, oddn;
	int i, j, k, jj/*, kk #70050 */, ii, yisint, n, m, hx, hy, ix, iy;
	unsigned lx, ly, kk;

	hx = HI(x);
	lx = LO(x);
	ix = hx & ~0x80000000;
	hy = HI(y);
	ly = LO(y);
	iy = hy & ~0x80000000;

	/* compute log2(x) using Quick Alogrithm */
	if ((hx | lx) == 0) {	/* special case for x=+0 */
		if ((iy | ly) == 0)
			return __kernel_standard (x, y, 20);
		i = iy | ((ly | -ly) >> 31);
		if (i > 0x7ff00000)
			return (y * y);	/* y is NaN; + -> * for Cheetah */
		else if (hy < 0) {
		        return (i == 0x7ff00000) ? (-y / x) :	/* y negative, return inf */
			        __kernel_standard (x, y, 43);	/* pow(+0.0,negative) */
		}
		else
			return (x);
	}
	j = 0x3fe6a09e;
	n = (ix - j) >> 20;
	i = ix - (n << 20);
	/* X = x normalized to [0x3fe6a09e,0x3ff6a09d] */
	HI(X) = i;
	LO(X) = lx;
	ii = ((i + 0x800) >> 12);
	HI(Xi) = ii << 12;
	LO(Xi) = 0;		/* Xi = X rounded to 8 bits */
	v = X - Xi;
	ii = ii - (j >> 12);
	ii = ii + ii + ii;	/* index */
	v *= _TBL_log2_X[ii + 2];	/* v = (X-Xi)/Xi */

	/*
	 * a slightly effective way to compute w =
	 * v*(C0+v*(C1+v*(C2+v*(C3+v*(C4+v*C5)))));
	 */
	r = v * v;
	s = C2 + r * C4;
	t = C3 + r * C5;
	j = 0x3ff00000;

	/* special case: y is +-inf, nan, +-1.0, or |y|<2**-64 */
	if ((((iy + 0x00100000) >> 20) - 0x3c0) < 0 || ((iy - j) | ly) == 0) {
		k = 0x7ff00000;
		if ((iy | ly) == 0) {
		        if (x != x) /* pow(NaN,0.0) */
		                return __kernel_standard (x, y, 42);
		        if ((ix | lx) == 0) /* pow(0.0,0.0) */
			        return __kernel_standard (x, y, 20);
			return (ONE);	/* x**0 = 1 */
		}
		else if (((hx - j) | lx) == 0)
			return (ONE);	/* C99: 1**y = 1 */
		else if (iy == j)	/* x**+-1 = x or 1/x */
			return (hy >= 0 ? ONE * x : (x != 0) ?
				( w = ONE / x, (fabs (w) != INFINITY) ? w :
				  __kernel_standard (x, y, 21) ) : /* pow overflow */
				  __kernel_standard (x, y, 23)); /* pow(-0.0,negative) */
		else if (k - (int) (ix | ((lx | -lx) >> 31)) < 0 ||
			k - (int) (iy | ((ly | -ly) >> 31)) < 0)
			return (x * y);	/* x or y is NaN; + -> * for Cheetah */
					/* includes Sun: 1**NaN = NaN */
		else if (iy == k) {	/* y is +-inf */
			if (((ix - j) | lx) == 0) {
				return (ONE);
						/* C99: -1**+-inf = 1 */
			} else if (hy > 0)
				/* +-(|x|>1)**+inf=+inf, +-(|x|<1)**+inf=+0 */
				return (ix >= j ? y : ZERO);
			else	/* +-(|x|>1)**-inf=+0, +-(|x|<1)**-inf=+inf */
				return (ix >= j ? ZERO : -y);
		} else if ((ix | lx) == 0) {
//		        if (hx < 0 && hy < 0)
//		                return __kernel_standard (x, y, 43); /* pow(+0.0,negative) ????? */
//			/* +-0**(1>y>0) = 0, +-0**(-1<y<0) = inf */
//			return (hy >= 0 ? ZERO : ONE / ZERO);
		}
		else if (ix == k)	/* +-inf**+-(1>y>0) = +inf,+0 */
			return (hy >= 0 ? x * x : ZERO);
		else if (hx < 0)
			/* (-x except 0,inf)**(non-integer) is NaN w/signal */
			return __kernel_standard (x, y, 24);
		else {
			/*
			 * The remaining case is a positive normal x ** tiny y.
			 * By actual testing, we find that x**y rounded to 1 as
			 * long as |y|<=2**-64; thus,
			 * x**y = exp(y*lnx) ~= 1 + y*lnx ~= 1 + sign(x-1)*y
			 */
			HI(y) = hy ^ (((ix - j) >> 31) << 31);
			return (ONE + y);
		}
	}
	s = C0 + r * s;
	t = C1 + r * t;
	oddn = ONE;
	yisint = 0;

	/* special cases: x is subnormal, 0, negative, +-inf, or nan */
	if (hx + 0x00100000 < 0x00200000) {
		i = iy >> 20;
		if (0x7ff00000 - (int) (ix | ((lx | -lx) >> 31)) < 0)
			return (x * x);	/* x is NaN; + -> * for Cheetah */
		if (hx < 0) {
			/*
			 * determine if y is an odd int when x < 0
			 * yisint = 0 ... y is not an integer
			 * yisint = 1 ... y is an odd int
			 * yisint = 2 ... y is an even int
			 */
			if (i >= 0x434)
				yisint = 2;	/* even integer y */
			else if ((k = i - 0x3ff) >= 0) {
				if (k > 20) {
					j = ly >> (52 - k);
					if ((j << (52 - k)) == ly)
						yisint = 2 - (j & 1);
				} else if (ly == 0) {
					j = iy >> (20 - k);
					if ((j << (20 - k)) == iy)
						yisint = 2 - (j & 1);
				}
			}
			HI(oddn) = 0x3ff00000 | (yisint << 31);
						/* oddn= -1.0 if yisint==1 */
			HI(x) = ix;
		}
		if ((ix | lx) == 0)		/* +-0**+-y = s*0 or s*inf */
			return (hy >= 0 ? oddn * ZERO :
				__kernel_standard (x, y, (yisint == 1) ? 23 : 43)); /* pow(+-0.0,negative) */
		else if (ix == 0x7ff00000)	/* +-inf**+-y = s*inf or s*0 */
			return (hy >= 0 ? x * oddn : ZERO * oddn);
		else if ((((hx >> 31) + 1) | yisint) == 0)
			return __kernel_standard (x, y, 24);	/* neg**non-int = NaN */
		else if (ix < 0x00100000) {	/* x subnormal */
			if (((hy - 0x3fe00000) | (hx >> 31) | ly) == 0)
				return (__builtin_sqrt(x));	/* y is 0.5, x>=0 */
			x *= TWO54;
			hx = HI(x);
			lx = LO(x);
			ix = hx & ~0x80000000;
			/* recompute the Quick algorithm */
			j = 0x3fe6a09e;
			n = (ix - j) >> 20;
			i = ix - (n << 20);
			/* X = x normalized to [0x3fe6a09e,0x3ff6a09d] */
			HI(X) = i;
			LO(X) = lx;
			ii = ((i + 0x800) >> 12);
			HI(Xi) = ii << 12;
			LO(Xi) = 0;	/* Xi = X rounded to 8 bits */
			v = X - Xi;
			ii = ii - (j >> 12);
			ii = ii + ii + ii;	/* index */
			v *= _TBL_log2_X[ii + 2];	/* v = (X-Xi)/Xi */
			r = v * v;
			s = (C0 + r * (C2 + r * C4));
			t = (C1 + r * (C3 + r * C5));
			n -= 54;
		}
	}
	s = v * s;
	t = r * t;
	if (((hy - 0x40000000) | ly) == 0) {
                w = x * x;
		if (fabs (w) == INFINITY)
		        return __kernel_standard (x, y, 21); /* pow overflow */
		else if (w == 0)
		        return __kernel_standard (x, y, 22); /* pow underflow */
		return w;		/* y is 2 */
	}
	w = s + t;
	if (((hy - 0x3fe00000) | (hx >> 31) | ly) == 0)
		return (__builtin_sqrt(x));	/* y is 0.5, x>=0 */

	nx = (double) n;
	if (n != 0) {
		nx += _TBL_log2_X[ii];
		wx = w + _TBL_log2_X[ii + 1];
		i = (n >> 31);
		z = nx + wx;
		if (((64 - ((n ^ i) - i)) | (iy - 0x40400000)) < 0) {
							/* |n|>64 or y<32 */
			if (iy > 0x40400000) {
				/* if y>32, then |n|>64; hence over/underflow */
				if ((n ^ hy) >= 0)
					return __kernel_standard (oddn * x, y, 21); /* pow overflow */
				else
					return __kernel_standard (oddn * x, y, 22); /* pow underflow */
			}
		} else {
			Y = v + ONE;	/* Y = X/Xi */
			jj = (HI(Y) + 0x20) >> 6;
			HI(Yj) = jj << 6;
			LO(Yj) = 0;		/* Yj = Y rounded to 14 bits */
			u = X - Xi * Yj;
			jj -= 0xffbfc0;		/* (0x3feff000)>>6) */
			jj = jj + jj + jj;	/* index */
			u *= _TBL_log2_Y[jj + 2] * _TBL_log2_X[ii + 2];
			nx += _TBL_log2_Y[jj];
			/*
			 * p = u*(G0+u*(G1+u*(G2+u*(G3+u*G4))));
			 */
			r = u * u;
			p = u * (G0 + r * (G2 + r * G4));
			q = r * (G1 + r * G3);
			p += q;
			wx = (_TBL_log2_X[ii + 1] + _TBL_log2_Y[jj + 1]);
			wx += p;
			if (iy > 0x40add400) {
				/* if |y|>2154, must over/underflow */
				if ((n ^ hy) >= 0)
					return __kernel_standard (oddn * x, y, 21); /* pow overflow */
				else
					return __kernel_standard (oddn * x, y, 22); /* pow underflow */
			}
			z = nx + wx;
		}
		z *= y;
		i = ((((unsigned) ly) >> 26) + 1) >> 1;
		z *= C256;
		HI(yH) = hy + (i >> 5);
		t = y * wx;
		LO(yH) = i << 27;
		yL = y - yH;
		zH = yH * nx;
		z += HALF[((unsigned) (n ^ hy)) >> 31];
		zL = yL * nx + t;
		m = (int) z;
		r = zL - ((double) m * INV256 - zH);
		k = m >> 8;
		p = D2 + r * (D3 + r * D4);
		LO(s) = 0;
		j = (m & 0xff) << 1;
		HI(s) = (0x3ff00000 + (k << 20)) | (yisint << 31);
		p = r * (D1 + r * p);
		q = _TBL_exp2_z[j];
		t = _TBL_exp2_z[j + 1] + q * p;
		w = t + q;
		if (((1023 - k) | (k + 1021)) < 0) {	/* k>1023 or k<-1021 */
			i = HI(w);
			j = (i >> 20) + k;
			if (j <= 0) {	/* subnormal or underflow */
				if (j <= -54) {
					return __kernel_standard (oddn * x, y, 22); /* pow underflow */
				} else {
					k += 100;
					HI(w) = i + (k << 20);
					s = TWOM100 * oddn;
				}
			} else if (j >= 0x7ff) {
				return __kernel_standard (oddn * x, y, 21); /* pow overflow */
			} else {
				HI(w) = i + (k << 20);
				s = oddn;
			}
		}
		return (s * w);
	} else {		/* if n is 0 */
		i = (ix + 0x2000) & 0xffffc000;
		if (i == 0x3ff00000) {	/* -2**-8 <= x-1 < 2**-7 */
			r = X - ONE;
			k = 0x3ff00000;
			i = ix - k;
			r2 = r * r;
			j = ((ix + 0x40) & 0xffffff80);
			if (j == k) {	/* -2**-15<=r<=2**-14 */
				if ((i | lx) == 0)	/* x = +-1.0 */
					return (x * oddn);
				wL = r2 * (HALFN + r * (T1 + r * (T2 + r * T3)));
				if (iy > 0x43f00000) {
					/* if |y|>2**64, must over/underflow */
					if (((ix - 0x3ff00000) ^ hy) >= 0)
				                return __kernel_standard (oddn * x, y, 21); /* pow overflow */
					else
					        return __kernel_standard (oddn * x, y, 22); /* pow underflow */
				}
				w = r + wL;
				i = ((((unsigned) LO(w)) >> 26) + 1) >> 1;
				HI(wH) = HI(w) + (i >> 5);
				LO(wH) = i << 27;
				wL += (r - wH);
				j = ((((unsigned) ly) >> 26) + 1) >> 1;
				HI(yH) = hy + (j >> 5);
				LO(yH) = j << 27;
				yL = y - yH;
			} else {	/* -2**-8<=x-1<2**-7 */
				p = P0 + r2 * (P2 + r2 * P4);
				q = r * (P1 + r2 * (P3 + r2 * P5));
				i = ((((unsigned) LO(r)) >> 26) + 1) >> 1;
				HI(rH) = HI(r) + (i >> 5);
				LO(rH) = i << 27;
				rL = r - rH;
				t = r * (p + q);
				w = r + r2 * (t - HALFP);
				if (iy >= 0x41800000) {
					/* if |y|>=2**26 overflow/underflow */
					if (((ix - 0x3ff00000) ^ hy) >= 0)
				                return __kernel_standard (oddn * x, y, 21); /* pow overflow */
					else
					        return __kernel_standard (oddn * x, y, 22); /* pow underflow */
				}
				i = ((((unsigned) LO(w)) >> 26) + 1) >> 1;
				HI(wH) = HI(w) + (i >> 5);
				LO(wH) = i << 27;
				wL = (((r - wH) - (HALFP * rH * rH)) + r2 * t) -
					rL * (rH + HALFP * rL);
				j = ((((unsigned) ly) >> 26) + 1) >> 1;
				HI(yH) = hy + (j >> 5);
				LO(yH) = j << 27;
				yL = y - yH;
			}
		} else {
			ii = (ix + 0x1000) >> 13;
			HI(Ai) = ii << 13;
			LO(Ai) = 0;	/* Ai = x rounded to 7 bits */
			ii -= 0x1ff35;
			ii = ii + ii + ii;
			B = x * _TBL_log_A[ii + 2];
			wH = _TBL_log_A[ii];
			wL = _TBL_log_A[ii + 1];
			if (iy > 0x41100000) {
				/* if |y|>2**18, must over/underflow */
				if (((ix - 0x3ff00000) ^ hy) >= 0)
				        return __kernel_standard (oddn * x, y, 21); /* pow overflow */
				else
					return __kernel_standard (oddn * x, y, 22); /* pow underflow */
			}
			jj = (HI(B) + 0x20) >> 6;	/* rounded to 14 bits */
			HI(Bj) = jj << 6;
			LO(Bj) = 0;
			jj -= 0xffbf80; /* 0x3fefe000>>6 */
			jj = jj + jj + jj;
			C = B * _TBL_log_B[jj + 2];
			wH += _TBL_log_B[jj];
			wL += _TBL_log_B[jj + 1];
			i = (((unsigned) LO(C)) >> 30) + 1;
			kk = HI(C) + (i >> 2);	/* add carry */
			k = (i & 2) << 30;
			HI(Ck) = kk;
			LO(Ck) = k;
			kk = ((kk - 0x3fefffc0) << 1) + (((unsigned) k) >> 31);
			kk = kk + kk + kk;
			v = (x - Ai * Bj * Ck) * _TBL_log_A[ii + 2] *
				_TBL_log_B[jj + 2] * _TBL_log_C[kk + 2];
			wH += _TBL_log_C[kk];
			wL += _TBL_log_C[kk + 1];
			wL += v * (ONE + v * (v * Q3 - HALFP));
			j = ((((unsigned) ly) >> 26) + 1) >> 1;
			HI(yH) = hy + (j >> 5);
			LO(yH) = j << 27;
			yL = y - yH;
		}
		zL = yL * wH + y * wL;
		zH = yH * wH;
		z = zH + zL;
		m = (int) (z * INVLN2_256 +
			HALF[((unsigned) ((ix - 0x3ff00000) ^ hy)) >> 31]);
		r = zL - m * LN2_256L;
		r += zH - m * LN2_256H;
		p = r * (ONE + r * (B1 + r * (B2 + r * B3)));
		k = m >> 8;
		j = (m & 0xff) << 1;
		HI(s) = (0x3ff00000 + (k << 20)) | (yisint << 31);
		LO(s) = 0;
		q = _TBL_exp2_z[j];
		t = _TBL_exp2_z[j + 1] + q * p;
		t += q;
		if (((1023 - k) | (k + 1021)) < 0) {	/* k>1023 or k<-1021 */
			i = HI(t);
			j = (i >> 20) + k;
			if (j <= 0) {	/* subnormal or underflow */
				if (j <= -54) {
					return __kernel_standard (oddn * x, y, 22); /* pow underflow */
				} else {
					k += 100;
					HI(t) += k << 20;
					s = oddn * TWOM100;
				}
			} else if (j >= 0x7ff) {
				return __kernel_standard (oddn * x, y, 21); /* pow overflow */
			} else {
				HI(t) += k << 20;
				s = oddn;
			}
		}
		return (s * t);
	}
}

strong_alias (__pow, __pow_finite)
weak_alias (__pow, pow)
weak_alias (__pow, __ieee754_pow)
